searchData={"items":[{"type":"task","title":"mix nbx","doc":"Prints Nebulex tasks and their information.\n\n    mix nbx","ref":"Mix.Tasks.Nbx.html"},{"type":"task","title":"mix nbx.gen.cache","doc":"Generates a new cache.\n\nThe cache will be placed in the `lib` directory.","ref":"Mix.Tasks.Nbx.Gen.Cache.html"},{"type":"task","title":"Examples - mix nbx.gen.cache","doc":"$ mix nbx.gen.cache -c MyApp.Cache","ref":"Mix.Tasks.Nbx.Gen.Cache.html#module-examples"},{"type":"task","title":"Command line options - mix nbx.gen.cache","doc":"* `-c`, `--cache` - The cache to generate.","ref":"Mix.Tasks.Nbx.Gen.Cache.html#module-command-line-options"},{"type":"module","title":"Nebulex","doc":"Nebulex is split into two main components:\n\n  * `Nebulex.Cache` - Defines a standard Cache API for caching data.\n    This API implementation is intended to create a way for different\n    technologies to provide a common caching interface. It defines the\n    mechanism for creating, accessing, updating, and removing information\n    from a cache. This common interface makes it easier for software\n    developers to leverage various technologies as caches since the\n    software they write using the Nebulex Cache API does not need\n    to be rewritten to work with different underlying technologies.\n\n  * `Nebulex.Caching` - Defines a Cache Abstraction for transparently adding\n    caching into an existing Elixir application. The caching abstraction\n    allows consistent use of various caching solutions with minimal impact\n    on the code. This Cache Abstraction enables declarative decorator-based\n    caching via **`Nebulex.Caching.Decorators`**. Decorators provide an\n    elegant way of annotating functions to be cached or evicted. Caching\n    decorators also enable the adoption or implementation of cache usage\n    patterns such as **Read-through**, **Write-through**, **Cache-as-SoR**,\n    etc. See the [Cache Usage Patterns][cache-patterns] guide.\n\n[cache-patterns]: http://hexdocs.pm/nebulex/3.0.0-rc.1/cache-usage-patterns.html\n\nThe following sections will provide an overview of those components and their\nusage. Feel free to access their respective module documentation for more\nspecific examples, options, and configurations.\n\nIf you want to check a sample application using Nebulex quickly, please check\nthe [getting started guide](http://hexdocs.pm/nebulex/3.0.0-rc.1/getting-started.html).","ref":"Nebulex.html"},{"type":"module","title":"Caches - Nebulex","doc":"`Nebulex.Cache` is the wrapper around the Cache. We can define a\ncache as follows:\n\n    defmodule MyApp.MyCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nWhere the configuration for the Cache must be in your application\nenvironment, usually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.MyCache,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_memory_check_interval: :timer.seconds(10)\n\nEach cache in Nebulex defines a `start_link/1` function that needs to be\ninvoked before using the cache. In general, this function is not called\ndirectly, but used as part of your application supervision tree.\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.Cache, []}\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end\n\nOtherwise, you can start and stop the cache directly at any time by calling\n`MyApp.Cache.start_link/1` and `MyApp.Cache.stop/1`.","ref":"Nebulex.html#module-caches"},{"type":"module","title":"Declarative decorator-based caching - Nebulex","doc":"See [Nebulex.Caching](http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Caching.html).","ref":"Nebulex.html#module-declarative-decorator-based-caching"},{"type":"function","title":"Nebulex.vsn/0","doc":"Returns the current Nebulex version.","ref":"Nebulex.html#vsn/0"},{"type":"behaviour","title":"Nebulex.Cache","doc":"Cache abstraction layer inspired by\n[Ecto](https://github.com/elixir-ecto/ecto).\n\nA cache maps to an underlying in-memory storage controlled by the adapter.\nFor example, Nebulex ships with an adapter that implements a local\ngenerational cache.\n\nThe cache expects the `:otp_app` and `:adapter` as options when used.\nThe `:otp_app` should point to an OTP application with the cache\nconfiguration. See the compile time options for more information:\n\n* `:otp_app` (`t:atom/0`) - Required. The OTP application the cache configuration is under.\n\n* `:adapter` (`t:module/0`) - Required. The cache adapter module.\n\n* `:default_dynamic_cache` (`t:atom/0`) - Default dynamic cache for executing cache commands. Set to the\n  defined cache module by default. For example, when you call\n  `MyApp.Cache.start_link/1`, it will start a cache with the name\n  `MyApp.Cache`.\n\n  See the [\"Dynamic caches\"](#module-dynamic-caches) section\n  for more information.\n\n* `:adapter_opts` (`t:keyword/0`) - Specifies a list of options passed to the adapter in compilation time. The default value is `[]`.\n\n\n\nFor example, the cache:\n\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nCould be configured with:\n\n    config :my_app, MyApp.Cache,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_memory_check_interval: :timer.seconds(10)\n\nMost of the configuration that goes into the `config` is specific\nto the adapter. For this particular example, you can check\n[`Nebulex.Adapters.Local`][local_adapter] for more information.\nDespite this, the following configuration values are shared\nacross all adapters:\n\n* `:name` (`t:atom/0` | `{:via, reg_mod :: module(), via_name :: any()}`) - The name of the supervisor process the cache is started under.\n  Set to the defined cache module by default. For example, when\n  you call `MyApp.Cache.start_link/1`, a cache named `MyApp.Cache`\n  is started.\n\n* `:telemetry` (`t:boolean/0`) - A flag to determine whether to emit the Telemetry cache command events. The default value is `true`.\n\n* `:telemetry_prefix` (list of `t:atom/0`) - Nebulex emits cache events using the [Telemetry](`:telemetry`) library.\n  By default, the telemetry prefix is based on the module name, so if your\n  module is called `MyApp.Cache`, the prefix will be `[:my_app, :cache]`.\n  See the [\"Telemetry events\"](#module-telemetry-events) section to see\n  which events are emitted by Nebulex out-of-box.\n\n  If you have multiple caches, keep the `:telemetry_prefix` consistent for\n  each cache and use the `:cache` property within the `:adapter_meta` coming\n  in the event metadata to distinguish between caches. For dynamic caches,\n  you should additionally use the `:name` property. Alternatively, you can\n  use different `:telemetry_prefix` values.\n\n* `:bypass_mode` (`t:boolean/0`) - If `true`, the cache calls are skipped by overwriting the configured\n  adapter with `Nebulex.Adapters.Nil` when the cache starts. This option\n  is handy for tests if you want to disable or bypass the cache while\n  running the tests. The default value is `false`.\n\n\n\n[local_adapter]: http://hexdocs.pm/nebulex_local/3.0.0-rc.1/Nebulex.Adapters.Local.html","ref":"Nebulex.Cache.html"},{"type":"behaviour","title":"Shared options - Nebulex.Cache","doc":"All of the cache functions outlined in this module accept the following\noptions:\n\n* `:timeout` (`t:timeout/0`) - The time in **milliseconds** to wait for a command to finish\n  (`:infinity` to wait indefinitely). The default value is `5000`.\n  However, it may change depending on the adapter.\n\n  > #### Timeout option {: .warning}\n  >\n  > Despite being a shared option accepted by almost all cache functions,\n  > it is up to the adapter to support it.\n\n* `:telemetry_event` (list of `t:atom/0`) - The telemetry event name to dispatch the event under. Defaults to what\n  is configured in the `:telemetry_prefix` option. See the\n  [\"Telemetry events\"](#module-telemetry-events) section\n  for more information.\n\n* `:telemetry_metadata` (map of `t:term/0` keys and `t:term/0` values) - Extra metadata to add to the Telemetry cache command events.\n  These end up in the `:extra_metadata` metadata key of these events.\n\n  See the [\"Telemetry events\"](#module-telemetry-events) section\n  for more information.\n\n\n\n> #### Adapter-specific options {: .info}\n>\n> In addition to the shared options, each adapter can define its\n> specific options. Therefore, Nebulex recommends reviewing the\n> adapter's documentation.","ref":"Nebulex.Cache.html#module-shared-options"},{"type":"behaviour","title":"Telemetry events - Nebulex.Cache","doc":"There are two types of telemetry events. The ones emitted by Nebulex and the\nones that are adapter specific. The ones emitted by Nebulex are divided into\ntwo categories: cache lifecycle events and cache command events. Let us take\na closer look at each of them.","ref":"Nebulex.Cache.html#module-telemetry-events"},{"type":"behaviour","title":"Cache lifecycle events - Nebulex.Cache","doc":"All Nebulex caches emit the following events:\n\n  * `[:nebulex, :cache, :init]` - It is dispatched whenever a cache starts.\n    The only measurement is the current system time in native units from\n    calling: `System.system_time()`. The `:opts` key in the metadata\n    contains all initialization options.\n\n    * Measurement: `%{system_time: integer()}`\n    * Metadata: `%{cache: module(), name: atom(), opts: keyword()}`","ref":"Nebulex.Cache.html#module-cache-lifecycle-events"},{"type":"behaviour","title":"Cache command events - Nebulex.Cache","doc":"When the option `:telemetry` is set to `true` (the default), Nebulex will\nemit Telemetry span events for each cache command. Those events will use\nthe `:telemetry_prefix` outlined in the options above which defaults to\n`[:my_app, :cache]`.\n\nFor instance, to receive all events published for the cache `MyApp.Cache`,\none could define a module:\n\n    defmodule MyApp.Telemetry do\n      def handle_event(\n            [:my_app, :cache, :command, event],\n            measurements,\n            metadata,\n            _config\n          ) do\n        case event do\n          :start ->\n            # Handle start event ...\n            IO.puts(\"Cache command started: #{inspect(metadata.command)}\")\n            IO.puts(\"Arguments: #{inspect(metadata.args)}\")\n\n          :stop ->\n            # Handle stop event ...\n            duration = measurements.duration\n            IO.puts(\"Cache command completed: #{inspect(metadata.command)}\")\n            IO.puts(\"Duration: #{duration} native units\")\n            IO.puts(\"Result: #{inspect(metadata.result)}\")\n\n          :exception ->\n            # Handle exception event ...\n            IO.puts(\"Cache command failed: #{inspect(metadata.command)}\")\n            IO.puts(\"Error: #{inspect(metadata.reason)}\")\n            IO.puts(\"Stacktrace: #{inspect(metadata.stacktrace)}\")\n        end\n      end\n    end\n\nThen, in the `Application.start/2` callback, attach the handler to the events\nusing a unique handler id:\n\n    :telemetry.attach_many(\n      \"my-app-handler-id\",\n      [\n        [:my_app, :cache, :command, :start],\n        [:my_app, :cache, :command, :stop],\n        [:my_app, :cache, :command, :exception]\n      ],\n      &MyApp.Telemetry.handle_event/4,\n      :no_config\n    )\n\nSee the [telemetry documentation](http://hexdocs.pm/nebulex/3.0.0-rc.1/telemetry.html)\nfor more information.\n\nThe following are the events you should expect from Nebulex. All examples\nbelow consider a cache named `MyApp.Cache`:\n\n#### `[:my_app, :cache, :command, :start]`\n\nThis event is emitted before a cache command is executed.\n\nThe `:measurements` map will include the following:\n\n  * `:system_time` - The current system time in native units from calling:\n    `System.system_time()`.\n\nA Telemetry `:metadata` map including the following fields:\n\n  * `:adapter_meta` - The adapter metadata.\n  * `:command` - The name of the invoked adapter's command.\n  * `:args` - The arguments passed to the invoked adapter, except for the\n    first one, since the adapter's metadata is available in the event's\n    metadata.\n  * `:extra_metadata` - Additional metadata through the runtime option\n    `:telemetry_metadata.`\n\nExample event data:\n\n    %{\n      measurements: %{system_time: 1_678_123_456_789},\n      metadata: %{\n        adapter_meta: %{...},\n        command: :put,\n        args: [\"key\", \"value\", [ttl: :timer.seconds(10)]],\n        extra_metadata: %{user_id: 123}\n      }\n    }\n\n#### `[:my_app, :cache, :command, :stop]`\n\nThis event is emitted after a cache command is executed.\n\nThe `:measurements` map will include the following:\n\n  * `:duration` - The time spent executing the cache command. The measurement\n    is given in the `:native` time unit. You can read more about it in the\n    docs for `System.convert_time_unit/3`.\n\nA Telemetry `:metadata` map including the following fields:\n\n  * `:adapter_meta` - The adapter metadata.\n  * `:command` - The name of the invoked adapter's command.\n  * `:args` - The arguments passed to the invoked adapter, except for the\n    first one, since the adapter's metadata is available in the event's\n    metadata.\n  * `:extra_metadata` - Additional metadata through the runtime option\n    `:telemetry_metadata.`\n  * `:result` - The command's result.\n\nExample event data:\n\n    %{\n      measurements: %{duration: 1_234_567},\n      metadata: %{\n        adapter_meta: %{...},\n        command: :put,\n        args: [\"key\", \"value\", [ttl: :timer.seconds(10)]],\n        extra_metadata: %{user_id: 123},\n        result: :ok\n      }\n    }\n\n#### `[:my_app, :cache, :command, :exception]`\n\nThis event is emitted when an error or exception occurs during the\ncache command execution.\n\nThe `:measurements` map will include the following:\n\n  * `:duration` - The time spent executing the cache command. The measurement\n    is given in the `:native` time unit. You can read more about it in the\n    docs for `System.convert_time_unit/3`.\n\nA Telemetry `:metadata` map including the following fields:\n\n  * `:adapter_meta` - The adapter metadata.\n  * `:command` - The name of the invoked adapter's command.\n  * `:args` - The arguments passed to the invoked adapter, except for the\n    first one, since the adapter's metadata is available in the event's\n    metadata.\n  * `:extra_metadata` - Additional metadata through the runtime option\n    `:telemetry_metadata.`\n  * `:kind` - The type of the error: `:error`, `:exit`, or `:throw`.\n  * `:reason` - The reason of the error.\n  * `:stacktrace` - Exception's stack trace.\n\nExample event data:\n\n    %{\n      measurements: %{duration: 1_234_567},\n      metadata: %{\n        adapter_meta: %{...},\n        command: :put,\n        args: [\"key\", \"value\", [ttl: :timer.seconds(10)]],\n        extra_metadata: %{user_id: 123},\n        kind: :error,\n        reason: %Nebulex.KeyError{key: \"key\", reason: :not_found},\n        stacktrace: [...]\n      }\n    }","ref":"Nebulex.Cache.html#module-cache-command-events"},{"type":"behaviour","title":"Adapter-specific events - Nebulex.Cache","doc":"Regardless of whether Nebulex emits the telemetry events outlined above or\nnot, the adapters can and are free to expose their own, but they will be\nout of Nebulex's scope. Therefore, if you are interested in using specific\nadapter events, you should review the adapters' documentation.","ref":"Nebulex.Cache.html#module-adapter-specific-events"},{"type":"behaviour","title":"Dynamic caches - Nebulex.Cache","doc":"Nebulex allows you to start multiple processes from the same cache module.\nThis feature is typically useful when you want to have different cache\ninstances but access them through the same cache module.\n\nWhen you list a cache in your supervision tree, such as `MyApp.Cache`, it will\nstart a supervision tree with a process named `MyApp.Cache` under the hood.\nBy default, the process has the same name as the cache module. Hence, whenever\nyou invoke a function in `MyApp.Cache`, such as `MyApp.Cache.put/3`, Nebulex\nwill execute the command in the cache process named `MyApp.Cache`.\n\nHowever, with Nebulex, you can start multiple processes from the same cache.\nThe only requirement is that they must have different process names, like\nthis:\n\n    children = [\n      MyApp.Cache,\n      {MyApp.Cache, name: MyApp.UsersCache}\n    ]\n\nNow you have two cache instances running: one is named `MyApp.Cache`, and the\nother one is named `MyApp.UsersCache`. You can tell Nebulex which process you\nwant to use in your cache operations by calling:\n\n    MyApp.Cache.put_dynamic_cache(MyApp.Cache)\n    MyApp.Cache.put_dynamic_cache(MyApp.UsersCache)\n\nOnce you call `MyApp.Cache.put_dynamic_cache(name)`, all invocations made on\n`MyApp.Cache` will use the cache instance denoted by `name`.\n\nNebulex also provides a handy function for invoking commands using dynamic\ncaches: `c:with_dynamic_cache/2`.\n\n    MyApp.Cache.with_dynamic_cache(MyApp.UsersCache, fn ->\n      # all commands here will use MyApp.UsersCache\n      MyApp.Cache.put(\"u1\", \"joe\")\n      ...\n    end)\n\nWhile these functions are handy, you may want to have the ability to pass\nthe dynamic cache directly to the command, avoiding the boilerplate logic\nof using `c:put_dynamic_cache/1` or `c:with_dynamic_cache/2`. From **v3.0**,\nall Cache API commands expose an extended callback version that admits a\ndynamic cache at the first argument, so you can directly interact with a\ncache instance.\n\n    MyApp.Cache.put(MyApp.UsersCache, \"u1\", \"joe\", ttl: :timer.hours(1))\n    MyApp.Cache.get(MyApp.UsersCache, \"u1\", nil, [])\n    MyApp.Cache.delete(MyApp.UsersCache, \"u1\", [])\n\nThis is another handy way to work with multiple cache instances through\nthe same cache module.","ref":"Nebulex.Cache.html#module-dynamic-caches"},{"type":"behaviour","title":"Distributed topologies - Nebulex.Cache","doc":"One of the goals of Nebulex is also to provide the ability to set up\ndistributed cache topologies, but this feature will depend on the adapters.\n\nSee [\"Nebulex Distributed\"](https://github.com/elixir-nebulex/nebulex_distributed).","ref":"Nebulex.Cache.html#module-distributed-topologies"},{"type":"callback","title":"Nebulex.Cache.__adapter__/0","doc":"Returns the adapter tied to the cache.","ref":"Nebulex.Cache.html#c:__adapter__/0"},{"type":"callback","title":"Nebulex.Cache.config/0","doc":"Returns the adapter configuration stored in the `:otp_app` environment.\n\nIf the `c:init/1` callback is implemented in the cache, it will be invoked.","ref":"Nebulex.Cache.html#c:config/0"},{"type":"callback","title":"Nebulex.Cache.count_all/2","doc":"Counts all entries matching the query specified by the given `query_spec`.\n\nSee `c:get_all/2` for more information about the `query_spec`.\n\nThis function returns:\n\n  * `{:ok, count}` - The cache executes the query successfully and\n    returns the `count` of the matched entries.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:count_all/2"},{"type":"callback","title":"Options - Nebulex.Cache.count_all/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:count_all/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.count_all/2","doc":"Populate the cache with some entries:\n\n    iex> Enum.each(1..5, &MyCache.put(&1, &1 * 2))\n    :ok\n\nCount all entries in cache (cache size):\n\n    iex> MyCache.count_all()\n    {:ok, 5}\n\nCount all entries that match with the given query, assuming we are using\n`Nebulex.Adapters.Local` adapter:\n\n    iex> query = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 5}], [true]}]\n    iex> {:ok, count} = MyCache.count_all(query: query)\n\nSee `c:get_all/2` for more query examples.","ref":"Nebulex.Cache.html#c:count_all/2-examples"},{"type":"callback","title":"Nebulex.Cache.count_all/3","doc":"Same as `c:count_all/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:count_all/3"},{"type":"callback","title":"Examples - Nebulex.Cache.count_all/3","doc":"iex> MyCache.count_all(MyCache1, [], [])\n    {:ok, 0}","ref":"Nebulex.Cache.html#c:count_all/3-examples"},{"type":"callback","title":"Nebulex.Cache.count_all!/2","doc":"Same as `c:count_all/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:count_all!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.count_all!/2","doc":"iex> MyCache.count_all!()\n    0","ref":"Nebulex.Cache.html#c:count_all!/2-examples"},{"type":"callback","title":"Nebulex.Cache.count_all!/3","doc":"Same as `c:count_all!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:count_all!/3"},{"type":"callback","title":"Nebulex.Cache.decr/3","doc":"Decrements the counter stored at `key` by the given `amount` and returns\nthe current count as `{:ok, count}`.\n\nIf `amount < 0`, the value is incremented by that `amount` instead\n(opposite to `incr/3`).\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nIf the key doesn't exist, the TTL is set. Otherwise, only the counter value\nis updated, keeping the TTL set for the first time.","ref":"Nebulex.Cache.html#c:decr/3"},{"type":"callback","title":"Options - Nebulex.Cache.decr/3","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n* `:default` (`t:integer/0`) - If the key is not present in the cache, the default value is inserted as\n  the key's initial value before it is incremented. The default value is `0`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:decr/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.decr/3","doc":"iex> MyCache.decr(:a)\n    {:ok, -1}\n    iex> MyCache.decr(:a, 2)\n    {:ok, -3}\n    iex> MyCache.decr(:a, -1)\n    {:ok, -2}\n\n    iex> MyCache.decr(:missing_key, 2, default: 10)\n    {:ok, 8}","ref":"Nebulex.Cache.html#c:decr/3-examples"},{"type":"callback","title":"Nebulex.Cache.decr/4","doc":"Same as `c:decr/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:decr/4"},{"type":"callback","title":"Examples - Nebulex.Cache.decr/4","doc":"iex> MyCache.decr(MyCache1, :a, 1, [])\n    {:ok, -1}","ref":"Nebulex.Cache.html#c:decr/4-examples"},{"type":"callback","title":"Nebulex.Cache.decr!/3","doc":"Same as `c:decr/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:decr!/3"},{"type":"callback","title":"Examples - Nebulex.Cache.decr!/3","doc":"iex> MyCache.decr!(:a)\n    -1","ref":"Nebulex.Cache.html#c:decr!/3-examples"},{"type":"callback","title":"Nebulex.Cache.decr!/4","doc":"Same as `c:decr!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:decr!/4"},{"type":"callback","title":"Nebulex.Cache.delete/2","doc":"Deletes the entry in the cache for a specific `key`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:delete/2"},{"type":"callback","title":"Options - Nebulex.Cache.delete/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:delete/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.delete/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.delete(:a)\n    :ok\n    iex> MyCache.get!(:a)\n    nil\n    iex> MyCache.delete(:nonexistent)\n    :ok","ref":"Nebulex.Cache.html#c:delete/2-examples"},{"type":"callback","title":"Nebulex.Cache.delete/3","doc":"Same as `c:delete/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:delete/3"},{"type":"callback","title":"Examples - Nebulex.Cache.delete/3","doc":"iex> MyCache.delete(MyCache1, :a, [])\n    :ok","ref":"Nebulex.Cache.html#c:delete/3-examples"},{"type":"callback","title":"Nebulex.Cache.delete!/2","doc":"Same as `c:delete/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:delete!/2"},{"type":"callback","title":"Nebulex.Cache.delete!/3","doc":"Same as `c:delete!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:delete!/3"},{"type":"callback","title":"Nebulex.Cache.delete_all/2","doc":"Deletes all entries matching the query specified by the given `query_spec`.\n\nSee `c:get_all/2` for more information about the `query_spec`.\n\nThis function returns:\n\n  * `{:ok, deleted_count}` - The cache executes the query successfully and\n    returns the deleted entries count.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:delete_all/2"},{"type":"callback","title":"Options - Nebulex.Cache.delete_all/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:delete_all/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.delete_all/2","doc":"Populate the cache with some entries:\n\n    iex> Enum.each(1..5, &MyCache.put(&1, &1 * 2))\n    :ok\n\nDelete all (default args):\n\n    iex> MyCache.delete_all()\n    {:ok, 5}\n\nDelete only the requested keys (bulk delete):\n\n    iex> MyCache.delete_all(in: [1, 2, 10])\n    {:ok, 2}\n\nDelete all entries that match with the given query, assuming we are using\n`Nebulex.Adapters.Local` adapter:\n\n    iex> query = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 5}], [true]}]\n    iex> {:ok, deleted_count} = MyCache.delete_all(query: query)\n\nSee `c:get_all/2` for more query examples.","ref":"Nebulex.Cache.html#c:delete_all/2-examples"},{"type":"callback","title":"Nebulex.Cache.delete_all/3","doc":"Same as `c:delete_all/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:delete_all/3"},{"type":"callback","title":"Examples - Nebulex.Cache.delete_all/3","doc":"iex> MyCache.delete_all(MyCache1, [], [])\n    {:ok, 0}","ref":"Nebulex.Cache.html#c:delete_all/3-examples"},{"type":"callback","title":"Nebulex.Cache.delete_all!/2","doc":"Same as `c:delete_all/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:delete_all!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.delete_all!/2","doc":"iex> MyCache.delete_all!()\n    0","ref":"Nebulex.Cache.html#c:delete_all!/2-examples"},{"type":"callback","title":"Nebulex.Cache.delete_all!/3","doc":"Same as `c:delete_all!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:delete_all!/3"},{"type":"type","title":"Nebulex.Cache.dynamic_cache/0","doc":"Dynamic cache value","ref":"Nebulex.Cache.html#t:dynamic_cache/0"},{"type":"type","title":"Nebulex.Cache.entries/0","doc":"Cache entries","ref":"Nebulex.Cache.html#t:entries/0"},{"type":"type","title":"Nebulex.Cache.error_tuple/0","doc":"Common error type","ref":"Nebulex.Cache.html#t:error_tuple/0"},{"type":"type","title":"Nebulex.Cache.error_tuple/1","doc":"Error type for the given reason","ref":"Nebulex.Cache.html#t:error_tuple/1"},{"type":"type","title":"Nebulex.Cache.event_filter/0","doc":"Proxy type to a cache event  filter","ref":"Nebulex.Cache.html#t:event_filter/0"},{"type":"type","title":"Nebulex.Cache.event_listener/0","doc":"Proxy type to a cache event listener","ref":"Nebulex.Cache.html#t:event_listener/0"},{"type":"callback","title":"Nebulex.Cache.expire/3","doc":"Returns `{:ok, true}` if the given `key` exists and the new `ttl` is\nsuccessfully updated; otherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned; where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:expire/3"},{"type":"callback","title":"Options - Nebulex.Cache.expire/3","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:expire/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.expire/3","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.expire(:a, :timer.hours(1))\n    {:ok, true}\n    iex> MyCache.expire(:a, :infinity)\n    {:ok, true}\n    iex> MyCache.expire(:b, 5)\n    {:ok, false}","ref":"Nebulex.Cache.html#c:expire/3-examples"},{"type":"callback","title":"Nebulex.Cache.expire/4","doc":"Same as `c:expire/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:expire/4"},{"type":"callback","title":"Examples - Nebulex.Cache.expire/4","doc":"iex> MyCache.expire(MyCache1, :a, :timer.hours(1), [])\n    {:ok, false}","ref":"Nebulex.Cache.html#c:expire/4-examples"},{"type":"callback","title":"Nebulex.Cache.expire!/3","doc":"Same as `c:expire/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:expire!/3"},{"type":"callback","title":"Examples - Nebulex.Cache.expire!/3","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.expire!(:a, :timer.hours(1))\n    true","ref":"Nebulex.Cache.html#c:expire!/3-examples"},{"type":"callback","title":"Nebulex.Cache.expire!/4","doc":"Same as `c:expire!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:expire!/4"},{"type":"callback","title":"Nebulex.Cache.fetch/2","doc":"Fetches the value for a specific `key` in the cache.\n\nIf the cache contains the given `key`, then its value is returned\nin the shape of `{:ok, value}`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned. `reason` is the cause of the error and can be\n`Nebulex.KeyError` if the cache does not contain `key`,\n`Nebulex.Error` otherwise.","ref":"Nebulex.Cache.html#c:fetch/2"},{"type":"callback","title":"Options - Nebulex.Cache.fetch/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:fetch/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.fetch/2","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n    iex> MyCache.fetch(\"foo\")\n    {:ok, \"bar\"}\n\n    # Key not found error\n    iex> {:error, %Nebulex.KeyError{key: \"bar\"} = e} = MyCache.fetch(\"bar\")\n    iex> e.reason\n    :not_found","ref":"Nebulex.Cache.html#c:fetch/2-examples"},{"type":"callback","title":"Nebulex.Cache.fetch/3","doc":"Same as `c:fetch/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:fetch/3"},{"type":"callback","title":"Examples - Nebulex.Cache.fetch/3","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n    iex> MyCache.fetch(MyCache1, \"foo\", [])\n    {:ok, \"bar\"}","ref":"Nebulex.Cache.html#c:fetch/3-examples"},{"type":"callback","title":"Nebulex.Cache.fetch!/2","doc":"Same as `c:fetch/2` but raises `Nebulex.KeyError` if the cache doesn't contain\n`key` or `Nebulex.Error` if another error occurs while executing the command.","ref":"Nebulex.Cache.html#c:fetch!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.fetch!/2","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n    iex>  MyCache.fetch!(\"foo\")\n    \"bar\"","ref":"Nebulex.Cache.html#c:fetch!/2-examples"},{"type":"callback","title":"Nebulex.Cache.fetch!/3","doc":"Same as `c:fetch/3` but raises `Nebulex.KeyError` if the cache doesn't contain\n`key` or `Nebulex.Error` if another error occurs while executing the command.","ref":"Nebulex.Cache.html#c:fetch!/3"},{"type":"type","title":"Nebulex.Cache.fetch_error_reason/0","doc":"Fetch error reason","ref":"Nebulex.Cache.html#t:fetch_error_reason/0"},{"type":"callback","title":"Nebulex.Cache.get/3","doc":"Gets a value from the cache where the key matches the given `key`.\n\nIf the cache contains the given `key` its value is returned as\n`{:ok, value}`.\n\nIf the cache does not contain `key`, `{:ok, default}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:get/3"},{"type":"callback","title":"Options - Nebulex.Cache.get/3","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:get/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.get/3","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n    iex> MyCache.get(\"foo\")\n    {:ok, \"bar\"}\n    iex> MyCache.get(:nonexistent)\n    {:ok, nil}\n    iex> MyCache.get(:nonexistent, :default)\n    {:ok, :default}","ref":"Nebulex.Cache.html#c:get/3-examples"},{"type":"callback","title":"Nebulex.Cache.get/4","doc":"Same as `c:get/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:get/4"},{"type":"callback","title":"Examples - Nebulex.Cache.get/4","doc":"iex> MyCache.get(MyCache1, \"key\", nil, [])\n    {:ok, nil}","ref":"Nebulex.Cache.html#c:get/4-examples"},{"type":"callback","title":"Nebulex.Cache.get!/3","doc":"Same as `c:get/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:get!/3"},{"type":"callback","title":"Nebulex.Cache.get!/4","doc":"Same as `c:get!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:get!/4"},{"type":"callback","title":"Nebulex.Cache.get_all/2","doc":"Fetches all entries from the cache matching the given query specified through\nthe [\"query-spec\"](#c:get_all/2-query-specification).\n\nThis function returns:\n\n  * `{:ok, result}` - The cache executes the query successfully. The `result`\n    is a list with the matched entries.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:get_all/2"},{"type":"callback","title":"Query specification - Nebulex.Cache.get_all/2","doc":"There are two ways to use the Query API:\n\n  * Fetch multiple keys (all at once), like a bulk fetch.\n  * Fetch all entries from the cache matching a given query, more like a\n    search (this is the most generic option).\n\nHere is where the `query_spec` argument comes in to specify the type of query\nto run.\n\nThe `query_spec` argument is a `t:keyword/0` with options defining the desired\nquery. The `query_spec` fields or options are:\n\n* `:in` (list of `t:term/0`) - The list of keys to fetch. The value to return depends on the `:select`\n  option. The `:in` option is a predefined query meant to fetch multiple\n  keys simultaneously.\n\n  If present, it overrides the `:query` option and instructs the underlying\n  adapter to match the entries associated with the set of keys requested.\n  For every key that does not hold a value or does not exist, it is ignored\n  and not added to the returned list.\n\n* `:query` (`t:term/0`) - The query specification to match entries in the cache.\n\n  If present and set to `nil`, it matches all entries in the cache. The\n  `nil` is a predefined value all adapters must support. Other than that,\n  the value depends entirely on the adapter. The adapter is responsible\n  for defining the query or matching specification. For example, the\n  `Nebulex.Adapters.Local` adapter supports the\n  [**\"ETS Match Spec\"**](https://www.erlang.org/doc/man/ets#match_spec).\n\n  The default value is `nil`.\n\n* `:select` - Selects which fields to choose from the entry.\n\n  The possible values are:\n\n    * `{:key, :value}` - (Default) Selects the key and the value from\n      the entry. They are returned as a tuple `{key, value}`.\n    * `:key` - Selects the key from the entry.\n    * `:value` - Selects the value from the entry.\n    * `:entry` - Selects the whole entry with its fields (use it carefully).\n      The adapter defines the entry, the structure, and its fields.\n      Therefore, Nebulex recommends checking the adapter's documentation to\n      understand the entry's structure with its fields and to verify if the\n      select option is supported.\n\n  The default value is `{:key, :value}`.","ref":"Nebulex.Cache.html#c:get_all/2-query-specification"},{"type":"callback","title":"Fetching multiple keys - Nebulex.Cache.get_all/2","doc":"While you can perform any query using the `:query` option (even fetching\nmultiple keys), the option `:in` is preferable. For example:\n\n    MyCache.get_all(in: [\"a\", \"list\", \"of\", \"keys\"])","ref":"Nebulex.Cache.html#c:get_all/2-fetching-multiple-keys"},{"type":"callback","title":"Fetching all entries matching a given query - Nebulex.Cache.get_all/2","doc":"As mentioned above, the option `:query` is the most generic way to match\nentries in a cache. This option allows users to write custom queries\nto be executed by the underlying adapter.\n\nFor matching all cached entries, you can skip the `:query` option or set it\nto `nil` instead (the default). For example:\n\n    MyCache.get_all() #=> Equivalent to MyCache.get_all(query: nil)\n\nUsing a custom query:\n\n    MyCache.get_all(query: query_supported_by_the_adapter)\n\n> _Nebulex recommends to see the adapter documentation when using this option._","ref":"Nebulex.Cache.html#c:get_all/2-fetching-all-entries-matching-a-given-query"},{"type":"callback","title":"Options - Nebulex.Cache.get_all/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:get_all/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.get_all/2","doc":"Populate the cache with some entries:\n\n    iex> MyCache.put_all(a: 1, b: 2, c: 3)\n    :ok\n\nFetch all entries in the cache:\n\n    iex> MyCache.get_all()\n    {:ok, [a: 1, b: 2, c: 3]}\n\nFetch all entries returning only the keys:\n\n    iex> MyCache.get_all(select: :key)\n    {:ok, [:a, :b, :c]}\n\nFetch all entries returning only the values:\n\n    iex> MyCache.get_all(select: :value)\n    {:ok, [1, 2, 3]}\n\nFetch only the requested keys (bulk fetch):\n\n    iex> MyCache.get_all(in: [:a, :b, :d])\n    {:ok, [a: 1, b: 2]}\n\nFetch the requested keys returning only the keys or values:\n\n    iex> MyCache.get_all(in: [:a, :b, :d], select: :key)\n    {:ok, [:a, :b]}\n    iex> MyCache.get_all(in: [:a, :b, :d], select: :value)\n    {:ok, [1, 2]}","ref":"Nebulex.Cache.html#c:get_all/2-examples"},{"type":"callback","title":"Query examples for `Nebulex.Adapters.Local` adapter - Nebulex.Cache.get_all/2","doc":"The `Nebulex.Adapters.Local` adapter supports **\"ETS Match Spec\"** as query\nvalues (in addition to `nil` or the option `:in`).\n\nYou must know the adapter's entry structure for match-spec queries, which is\n`{:entry, key, value, touched, ttl}`. For example, one may write the following\nquery:\n\n    iex> match_spec = [\n    ...>   {\n    ...>     {:entry, :\"$1\", :\"$2\", :_, :_},\n    ...>     [{:>, :\"$2\", 1}],\n    ...>     [{{:\"$1\", :\"$2\"}}]\n    ...>   }\n    ...> ]\n    iex> MyCache.get_all(query: match_spec)\n    {:ok, [b: 1, c: 3]}","ref":"Nebulex.Cache.html#c:get_all/2-query-examples-for-nebulex-adapters-local-adapter"},{"type":"callback","title":"Nebulex.Cache.get_all/3","doc":"Same as `c:get_all/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:get_all/3"},{"type":"callback","title":"Examples - Nebulex.Cache.get_all/3","doc":"iex> MyCache.get_all(MyCache1, [], [])\n    {:ok, _matched_entries}","ref":"Nebulex.Cache.html#c:get_all/3-examples"},{"type":"callback","title":"Nebulex.Cache.get_all!/2","doc":"Same as `c:get_all/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:get_all!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.get_all!/2","doc":"iex> MyCache.put_all(a: 1, b: 2, c: 3)\n    :ok\n    iex> MyCache.get_all!()\n    [a: 1, b: 2, c: 3]\n    iex> MyCache.get_all!(in: [:a, :b])\n    [a: 1, b: 2]","ref":"Nebulex.Cache.html#c:get_all!/2-examples"},{"type":"callback","title":"Nebulex.Cache.get_all!/3","doc":"Same as `c:get_all!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:get_all!/3"},{"type":"callback","title":"Nebulex.Cache.get_and_update/3","doc":"Gets the value from `key` and updates it, all in one pass.\n\n`fun` is called with the current cached value under `key` (or `nil` if `key`\nhasn't been cached) and must return a two-element tuple: the current value\n(the retrieved value, which can be operated on before being returned) and\nthe new value to be stored under `key`. `fun` may also return `:pop`, which\nmeans the current value shall be removed from the cache and returned.\n\nThis function returns:\n\n  * `{:ok, {current_value, new_value}}` - The `current_value` is the current\n    cached value and `new_value` the updated one returned by `fun`.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:get_and_update/3"},{"type":"callback","title":"Options - Nebulex.Cache.get_and_update/3","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n* `:keep_ttl` (`t:boolean/0`) - Indicates whether to retain the time to live associated with the key.\n  Otherwise, the value in the `:ttl` option overwrites the existing one. The default value is `false`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:get_and_update/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.get_and_update/3","doc":"Update nonexistent key:\n\n    iex> MyCache.get_and_update(:a, fn current_value ->\n    ...>   {current_value, \"value!\"}\n    ...> end)\n    {:ok, {nil, \"value!\"}}\n\nUpdate existing key:\n\n    iex> MyCache.get_and_update(:a, fn current_value ->\n    ...>   {current_value, \"new value!\"}\n    ...> end)\n    {:ok, {\"value!\", \"new value!\"}}\n\nPop/remove value if exist:\n\n    iex> MyCache.get_and_update(:a, fn _ -> :pop end)\n    {:ok, {\"new value!\", nil}}\n\nPop/remove nonexistent key:\n\n    iex> MyCache.get_and_update(:b, fn _ -> :pop end)\n    {:ok, {nil, nil}}","ref":"Nebulex.Cache.html#c:get_and_update/3-examples"},{"type":"callback","title":"Nebulex.Cache.get_and_update/4","doc":"Same as `c:get_and_update/3`, but the command is executed on the cache\ninstance given at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:get_and_update/4"},{"type":"callback","title":"Examples - Nebulex.Cache.get_and_update/4","doc":"iex> MyCache.get_and_update(MyCache1, :a, &{&1, \"value!\"}, [])\n    {:ok, {nil, \"value!\"}}","ref":"Nebulex.Cache.html#c:get_and_update/4-examples"},{"type":"callback","title":"Nebulex.Cache.get_and_update!/3","doc":"Same as `c:get_and_update/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:get_and_update!/3"},{"type":"callback","title":"Examples - Nebulex.Cache.get_and_update!/3","doc":"iex> MyCache.get_and_update!(:a, &{&1, \"value!\"})\n    {nil, \"value!\"}","ref":"Nebulex.Cache.html#c:get_and_update!/3-examples"},{"type":"callback","title":"Nebulex.Cache.get_and_update!/4","doc":"Same as `c:get_and_update!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:get_and_update!/4"},{"type":"callback","title":"Nebulex.Cache.get_dynamic_cache/0","doc":"Returns the atom name or pid of the current cache\n(based on Ecto dynamic repo).\n\nSee also `c:put_dynamic_cache/1`.","ref":"Nebulex.Cache.html#c:get_dynamic_cache/0"},{"type":"callback","title":"Nebulex.Cache.has_key?/2","doc":"Determines if the cache contains an entry for the specified `key`.\n\nMore formally, it returns `{:ok, true}` if the cache contains the given `key`.\nIf the cache doesn't contain `key`, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:has_key?/2"},{"type":"callback","title":"Options - Nebulex.Cache.has_key?/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:has_key?/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.has_key?/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.has_key?(:a)\n    {:ok, true}\n    iex> MyCache.has_key?(:b)\n    {:ok, false}","ref":"Nebulex.Cache.html#c:has_key?/2-examples"},{"type":"callback","title":"Nebulex.Cache.has_key?/3","doc":"Same as `c:has_key?/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:has_key?/3"},{"type":"callback","title":"Examples - Nebulex.Cache.has_key?/3","doc":"iex> MyCache.has_key?(MyCache1, :a, [])\n    {:ok, false}","ref":"Nebulex.Cache.html#c:has_key?/3-examples"},{"type":"callback","title":"Nebulex.Cache.in_transaction?/1","doc":"Returns `{:ok, true}` if the current process is inside a transaction;\notherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:in_transaction?/1"},{"type":"callback","title":"Options - Nebulex.Cache.in_transaction?/1","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:in_transaction?/1-options"},{"type":"callback","title":"Examples - Nebulex.Cache.in_transaction?/1","doc":"MyCache.in_transaction?()\n    #=> {:ok, false}\n\n    MyCache.transaction(fn ->\n      MyCache.in_transaction? #=> {:ok, true}\n    end)","ref":"Nebulex.Cache.html#c:in_transaction?/1-examples"},{"type":"callback","title":"Nebulex.Cache.in_transaction?/2","doc":"Same as `c:in_transaction?/1`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:in_transaction?/2"},{"type":"callback","title":"Examples - Nebulex.Cache.in_transaction?/2","doc":"MyCache.in_transaction?(MyCache1, [])","ref":"Nebulex.Cache.html#c:in_transaction?/2-examples"},{"type":"callback","title":"Nebulex.Cache.incr/3","doc":"Increments the counter stored at `key` by the given `amount` and returns\nthe current count as `{:ok, count}`.\n\nIf `amount < 0`, the value is decremented by that `amount` instead.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nIf the key doesn't exist, the TTL is set. Otherwise, only the counter value\nis updated, keeping the TTL set for the first time.","ref":"Nebulex.Cache.html#c:incr/3"},{"type":"callback","title":"Options - Nebulex.Cache.incr/3","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n* `:default` (`t:integer/0`) - If the key is not present in the cache, the default value is inserted as\n  the key's initial value before it is incremented. The default value is `0`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:incr/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.incr/3","doc":"iex> MyCache.incr(:a)\n    {:ok, 1}\n    iex> MyCache.incr(:a, 2)\n    {:ok, 3}\n    iex> MyCache.incr(:a, -1)\n    {:ok, 2}\n    iex> MyCache.incr(:missing_key, 2, default: 10)\n    {:ok, 12}\n\n    # Initialize the counter with a TTL\n    iex> MyCache.incr(:new_counter, 10, ttl: :timer.seconds(1))\n    {:ok, 10}","ref":"Nebulex.Cache.html#c:incr/3-examples"},{"type":"callback","title":"Nebulex.Cache.incr/4","doc":"Same as `c:incr/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:incr/4"},{"type":"callback","title":"Examples - Nebulex.Cache.incr/4","doc":"iex> MyCache.incr(MyCache1, :a, 1, [])\n    {:ok, 1}","ref":"Nebulex.Cache.html#c:incr/4-examples"},{"type":"callback","title":"Nebulex.Cache.incr!/3","doc":"Same as `c:incr/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:incr!/3"},{"type":"callback","title":"Examples - Nebulex.Cache.incr!/3","doc":"iex> MyCache.incr!(:a)\n    1\n    iex> MyCache.incr!(:a, 2)\n    3","ref":"Nebulex.Cache.html#c:incr!/3-examples"},{"type":"callback","title":"Nebulex.Cache.incr!/4","doc":"Same as `c:incr!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:incr!/4"},{"type":"callback","title":"Nebulex.Cache.info/2","doc":"Returns `{:ok, info}` where `info` contains the requested cache information,\nas specified by the `spec`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nThe `spec` (information specification key) can be:\n\n  * **The atom `:all`** - returns a map with all information items.\n  * **An atom** - returns the value for the requested information item.\n  * **A list of atoms** - returns a map only with the requested information\n    items.\n\nIf the argument `spec` is omitted, all information items are returned;\nsame as if the `spec` was the atom `:all`.\n\nThe adapters are free to add the information specification keys they want.\nHowever, Nebulex suggests the adapters add the following keys:\n\n  * `:server` - General information about the cache server (e.g., cache name,\n    adapter, PID, etc.).\n  * `:memory` - Memory consumption information (e.g., used memory,\n    allocated memory, etc.).\n  * `:stats` - Cache statistics (e.g., hits, misses, etc.).","ref":"Nebulex.Cache.html#c:info/2"},{"type":"callback","title":"Examples - Nebulex.Cache.info/2","doc":"The following examples assume the underlying adapter uses the implementation\nprovided by `Nebulex.Adapters.Common.Info`.\n\n    iex> {:ok, info} = MyCache.info()\n    iex> info\n    %{\n      server: %{\n        nbx_version: \"3.0.0\",\n        cache_module: \"MyCache\",\n        cache_adapter: \"Nebulex.Adapters.Local\",\n        cache_name: \"MyCache\",\n        cache_pid: #PID<0.111.0>\n      },\n      memory: %{\n        total: 1_000_000,\n        used: 0\n      },\n      stats: %{\n        deletions: 0,\n        evictions: 0,\n        expirations: 0,\n        hits: 0,\n        misses: 0,\n        updates: 0,\n        writes: 0\n      }\n    }\n\n    iex> {:ok, info} = MyCache.info(:server)\n    iex> info\n    %{\n      nbx_version: \"3.0.0\",\n      cache_module: \"MyCache\",\n      cache_adapter: \"Nebulex.Adapters.Local\",\n      cache_name: \"MyCache\",\n      cache_pid: #PID<0.111.0>\n    }\n\n    iex> {:ok, info} = MyCache.info([:server, :stats])\n    iex> info\n    %{\n      server: %{\n        nbx_version: \"3.0.0\",\n        cache_module: \"MyCache\",\n        cache_adapter: \"Nebulex.Adapters.Local\",\n        cache_name: \"MyCache\",\n        cache_pid: #PID<0.111.0>\n      },\n      stats: %{\n        deletions: 0,\n        evictions: 0,\n        expirations: 0,\n        hits: 0,\n        misses: 0,\n        updates: 0,\n        writes: 0\n      }\n    }","ref":"Nebulex.Cache.html#c:info/2-examples"},{"type":"callback","title":"Nebulex.Cache.info/3","doc":"Same as `c:info/2`, but the command is executed on the cache\ninstance given at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:info/3"},{"type":"callback","title":"Examples - Nebulex.Cache.info/3","doc":"MyCache.info(MyCache1, :all, [])","ref":"Nebulex.Cache.html#c:info/3-examples"},{"type":"callback","title":"Nebulex.Cache.info!/2","doc":"Same as `c:info/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:info!/2"},{"type":"callback","title":"Nebulex.Cache.info!/3","doc":"Same as `c:info/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:info!/3"},{"type":"type","title":"Nebulex.Cache.info_data/0","doc":"The data type for the cache information","ref":"Nebulex.Cache.html#t:info_data/0"},{"type":"type","title":"Nebulex.Cache.info_item/0","doc":"The type for the info item's value","ref":"Nebulex.Cache.html#t:info_item/0"},{"type":"type","title":"Nebulex.Cache.info_map/0","doc":"Info map","ref":"Nebulex.Cache.html#t:info_map/0"},{"type":"type","title":"Nebulex.Cache.info_spec/0","doc":"Specification key for the item(s) to include in the returned info","ref":"Nebulex.Cache.html#t:info_spec/0"},{"type":"callback","title":"Nebulex.Cache.init/1","doc":"A callback executed when the cache starts or when configuration is read.","ref":"Nebulex.Cache.html#c:init/1"},{"type":"type","title":"Nebulex.Cache.key/0","doc":"Cache entry key","ref":"Nebulex.Cache.html#t:key/0"},{"type":"type","title":"Nebulex.Cache.nbx_error_reason/0","doc":"Proxy type for generic Nebulex error","ref":"Nebulex.Cache.html#t:nbx_error_reason/0"},{"type":"type","title":"Nebulex.Cache.ok_error_tuple/1","doc":"Ok/Error tuple with default error reasons","ref":"Nebulex.Cache.html#t:ok_error_tuple/1"},{"type":"type","title":"Nebulex.Cache.ok_error_tuple/2","doc":"Ok/Error type","ref":"Nebulex.Cache.html#t:ok_error_tuple/2"},{"type":"type","title":"Nebulex.Cache.opts/0","doc":"Cache action options","ref":"Nebulex.Cache.html#t:opts/0"},{"type":"callback","title":"Nebulex.Cache.put/3","doc":"Puts the given `value` under `key` into the cache.\n\nIf `key` already holds an entry, it is overwritten. Any previous TTL\n(time to live) associated with the key is discarded on a successful\n`put` operation.\n\nReturns `:ok` if successful; `{:error, reason}` otherwise.","ref":"Nebulex.Cache.html#c:put/3"},{"type":"callback","title":"Options - Nebulex.Cache.put/3","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n* `:keep_ttl` (`t:boolean/0`) - Indicates whether to retain the time to live associated with the key.\n  Otherwise, the value in the `:ttl` option overwrites the existing one. The default value is `false`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:put/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.put/3","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n\nPutting entries with specific time-to-live:\n\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.seconds(10))\n    :ok\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.hours(1))\n    :ok\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.minutes(1))\n    :ok\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.seconds(30))\n    :ok\n\nTo keep the current TTL in case the key already exists:\n\n    iex> MyCache.put(\"foo\", \"bar\", keep_ttl: true)\n    :ok","ref":"Nebulex.Cache.html#c:put/3-examples"},{"type":"callback","title":"Nebulex.Cache.put/4","doc":"Same as `c:put/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:put/4"},{"type":"callback","title":"Examples - Nebulex.Cache.put/4","doc":"iex> MyCache.put(MyCache1, \"foo\", \"bar\", [])\n    :ok\n    iex> MyCache.put(MyCache2, \"foo\", \"bar\", ttl: :timer.hours(1))\n    :ok","ref":"Nebulex.Cache.html#c:put/4-examples"},{"type":"callback","title":"Nebulex.Cache.put!/3","doc":"Same as `c:put/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put!/3"},{"type":"callback","title":"Nebulex.Cache.put!/4","doc":"Same as `c:put!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put!/4"},{"type":"callback","title":"Nebulex.Cache.put_all/2","doc":"Puts the given `entries` (key/value pairs) into the cache. It replaces\nexisting values with new values (just as regular `put`).\n\nReturns `:ok` if successful; `{:error, reason}` otherwise.","ref":"Nebulex.Cache.html#c:put_all/2"},{"type":"callback","title":"Options - Nebulex.Cache.put_all/2","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:put_all/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.put_all/2","doc":"iex> MyCache.put_all(apples: 3, bananas: 1)\n    :ok\n    iex> MyCache.put_all(%{apples: 2, oranges: 1}, ttl: :timer.hours(1))\n    :ok\n\n> #### Atomic operation {: .warning}\n>\n> Ideally, this operation should be atomic, so all given keys are put at once.\n> But it depends purely on the adapter's implementation and the backend used\n> internally by the adapter. Hence, reviewing the adapter's documentation is\n> highly recommended.","ref":"Nebulex.Cache.html#c:put_all/2-examples"},{"type":"callback","title":"Nebulex.Cache.put_all/3","doc":"Same as `c:put_all/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:put_all/3"},{"type":"callback","title":"Examples - Nebulex.Cache.put_all/3","doc":"iex> MyCache.put_all(MyCache1, [apples: 3, bananas: 1], [])\n    :ok\n    iex> MyCache.put_all(MyCache1, %{oranges: 1}, ttl: :timer.hours(1))\n    :ok","ref":"Nebulex.Cache.html#c:put_all/3-examples"},{"type":"callback","title":"Nebulex.Cache.put_all!/2","doc":"Same as `c:put_all/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put_all!/2"},{"type":"callback","title":"Nebulex.Cache.put_all!/3","doc":"Same as `c:put_all!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put_all!/3"},{"type":"callback","title":"Nebulex.Cache.put_dynamic_cache/1","doc":"Sets the dynamic cache to be used in further commands\n(based on Ecto dynamic repo).\n\nThere are cases where you may want to have different cache instances but\naccess them through the same cache module. By default, when you call\n`MyApp.Cache.start_link/1`, it will start a cache with the name\n`MyApp.Cache`. But it is also possible to start multiple caches by using\na different name for each of them:\n\n    MyApp.Cache.start_link(name: :cache1)\n    MyApp.Cache.start_link(name: :cache2)\n\nYou can also start caches without names by explicitly setting the name\nto `nil`:\n\n    MyApp.Cache.start_link(name: nil)\n\n> **NOTE:** There may be adapters requiring the `:name` option anyway,\n  therefore, it is highly recommended to see the adapter's documentation\n  you want to use.\n\nAll operations through `MyApp.Cache` are sent by default to the cache named\n`MyApp.Cache`. But you can change the default cache at compile-time:\n\n    use Nebulex.Cache, default_dynamic_cache: :cache_name\n\nOr anytime at runtime by calling `put_dynamic_cache/1`:\n\n    MyApp.Cache.put_dynamic_cache(:another_cache_name)\n\nFrom this moment on, all future commands performed by the current process\nwill run on `:another_cache_name`.\n\nAdditionally, all cache commands optionally support passing the wanted\ndynamic cache (name or PID) as the first argument so you can o directly\ninteract with a cache instance. See the\n[\"Dynamic caches\"](#module-dynamic-caches) section in the module\ndocumentation for more information.","ref":"Nebulex.Cache.html#c:put_dynamic_cache/1"},{"type":"callback","title":"Nebulex.Cache.put_new/3","doc":"Puts the given `value` under `key` into the cache only if it does not\nalready exist.\n\nReturns `{:ok, true}` if the value is stored; otherwise, `{:ok, false}`\nis returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:put_new/3"},{"type":"callback","title":"Options - Nebulex.Cache.put_new/3","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:put_new/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.put_new/3","doc":"iex> MyCache.put_new(\"foo\", \"bar\")\n    {:ok, true}\n    iex> MyCache.put_new(\"foo\", \"bar\", ttl: :timer.hours(1))\n    {:ok, false}","ref":"Nebulex.Cache.html#c:put_new/3-examples"},{"type":"callback","title":"Nebulex.Cache.put_new/4","doc":"Same as `c:put_new/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:put_new/4"},{"type":"callback","title":"Examples - Nebulex.Cache.put_new/4","doc":"iex> MyCache.put_new(MyCache1, \"foo\", \"bar\", [])\n    {:ok, true}\n    iex> MyCache.put_new(MyCache1, \"foo\", \"bar\", ttl: :timer.hours(1))\n    {:ok, false}","ref":"Nebulex.Cache.html#c:put_new/4-examples"},{"type":"callback","title":"Nebulex.Cache.put_new!/3","doc":"Same as `c:put_new/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put_new!/3"},{"type":"callback","title":"Examples - Nebulex.Cache.put_new!/3","doc":"iex> MyCache.put_new!(\"foo\", \"bar\")\n    true\n    iex> MyCache.put_new!(\"foo\", \"bar\", ttl: :timer.hours(1))\n    false","ref":"Nebulex.Cache.html#c:put_new!/3-examples"},{"type":"callback","title":"Nebulex.Cache.put_new!/4","doc":"Same as `c:put_new!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put_new!/4"},{"type":"callback","title":"Nebulex.Cache.put_new_all/2","doc":"Puts the given `entries` (key/value pairs) into the `cache`. It will not\nperform any operation, even if a single key exists.\n\nReturns `{:ok, true}` if all entries are successfully stored, or\n`{:ok, false}` if no key was set (at least one key already existed).\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:put_new_all/2"},{"type":"callback","title":"Options - Nebulex.Cache.put_new_all/2","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:put_new_all/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.put_new_all/2","doc":"iex> MyCache.put_new_all(apples: 3, bananas: 1)\n    {:ok, true}\n    iex> MyCache.put_new_all(%{apples: 3, oranges: 1}, ttl: :timer.hours(1))\n    {:ok, false}\n\n> #### Atomic operation {: .warning}\n>\n> Ideally, this operation should be atomic, so all given keys are put at once.\n> But it depends purely on the adapter's implementation and the backend used\n> internally by the adapter. Hence, reviewing the adapter's documentation is\n> highly recommended.","ref":"Nebulex.Cache.html#c:put_new_all/2-examples"},{"type":"callback","title":"Nebulex.Cache.put_new_all/3","doc":"Same as `c:put_new_all/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:put_new_all/3"},{"type":"callback","title":"Examples - Nebulex.Cache.put_new_all/3","doc":"iex> MyCache.put_new_all(MyCache1, [apples: 3, bananas: 1], [])\n    {:ok, true}\n    iex> MyCache.put_new_all(MyCache1, %{apples: 3, oranges: 1}, ttl: :timer.seconds(10))\n    {:ok, false}","ref":"Nebulex.Cache.html#c:put_new_all/3-examples"},{"type":"callback","title":"Nebulex.Cache.put_new_all!/2","doc":"Same as `c:put_new_all/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put_new_all!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.put_new_all!/2","doc":"iex> MyCache.put_new_all!(apples: 3, bananas: 1)\n    true\n    iex> MyCache.put_new_all!(%{apples: 3, oranges: 1}, ttl: :timer.hours(1))\n    false","ref":"Nebulex.Cache.html#c:put_new_all!/2-examples"},{"type":"callback","title":"Nebulex.Cache.put_new_all!/3","doc":"Same as `c:put_new_all!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:put_new_all!/3"},{"type":"type","title":"Nebulex.Cache.query_spec/0","doc":"The data type for a query spec","ref":"Nebulex.Cache.html#t:query_spec/0"},{"type":"callback","title":"Nebulex.Cache.register_event_listener/2","doc":"Register a cache event listener `event_listener`.\n\nReturns `:ok` if successful; `{:error, reason}` otherwise.\n\nListeners should be implemented with care. In particular, it is important\nto consider their impact on performance and latency.\n\nListeners:\n\n  * are fired after the entry is mutated in the cache.\n  * block the calling process until the listener returns,\n    when the listener is invoked synchronously; depends on\n    the adapter's implementation.\n\nListeners follow the observer pattern. An exception raised by a listener does\nnot cause the cache operation to fail.\n\nListeners can only raise `Nebulex.Error` exception. Caching implementations\nmust catch any other exception from a listener, then wrap and reraise it as\na `Nebulex.Error` exception.","ref":"Nebulex.Cache.html#c:register_event_listener/2"},{"type":"callback","title":"Options - Nebulex.Cache.register_event_listener/2","doc":"* `:id` (`t:term/0`) - A unique identifier for the event listener. An error will be returned if\n  another listener with the same ID already exists. Defaults to the event\n  listener function itself.\n\n* `:filter` (`t:Nebulex.Event.filter/0`) - A function that may be used to check cache entry events prior to being\n  dispatched to event listeners.\n\n  A filter must not create side effects.\n\n* `:metadata` (`t:Nebulex.Event.metadata/0`) - The metadata is provided when registering the listener and added to the\n  event at invoking the listener and filter functions; the event must always\n  have a metadata field. The default value is `[]`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:register_event_listener/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.register_event_listener/2","doc":"iex> MyApp.Cache.register_event_listener(&MyApp.handle/1)\n    :ok\n    iex> MyApp.Cache.register_event_listener(&MyApp.handle/1,\n    ...>   filter: &MyApp.filter/1\n    ...> )\n    :ok\n    iex> MyApp.Cache.register_event_listener(&MyApp.handle/2,\n    ...>   filter: &MyApp.filter/2\n    ...>   metadata: [foo: :bar]\n    ...> )\n    :ok\n\n    # Register with `:id` (must be unregistered using the same `:id` value)\n    iex> MyApp.Cache.register_event_listener(&MyApp.handle/2,\n    ...>   id: :my_listener,\n    ...>   filter: &MyApp.filter/2\n    ...>   metadata: [foo: :bar]\n    ...> )\n    :ok","ref":"Nebulex.Cache.html#c:register_event_listener/2-examples"},{"type":"callback","title":"Nebulex.Cache.register_event_listener/3","doc":"Same as `c:register_event_listener/2`, but the command is executed on the cache\ninstance given at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:register_event_listener/3"},{"type":"callback","title":"Examples - Nebulex.Cache.register_event_listener/3","doc":"MyApp.Cache.register_event_listener(:my_cache, &MyApp.handle/1, [])","ref":"Nebulex.Cache.html#c:register_event_listener/3-examples"},{"type":"callback","title":"Nebulex.Cache.register_event_listener!/2","doc":"Same as `c:register_event_listener/2` but raises an exception if an error\noccurs.","ref":"Nebulex.Cache.html#c:register_event_listener!/2"},{"type":"callback","title":"Nebulex.Cache.register_event_listener!/3","doc":"Same as `c:register_event_listener/3` but raises an exception if an error\noccurs.","ref":"Nebulex.Cache.html#c:register_event_listener!/3"},{"type":"callback","title":"Nebulex.Cache.replace/3","doc":"Alters the entry stored under `key`, but only if the entry already exists\nin the cache.\n\nReturns `{:ok, true}` if the value is replaced. Otherwise, `{:ok, false}`\nis returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:replace/3"},{"type":"callback","title":"Options - Nebulex.Cache.replace/3","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n* `:keep_ttl` (`t:boolean/0`) - Indicates whether to retain the time to live associated with the key.\n  Otherwise, the value in the `:ttl` option overwrites the existing one. The default value is `true`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:replace/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.replace/3","doc":"iex> MyCache.replace(\"foo\", \"bar\")\n    {:ok, false}\n    iex> MyCache.put_new(\"foo\", \"bar\")\n    {:ok, true}\n    iex> MyCache.replace(\"foo\", \"bar2\")\n    {:ok, true}\n\nUpdate current value and TTL:\n\n    iex> MyCache.replace(\"foo\", \"bar3\", ttl: :timer.seconds(10))\n    {:ok, true}\n\nTo keep the current TTL:\n\n    iex> MyCache.replace(\"foo\", \"bar4\", keep_ttl: true)\n    {:ok, true}","ref":"Nebulex.Cache.html#c:replace/3-examples"},{"type":"callback","title":"Nebulex.Cache.replace/4","doc":"Same as `c:replace/3`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:replace/4"},{"type":"callback","title":"Examples - Nebulex.Cache.replace/4","doc":"iex> MyCache.replace(MyCache1, \"foo\", \"bar\", [])\n    {:ok, false}\n    iex> MyCache.put_new(\"foo\", \"bar\")\n    {:ok, true}\n    iex> MyCache.replace(MyCache1, \"foo\", \"bar\", ttl: :timer.hours(1))\n    {:ok, true}","ref":"Nebulex.Cache.html#c:replace/4-examples"},{"type":"callback","title":"Nebulex.Cache.replace!/3","doc":"Same as `c:replace/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:replace!/3"},{"type":"callback","title":"Examples - Nebulex.Cache.replace!/3","doc":"iex> MyCache.replace!(\"foo\", \"bar\")\n    false\n    iex> MyCache.put_new!(\"foo\", \"bar\")\n    true\n    iex> MyCache.replace!(\"foo\", \"bar2\")\n    true","ref":"Nebulex.Cache.html#c:replace!/3-examples"},{"type":"callback","title":"Nebulex.Cache.replace!/4","doc":"Same as `c:replace!/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:replace!/4"},{"type":"callback","title":"Nebulex.Cache.start_link/1","doc":"Starts a supervision and return `{:ok, pid}` or just `:ok` if nothing\nneeds to be done.\n\nReturns `{:error, {:already_started, pid}}` if the cache is already\nstarted or `{:error, term}` in case anything else goes wrong.","ref":"Nebulex.Cache.html#c:start_link/1"},{"type":"callback","title":"Options - Nebulex.Cache.start_link/1","doc":"See the configuration in the moduledoc for options shared between adapters,\nfor adapter-specific configuration see the adapter's documentation.","ref":"Nebulex.Cache.html#c:start_link/1-options"},{"type":"callback","title":"Nebulex.Cache.stop/1","doc":"Shuts down the cache.","ref":"Nebulex.Cache.html#c:stop/1"},{"type":"callback","title":"Options - Nebulex.Cache.stop/1","doc":"`:timeout` - It is an integer that specifies how many milliseconds to wait\n  for the cache supervisor process to terminate, or the atom `:infinity` to\n  wait indefinitely. Defaults to `5000`. See `Supervisor.stop/3`.\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:stop/1-options"},{"type":"callback","title":"Nebulex.Cache.stop/2","doc":"Same as `c:stop/1` but stops the cache instance given in the first argument\n`dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:stop/2"},{"type":"callback","title":"Nebulex.Cache.stream/2","doc":"Similar to `c:get_all/2`, but returns a lazy enumerable that emits all entries\nmatching the query specified by the given `query_spec`.\n\nSee `c:get_all/2` for more information about the `query_spec`.\n\nThis function returns:\n\n  * `{:ok, stream}` - It returns a `stream` of values.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:stream/2"},{"type":"callback","title":"Options - Nebulex.Cache.stream/2","doc":"* `:max_entries` (`t:pos_integer/0`) - The number of entries to load from the cache as we stream The default value is `100`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:stream/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.stream/2","doc":"Populate the cache with some entries:\n\n    iex> MyCache.put_all(a: 1, b: 2, c: 3)\n    :ok\n\nStream all (default args):\n\n    iex> {:ok, stream} = MyCache.stream()\n    iex> Enum.to_list(stream)\n    [a: 1, b: 2, c: 3]\n\nStream all entries returning only the keys (with :max_entries option):\n\n    iex> {:ok, stream} = MyCache.stream([select: :key], max_entries: 2)\n    iex> Enum.to_list(stream)\n    [:a, :b, :c]\n\nStream all entries returning only the values:\n\n    iex> {:ok, stream} = MyCache.stream(select: :value)\n    iex> Enum.to_list(stream)\n    [1, 2, 3]\n\nStream only the requested keys (lazy bulk-fetch):\n\n    iex> {:ok, stream} = MyCache.stream(in: [:a, :b, :d])\n    iex> Enum.to_list(stream)\n    [a: 1, b: 2]\n    iex> {:ok, stream} = MyCache.stream(in: [:a, :b, :d], select: :key)\n    iex> Enum.to_list(stream)\n    [:a, :b]","ref":"Nebulex.Cache.html#c:stream/2-examples"},{"type":"callback","title":"Nebulex.Cache.stream/3","doc":"Same as `c:stream/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:stream/3"},{"type":"callback","title":"Examples - Nebulex.Cache.stream/3","doc":"iex> = MyCache.stream(MyCache1, [], [])\n    {:ok, _stream}","ref":"Nebulex.Cache.html#c:stream/3-examples"},{"type":"callback","title":"Nebulex.Cache.stream!/2","doc":"Same as `c:stream/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:stream!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.stream!/2","doc":"iex> MyCache.put_all(a: 1, b: 2, c: 3)\n    :ok\n    iex> MyCache.stream!() |> Enum.to_list()\n    [a: 1, b: 2, c: 3]","ref":"Nebulex.Cache.html#c:stream!/2-examples"},{"type":"callback","title":"Nebulex.Cache.stream!/3","doc":"Same as `c:stream!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:stream!/3"},{"type":"type","title":"Nebulex.Cache.t/0","doc":"Cache type","ref":"Nebulex.Cache.html#t:t/0"},{"type":"callback","title":"Nebulex.Cache.take/2","doc":"Removes and returns the value associated with `key` in the cache.\n\nIf `key` is present in the cache, its value is removed and returned as\n`{:ok, value}`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned. `reason` is the cause of the error and can be\n`Nebulex.KeyError` if the cache does not contain `key` or\n`Nebulex.Error` otherwise.","ref":"Nebulex.Cache.html#c:take/2"},{"type":"callback","title":"Options - Nebulex.Cache.take/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:take/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.take/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.take(:a)\n    {:ok, 1}\n\n    iex> {:error, %Nebulex.KeyError{key: :a} = e} = MyCache.take(:a)\n    iex> e.reason\n    :not_found","ref":"Nebulex.Cache.html#c:take/2-examples"},{"type":"callback","title":"Nebulex.Cache.take/3","doc":"Same as `c:take/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:take/3"},{"type":"callback","title":"Examples - Nebulex.Cache.take/3","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.take(MyCache1, :a, [])\n    {:ok, 1}","ref":"Nebulex.Cache.html#c:take/3-examples"},{"type":"callback","title":"Nebulex.Cache.take!/2","doc":"Same as `c:take/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:take!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.take!/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.take!(:a)\n    1","ref":"Nebulex.Cache.html#c:take!/2-examples"},{"type":"callback","title":"Nebulex.Cache.take!/3","doc":"Same as `c:take!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:take!/3"},{"type":"callback","title":"Nebulex.Cache.touch/2","doc":"Returns `{:ok, true}` if the given `key` exists and the last access time is\nsuccessfully updated; otherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.","ref":"Nebulex.Cache.html#c:touch/2"},{"type":"callback","title":"Options - Nebulex.Cache.touch/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:touch/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.touch/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.touch(:a)\n    {:ok, true}\n    iex> MyCache.ttl(:b)\n    {:ok, false}","ref":"Nebulex.Cache.html#c:touch/2-examples"},{"type":"callback","title":"Nebulex.Cache.touch/3","doc":"Same as `c:touch/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:touch/3"},{"type":"callback","title":"Examples - Nebulex.Cache.touch/3","doc":"iex> MyCache.touch(MyCache1, :a, [])\n    {:ok, false}","ref":"Nebulex.Cache.html#c:touch/3-examples"},{"type":"callback","title":"Nebulex.Cache.touch!/2","doc":"Same as `c:touch/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:touch!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.touch!/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n    iex> MyCache.touch!(:a)\n    true","ref":"Nebulex.Cache.html#c:touch!/2-examples"},{"type":"callback","title":"Nebulex.Cache.touch!/3","doc":"Same as `c:touch!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:touch!/3"},{"type":"callback","title":"Nebulex.Cache.transaction/2","doc":"Runs the given function inside a transaction.\n\nIf an Elixir exception occurs, the exception will bubble up from the\ntransaction function. If the cache aborts the transaction, it returns\n`{:error, reason}`.\n\nA successful transaction returns the value returned by the function wrapped\nin a tuple as `{:ok, value}`.","ref":"Nebulex.Cache.html#c:transaction/2"},{"type":"callback","title":"Nested transactions - Nebulex.Cache.transaction/2","doc":"If `transaction/2` is called inside another transaction, the cache executes\nthe function without wrapping the new transaction call in any way.","ref":"Nebulex.Cache.html#c:transaction/2-nested-transactions"},{"type":"callback","title":"Options - Nebulex.Cache.transaction/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:transaction/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.transaction/2","doc":"MyCache.transaction(fn ->\n      alice = MyCache.get(:alice)\n      bob = MyCache.get(:bob)\n      MyCache.put(:alice, %{alice | balance: alice.balance + 100})\n      MyCache.put(:bob, %{bob | balance: bob.balance + 100})\n    end)\n\nWe can provide the keys to lock when using the `Nebulex.Adapters.Local`\nadapter: (recommended):\n\n    MyCache.transaction(\n      fn ->\n        alice = MyCache.get(:alice)\n        bob = MyCache.get(:bob)\n        MyCache.put(:alice, %{alice | balance: alice.balance + 100})\n        MyCache.put(:bob, %{bob | balance: bob.balance + 100})\n      end,\n      [keys: [:alice, :bob]]\n    )","ref":"Nebulex.Cache.html#c:transaction/2-examples"},{"type":"callback","title":"Nebulex.Cache.transaction/3","doc":"Same as `c:transaction/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:transaction/3"},{"type":"callback","title":"Examples - Nebulex.Cache.transaction/3","doc":"MyCache.transaction(\n      MyCache1,\n      fn ->\n        alice = MyCache.get(:alice)\n        bob = MyCache.get(:bob)\n        MyCache.put(:alice, %{alice | balance: alice.balance + 100})\n        MyCache.put(:bob, %{bob | balance: bob.balance + 100})\n      end,\n      [keys: [:alice, :bob]]\n    )","ref":"Nebulex.Cache.html#c:transaction/3-examples"},{"type":"callback","title":"Nebulex.Cache.ttl/2","doc":"Returns the remaining time-to-live for the given `key`.\n\nIf `key` is present in the cache, its remaining TTL is returned as\n`{:ok, ttl}`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned. `reason` is the cause of the error and can be\n`Nebulex.KeyError` if the cache does not contain `key`,\n`Nebulex.Error` otherwise.","ref":"Nebulex.Cache.html#c:ttl/2"},{"type":"callback","title":"Options - Nebulex.Cache.ttl/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:ttl/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.ttl/2","doc":"iex> MyCache.put(:a, 1, ttl: :timer.seconds(5))\n    :ok\n    iex> MyCache.put(:b, 2)\n    :ok\n    iex> MyCache.ttl(:a)\n    {:ok, _remaining_ttl}\n    iex> MyCache.ttl(:b)\n    {:ok, :infinity}\n\n    iex> {:error, %Nebulex.KeyError{key: :c} = e} = MyCache.ttl(:c)\n    iex> e.reason\n    :not_found","ref":"Nebulex.Cache.html#c:ttl/2-examples"},{"type":"callback","title":"Nebulex.Cache.ttl/3","doc":"Same as `c:ttl/2`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:ttl/3"},{"type":"callback","title":"Examples - Nebulex.Cache.ttl/3","doc":"iex> MyCache.put(:a, 1, ttl: :timer.seconds(5))\n    :ok\n    iex> MyCache.ttl(MyCache1, :a, [])\n    {:ok, _remaining_ttl}","ref":"Nebulex.Cache.html#c:ttl/3-examples"},{"type":"callback","title":"Nebulex.Cache.ttl!/2","doc":"Same as `c:ttl/2` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:ttl!/2"},{"type":"callback","title":"Examples - Nebulex.Cache.ttl!/2","doc":"iex> MyCache.put(:a, 1, ttl: :timer.seconds(5))\n    :ok\n    iex> MyCache.ttl!(:a)\n    _remaining_ttl","ref":"Nebulex.Cache.html#c:ttl!/2-examples"},{"type":"callback","title":"Nebulex.Cache.ttl!/3","doc":"Same as `c:ttl!/3` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:ttl!/3"},{"type":"callback","title":"Nebulex.Cache.unregister_event_listener/2","doc":"Un-register a cache event listener.\n\nReturns `:ok` if successful; `{:error, reason}` otherwise.","ref":"Nebulex.Cache.html#c:unregister_event_listener/2"},{"type":"callback","title":"Options - Nebulex.Cache.unregister_event_listener/2","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:unregister_event_listener/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.unregister_event_listener/2","doc":"# Register with default ID\n    iex> MyApp.Cache.register_event_listener(&MyApp.handle/1)\n    :ok\n    # Unregister with default ID\n    iex> MyApp.Cache.unregister_event_listener(&MyApp.handle/1)\n    :ok\n\n    # Register with `:id`\n    iex> MyApp.Cache.register_event_listener(&MyApp.handle/1, id: :listener)\n    :ok\n    # Unregister using the previously registered `:id`\n    iex> MyApp.Cache.unregister_event_listener(:listener)\n    :ok","ref":"Nebulex.Cache.html#c:unregister_event_listener/2-examples"},{"type":"callback","title":"Nebulex.Cache.unregister_event_listener/3","doc":"Same as `c:unregister_event_listener/2`, but the command is executed on the\ncache instance given at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:unregister_event_listener/3"},{"type":"callback","title":"Examples - Nebulex.Cache.unregister_event_listener/3","doc":"MyApp.Cache.unregister_event_listener(:my_cache, &MyApp.handle/1, [])","ref":"Nebulex.Cache.html#c:unregister_event_listener/3-examples"},{"type":"callback","title":"Nebulex.Cache.unregister_event_listener!/2","doc":"Same as `c:unregister_event_listener/2` but raises an exception if an error\noccurs.","ref":"Nebulex.Cache.html#c:unregister_event_listener!/2"},{"type":"callback","title":"Nebulex.Cache.unregister_event_listener!/3","doc":"Same as `c:unregister_event_listener/3` but raises an exception if an error\noccurs.","ref":"Nebulex.Cache.html#c:unregister_event_listener!/3"},{"type":"callback","title":"Nebulex.Cache.update/4","doc":"Updates the `key` in the cache with the given function.\n\nIf `key` is present in the cache, the existing value is passed to `fun` and\nits result is used as the updated value of `key`. If `key` is not present in\nthe cache, `default` is inserted as the value of `key`. The default value\nwill not be passed through the update function.\n\nThis function returns:\n\n  * `{:ok, value}` - The value associated with the `key` is updated.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause.","ref":"Nebulex.Cache.html#c:update/4"},{"type":"callback","title":"Options - Nebulex.Cache.update/4","doc":"* `:ttl` (`t:timeout/0`) - The key's time-to-live (or expiry time) in **milliseconds**\n  (`:infinity` to store indefinitely). The default value is `:infinity`.\n\n* `:keep_ttl` (`t:boolean/0`) - Indicates whether to retain the time to live associated with the key.\n  Otherwise, the value in the `:ttl` option overwrites the existing one. The default value is `false`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Cache.html#c:update/4-options"},{"type":"callback","title":"Examples - Nebulex.Cache.update/4","doc":"iex> MyCache.update(:a, 1, &(&1 * 2))\n    {:ok, 1}\n    iex> MyCache.update(:a, 1, &(&1 * 2))\n    {:ok, 2}","ref":"Nebulex.Cache.html#c:update/4-examples"},{"type":"callback","title":"Nebulex.Cache.update/5","doc":"Same as `c:update/4`, but the command is executed on the cache instance\ngiven at the first argument `dynamic_cache`.\n\nSee the [\"Dynamic caches\"](#module-dynamic-caches) section in the\nmodule documentation for more information.","ref":"Nebulex.Cache.html#c:update/5"},{"type":"callback","title":"Examples - Nebulex.Cache.update/5","doc":"iex> MyCache.update(MyCache1, :a, 1, &(&1 * 2), [])\n    {:ok, 1}","ref":"Nebulex.Cache.html#c:update/5-examples"},{"type":"callback","title":"Nebulex.Cache.update!/4","doc":"Same as `c:update/4` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:update!/4"},{"type":"callback","title":"Examples - Nebulex.Cache.update!/4","doc":"iex> MyCache.update!(:a, 1, &(&1 * 2))\n    1","ref":"Nebulex.Cache.html#c:update!/4-examples"},{"type":"callback","title":"Nebulex.Cache.update!/5","doc":"Same as `c:update!/5` but raises an exception if an error occurs.","ref":"Nebulex.Cache.html#c:update!/5"},{"type":"type","title":"Nebulex.Cache.value/0","doc":"Cache entry value","ref":"Nebulex.Cache.html#t:value/0"},{"type":"callback","title":"Nebulex.Cache.with_dynamic_cache/2","doc":"Invokes the function `fun` using the given dynamic cache.","ref":"Nebulex.Cache.html#c:with_dynamic_cache/2"},{"type":"callback","title":"Example - Nebulex.Cache.with_dynamic_cache/2","doc":"MyCache.with_dynamic_cache(:my_cache, fn ->\n      MyCache.put(\"foo\", \"var\")\n    end)\n\nSee `c:get_dynamic_cache/0` and `c:put_dynamic_cache/1`.","ref":"Nebulex.Cache.html#c:with_dynamic_cache/2-example"},{"type":"module","title":"Nebulex.Event","doc":"Nebulex cache event.","ref":"Nebulex.Event.html"},{"type":"type","title":"Nebulex.Event.filter/0","doc":"Type for an entry event filter","ref":"Nebulex.Event.html#t:filter/0"},{"type":"type","title":"Nebulex.Event.listener/0","doc":"Type for an entry event listener","ref":"Nebulex.Event.html#t:listener/0"},{"type":"type","title":"Nebulex.Event.metadata/0","doc":"Type for the listener and filter metadata","ref":"Nebulex.Event.html#t:metadata/0"},{"type":"type","title":"Nebulex.Event.t/0","doc":"Type for a cache event","ref":"Nebulex.Event.html#t:t/0"},{"type":"module","title":"Nebulex.Event.CacheEntryEvent","doc":"A cache entry event.","ref":"Nebulex.Event.CacheEntryEvent.html"},{"type":"function","title":"Nebulex.Event.CacheEntryEvent.__types__/0","doc":"Returns the event types.","ref":"Nebulex.Event.CacheEntryEvent.html#__types__/0"},{"type":"function","title":"Example - Nebulex.Event.CacheEntryEvent.__types__/0","doc":"iex> Nebulex.Event.CacheEntryEvent.__types__()\n    [:deleted, :expired, :inserted, :updated]","ref":"Nebulex.Event.CacheEntryEvent.html#__types__/0-example"},{"type":"function","title":"Nebulex.Event.CacheEntryEvent.deleted/1","doc":"Creates a new \"**deleted**\" event.","ref":"Nebulex.Event.CacheEntryEvent.html#deleted/1"},{"type":"function","title":"Nebulex.Event.CacheEntryEvent.expired/1","doc":"Creates a new \"**expired**\" event.","ref":"Nebulex.Event.CacheEntryEvent.html#expired/1"},{"type":"function","title":"Nebulex.Event.CacheEntryEvent.inserted/1","doc":"Creates a new \"**inserted**\" event.","ref":"Nebulex.Event.CacheEntryEvent.html#inserted/1"},{"type":"function","title":"Nebulex.Event.CacheEntryEvent.new/1","doc":"Creates a new event.","ref":"Nebulex.Event.CacheEntryEvent.html#new/1"},{"type":"function","title":"Example - Nebulex.Event.CacheEntryEvent.new/1","doc":"iex> Nebulex.Event.CacheEntryEvent.new(\n    ...>   cache: :my_cache,\n    ...>   target: {:key, \"foo\"},\n    ...>   command: :put,\n    ...>   type: :inserted\n    ...> )\n    %Nebulex.Event.CacheEntryEvent{\n      command: :put,\n      name: nil,\n      type: :inserted,\n      cache: :my_cache,\n      metadata: [],\n      target: {:key, \"foo\"}\n    }","ref":"Nebulex.Event.CacheEntryEvent.html#new/1-example"},{"type":"type","title":"Nebulex.Event.CacheEntryEvent.t/0","doc":"Type for a cache entry event.\n\nThe event will have the following keys:\n\n  * `:cache` - The defined cache module.\n  * `:name` - The cache name (for dynamic caches).\n  * `:type` - The event type.\n  * `:target` - The event target. It could be a key or a query\n    (in case of `delete_all`).\n  * `:command` - The cache command triggering the event.\n  * `:metadata` - The event metadata is provided when the listener is\n    registered.","ref":"Nebulex.Event.CacheEntryEvent.html#t:t/0"},{"type":"type","title":"Nebulex.Event.CacheEntryEvent.target/0","doc":"Type for the event target.\n\nThe event target can be a key or a query (for `delete_all` command).\nIf the target is a key, it will come in the shape of `{:key, deleted_key}`\ntuple. On the other hand, if the target is a query, it will come in the shape\nof a `{:query, {:in, deleted_keys}` tuple, or a `{:query, {:q, query}` tuple.","ref":"Nebulex.Event.CacheEntryEvent.html#t:target/0"},{"type":"type","title":"Nebulex.Event.CacheEntryEvent.type/0","doc":"Event type.\n\n  * `:deleted` - Invoked if a cache entry is deleted, or if a batch call\n    is made, after the entries are deleted. The commands `delete` and\n    `delete_all` triggers this event.\n  * `:expired` - Invoked if a cache entry or entries are evicted due to\n    expiration.\n  * `:inserted` - Invoked after a cache entry is inserted, or if a batch call\n    is made after the entries are inserted. The commands triggering this event\n    are: `put`, `put_new`, `put_all`, and `put_new_all`. Beware, for  `put`\n    and `put_all` commands, there is no way to know if the entry existed\n    before the operation.\n  * `:updated` - Invoked if an existing cache entry is updated via `replace`\n    command.","ref":"Nebulex.Event.CacheEntryEvent.html#t:type/0"},{"type":"function","title":"Nebulex.Event.CacheEntryEvent.updated/1","doc":"Creates a new \"**updated**\" event.","ref":"Nebulex.Event.CacheEntryEvent.html#updated/1"},{"type":"module","title":"Nebulex.Caching","doc":"At its core, the abstraction applies caching to Elixir functions, reducing\nthus the number of executions based on the information available in the\ncache. That is, each time a targeted function is invoked, the abstraction\nwill apply a caching behavior checking whether the function has been already\nexecuted and its result cached for the given arguments. If it has, then the\ncached result is returned without having to execute the actual function;\nif it has not, then function is executed, the result cached and returned\nto the user so that, the next time the method is invoked, the cached result\nis returned. This way, expensive functions (whether CPU or IO bound) can be\nexecuted only once for a given set of parameters and the result reused\nwithout having to actually execute the function again. The caching logic\nis applied transparently without any interference to the invoker.\n\nSee **`Nebulex.Caching.Decorators`** to learn more about the caching\ndecorators and their usage.","ref":"Nebulex.Caching.html"},{"type":"module","title":"Compilation time options - Nebulex.Caching","doc":"The following are the available compilation time options when defining\nthe caching usage via `use Nebulex.Caching`:\n\n* `:default_key_generator` (`t:Nebulex.Caching.Decorators.key/0`) - Defines the default key generation function for all decorated functions\n  in the module. It can be overridden at the decorator level via `:key`\n  option.\n\n  The function must be provided in the format `&Mod.fun/arity`.\n\n  The default value is `&Nebulex.Caching.Decorators.generate_key/1`.\n\n* `:cache` (`t:atom/0`) - Defines the cache for all decorated functions in the module.\n  It can be overridden at the decorator level.\n\n* `:on_error` (`t:Nebulex.Caching.Decorators.on_error/0`) - Whether to raise an exception or do nothing when there is a cache error.\n  It applies to all decorated functions in the module. It can be overridden\n  at the decorator level. The default value is `:nothing`.\n\n* `:match` (`t:Nebulex.Caching.Decorators.match/0`) - The match function for all decorated functions in the module. It can be\n  overridden at the decorator level.\n\n  The function must be provided in the format `&Mod.fun/arity`.\n\n  The default value is `&Nebulex.Caching.Decorators.default_match/1`.\n\n* `:opts` (`t:keyword/0`) - The options to use globally for all decorated functions in the module when\n  invoking cache commands. The default value is `[]`.\n\n\n\nSee the [\"Shared Options\"](http://hexdocs.pm/nebulex/Nebulex.Caching.Decorators.html#module-shared-options)\nsection in the decorators module for more information.\n\n> #### `use Nebulex.Caching, opts` {: .info}\n>\n> These options apply to all decorated functions in a module, but each\n> decorator declaration can overwrite them. They act as a global or default\n> configuration for the decorators. For example, if the cache is the same\n> for all decorated functions in a module, one can configure it globally\n> like this:\n>\n> ```elixir\n> use Nebulex.Caching, cache: MyCache\n> ```\n>\n> Therefore, you don't need to provide the `:cache` option at the decorator\n> level.","ref":"Nebulex.Caching.html#module-compilation-time-options"},{"type":"macro","title":"Nebulex.Caching.dynamic_cache/2","doc":"Creates a dynamic cache tuple form to use in the decorated function\n(wrapper macro for `Nebulex.Caching.Decorators.dynamic_cache_spec/2`).\n\nThe first argument, `cache`, specifies the defined cache module,\nand the second argument, `name`, is the actual name of the cache.\n\n> #### Using `dynamic_cache` {: .info}\n>\n> This macro is automatically imported and then available when using\n> `use Nebulex.Caching`.","ref":"Nebulex.Caching.html#dynamic_cache/2"},{"type":"macro","title":"Example - Nebulex.Caching.dynamic_cache/2","doc":"defmodule MyApp.Users do\n      use Nebulex.Caching\n\n      @decorate cacheable(cache: dynamic_cache(MyApp.Cache, :users))\n      def get_user(id) do\n        # your logic ...\n      end\n    end\n\nSee the **\"`:cache` option\"** section in the `Nebulex.Caching.Decorators`\nmodule documentation for more information.","ref":"Nebulex.Caching.html#dynamic_cache/2-example"},{"type":"macro","title":"Nebulex.Caching.keyref/2","doc":"Creates a reference tuple form to use in the decorated function\n(wrapper macro for `Nebulex.Caching.Decorators.keyref_spec/3`).\n\n> #### Using `keyref` {: .info}\n>\n> This macro is automatically imported and then available when using\n> `use Nebulex.Caching`.","ref":"Nebulex.Caching.html#keyref/2"},{"type":"macro","title":"Options - Nebulex.Caching.keyref/2","doc":"* `:cache` - The cache where the referenced key is stored.\n\n  * `:ttl` - The TTL for the referenced key. If configured, it overrides\n    the TTL given in the decorator's option `:opts`.\n\nSee `Nebulex.Caching.Decorators.cacheable/3` decorator\nfor more information.","ref":"Nebulex.Caching.html#keyref/2-options"},{"type":"module","title":"Nebulex.Caching.Decorators","doc":"Declarative decorator-based caching, inspired by\n[Spring Cache Abstraction][spring-cache].\n\n> *[`decorator`][decorator-lib] library is used underneath.*\n\n[spring-cache]: https://docs.spring.io/spring/docs/3.2.x/spring-framework-reference/html/cache.html\n[decorator-lib]: https://github.com/arjan/decorator\n\nFor caching declaration, the abstraction provides three Elixir function\ndecorators: `cacheable `, `cache_evict`, and `cache_put`, which allow\nfunctions to trigger cache population or cache eviction.\nLet us take a closer look at each decorator.\n\n## `cacheable` decorator\n\nAs the name implies, `cacheable` is used to delimit functions that are\ncacheable - that is, functions for whom the result is stored in the cache\nso that on subsequent invocations (with the same arguments), the value is\nreturned from the cache without having to execute the function. In its\nsimplest form, the decorator declaration requires the cache associated with\nthe decorated function if the [default cache](#module-default-cache) is not\nconfigured (see [\"Cache configuration\"](#module-cache-configuration)):\n\n    @decorate cacheable(cache: Cache)\n    def find_book(isbn) do\n      # the logic for retrieving the book ...\n    end\n\nIn the snippet above, the function `get_account/1` is associated with the\ncache named `Cache`. Each time the function is called, the cache is checked\nto see whether the invocation has been already executed and does not have\nto be repeated.\n\nSee `cacheable/3` for more information.\n\n## `cache_put` decorator\n\nFor cases where the cache needs to be updated without interfering with the\nfunction execution, one can use the `cache_put` decorator. That is, the\nfunction will always be executed and its result placed into the cache\n(according to the `cache_put` options). It supports the same options as\n`cacheable` and should be used for cache population or update rather than\nfunction flow optimization.\n\n    @decorate cache_put(cache: Cache)\n    def update_book(isbn) do\n      # the logic for retrieving the book and then updating it ...\n    end\n\nNote that using `cache_put` and `cacheable` decorators on the same function\nis generally discouraged because they have different behaviors. While the\nlatter causes the function execution to be skipped by using the cache, the\nformer forces the execution in order to execute a cache update. This leads\nto unexpected behavior and with the exception of specific corner-cases\n(such as decorators having conditions that exclude them from each other),\nsuch declarations should be avoided.\n\nSee `cache_put/3` for more information.\n\n## `cache_evict` decorator\n\nThe cache abstraction allows not just the population of a cache store but\nalso eviction. This process is useful for removing stale or unused data from\nthe cache. Opposed to `cacheable`, the decorator `cache_evict` demarcates\nfunctions that perform cache eviction, which are functions that act as\ntriggers for removing data from the cache. Just like its sibling,\n`cache_evict` requires specifying the cache that will be affected by the\naction, allows to provide a key or a list of keys to be evicted, but in\naddition, features an extra option `:all_entries` which indicates whether\na cache-wide eviction needs to be performed rather than just one or a few\nentries (based on `:key` or `:keys` option):\n\n    @decorate cache_evict(cache: Cache, all_entries: true)\n    def load_books(file_stream) do\n      # the logic for loading books ...\n    end\n\nThe option `:all_entries` comes in handy when an entire cache region needs\nto be cleared out - rather than evicting each entry (which would take a\nlong time since it is inefficient), all the entries are removed in one\noperation as shown above.\n\nOne can also indicate whether the eviction should occur after (the default)\nor before the function executes through the `:before_invocation` attribute.\nThe former provides the same semantics as the rest of the decorators; once\nthe method completes successfully, an action (in this case, eviction) on the\ncache is executed. If the function does not execute (as it might be cached)\nor an exception is raised, the eviction does not occur. The latter\n(`before_invocation: true`) causes the eviction to occur always before the\nmethod is invoked. This is useful in cases where the eviction does not need\nto be tied to the function execution outcome.\n\nSee `cache_evict/3` for more information.","ref":"Nebulex.Caching.Decorators.html"},{"type":"module","title":"Shared Options - Nebulex.Caching.Decorators","doc":"All three cache decorators explained previously accept the following\noptions:\n\n* `:cache` (`t:cache/0`) - Required. The cache to use. If configured, it overrides the\n  [default or global cache](#module-default-cache).\n  See `t:cache/0` for possible values.\n\n  Raises an exception if the `:cache` option is not provided in the\n  decorator declaration and is not configured when defining the\n  caching usage via `use Nebulex.Caching` either.\n\n  See [\"Cache configuration\"](#module-cache-configuration) section\n  for more information.\n\n* `:key` (`t:key/0`) - The cache access key the decorator will use when running the decorated\n  function. The default key generator generates a default key when the\n  option is unavailable.\n\n  The `:key` option admits the following values:\n\n    * An anonymous function to call to generate the key in runtime.\n      The function optionally receives the decorator context as an argument\n      and must return the key for caching.\n    * The tuple `{:in, keys}`, where `keys` is a list with the keys to\n      evict or update. This option is allowed for `cache_evict` and\n      `cache_put` decorators only.\n    * Any term.\n\n  See [\"Key Generation\"](#module-key-generation) section\n  for more information.\n\n* `:match` (`t:match/0`) - Anonymous function to decide whether or not the result (provided as a\n  first argument) of evaluating the decorated function is cached.\n  Optionally, the match function can receive the decorator context as a\n  second argument. The match function can return:\n\n    * `true` - The value returned by the decorated function invocation is\n      cached. (the default).\n    * `{true, value}` - `value` is cached. It is helpful to customize what\n      exactly must be cached.\n    * `{true, value, opts}` - The `value` is cached with the provided\n      options `opts`. It is helpful to customize what must be cached and the\n      runtime options for storing it. (e.g., `{true, value, [ttl: @ttl]}`).\n    * `false` - Cache nothing.\n\n  The default match function looks like this:\n\n  ```elixir\n  def default_match({:error, _}), do: false\n  def default_match(:error), do: false\n  def default_match(_other), do: true\n  ```\n\n  By default, if the evaluation of the decorated function returns any of the\n  following terms/values `:error` or `{:error, term}`, the default match\n  function returns `false` (cache nothing). Otherwise, `true` is returned\n  (the value is cached). Remember that the default match function may store\n  a `nil` value if the decorated function returns it. If you don't want to\n  cache `nil` values or, in general, desire a different behavior, you should\n  provide another match function to meet your requirements.\n\n  If configured, it overrides the global value (if any) defined when using\n  `use Nebulex.Caching, match: &MyApp.match/1`.\n\n  The default value is `&Nebulex.Caching.Decorators.default_match/1`.\n\n* `:on_error` (`t:on_error/0`) - The decorators perform cache commands under the hood. With the option\n  `:on_error`, we can tell the decorator what to do in case of an error\n  or exception. The option supports the following values:\n\n    * `:nothing` - ignores the error.\n    * `:raise` - raises if there is an error.\n\n  If configured, it overrides the global value (if any) defined when using\n  `use Nebulex.Caching, on_error: ...`.\n\n  The default value is `:nothing`.\n\n* `:opts` (`t:keyword/0`) - The options used by the decorator when invoking cache commands. The default value is `[]`.","ref":"Nebulex.Caching.Decorators.html#module-shared-options"},{"type":"module","title":"Cache configuration - Nebulex.Caching.Decorators","doc":"As documented in the options above, the `:cache` option configures the cache\nfor the decorated function (in the decorator declaration). However, there\nare three possible values, such as documented in the `t:cache/0` type.\nLet's go over these cache value alternatives in detail.","ref":"Nebulex.Caching.Decorators.html#module-cache-configuration"},{"type":"module","title":"Cache module - Nebulex.Caching.Decorators","doc":"The first cache value option is an existing cache module; this is the most\ncommon value. For example:\n\n    @decorate cacheable(cache: MyApp.Cache)\n    def find_book(isbn) do\n      # the logic for retrieving the book ...\n    end","ref":"Nebulex.Caching.Decorators.html#module-cache-module"},{"type":"module","title":"Dynamic cache - Nebulex.Caching.Decorators","doc":"In case one is using a dynamic cache:\n\n    @decorate cacheable(cache: dynamic_cache(MyApp.Cache, :books))\n    def find_book(isbn) do\n      # the logic for retrieving the book ...\n    end\n\n> See [\"Dynamic caches\"][dynamic-caches] for more information.\n\n[dynamic-caches]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Cache.html#module-dynamic-caches","ref":"Nebulex.Caching.Decorators.html#module-dynamic-cache"},{"type":"module","title":"Anonymous function - Nebulex.Caching.Decorators","doc":"Finally, it is also possible to configure an anonymous function to resolve\nthe cache value in runtime. The function receives the\n[decorator context](`t:context/0`) as an argument and must return either\na cache module or a dynamic cache.\n\n    @decorate cacheable(cache: &MyApp.Resolver.resolve_cache/1)\n    def find_book(isbn) do\n      # the logic for retrieving the book ...\n    end\n\nWhere `resolve_cache` function may look like this:\n\n    defmodule MyApp.Resolver do\n      alias Nebulex.Caching.Decorators.Context\n\n      def resolve_cache(%Context{} = context) do\n        # the logic for generating the cache value\n      end\n    end","ref":"Nebulex.Caching.Decorators.html#module-anonymous-function"},{"type":"module","title":"Default cache - Nebulex.Caching.Decorators","doc":"While option `:cache` is handy for specifying the decorated function's\ncache, it may be cumbersome when there is a module with several decorated\nfunctions, and all use the same cache. In that case, we must set the\n`:cache` option with the same value in all the decorated functions.\nFortunately, the `:cache` option can be configured globally for all\ndecorated functions in a module when defining the caching usage via\n`use Nebulex.Caching`. For example:\n\n    defmodule MyApp.Books do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      @decorate cacheable()\n      def get_book(isbn) do\n        # the logic for retrieving a book ...\n      end\n\n      @decorate cacheable(cache: MyApp.BestSellersCache)\n      def best_sellers do\n        # the logic for retrieving best seller books ...\n      end\n\n      ...\n    end\n\nIn the snippet above, the function `get_book/1` is associated with the\ncache `MyApp.Cache` by default since option `:cache` is not provided in\nthe decorator. In other words, when the `:cache` option is configured\nglobally (when defining the caching usage via `use Nebulex.Caching`),\nit is not required in the decorator declaration. However, one can always\noverride the global or default cache in the decorator declaration by\nproviding the option `:cache`, as is shown in the `best_sellers/0`\nfunction, which is associated with a different cache.\n\nTo conclude, it is crucial to know the decorator must be associated with\na cache, either a global or default cache defined at the caching usage\ndefinition (e.g., `use Nebulex.Caching, cache: MyCache`) or a specific\none configured in the decorator declaration.","ref":"Nebulex.Caching.Decorators.html#module-default-cache"},{"type":"module","title":"Key Generation - Nebulex.Caching.Decorators","doc":"Since caches are essentially key-value stores, each invocation of a cached\nfunction needs to be translated into a suitable key for cache access. The\nkey can be generated using a default key generator (which is configurable)\nor through decorator options `:key` or `:keys`. Let us take a closer look\nat each approach:","ref":"Nebulex.Caching.Decorators.html#module-key-generation"},{"type":"module","title":"Default Key Generation - Nebulex.Caching.Decorators","doc":"Out of the box, the caching abstraction uses a simple key generator\nstrategy given by `Nebulex.Caching.Decorators.generate_key/1`, which is\nbased on the following algorithm:\n\n  * If no arguments are given, return `0`.\n  * If only one argument is given, return that param as key.\n  * If more than one argument is given, return a key computed\n    from the hash of all arguments (`:erlang.phash2(args)`).\n\nOne could provide a different key generator via option\n`:default_key_generator`. Once it is configured, the generator will be used\nfor each declaration that does not specify its own key generation strategy.\nSee [\"Custom Key Generation\"](#module-custom-key-generation-declaration)\nsection down below.\n\nThe following example shows how to configure a custom default key generator:\n\n    defmodule MyApp.Keygen do\n      def generate(context) do\n        # your key generation logic ...\n      end\n    end\n\n    defmodule MyApp.Books do\n      use Nebulex.Caching,\n        cache: MyApp.Cache\n        default_key_generator: &MyApp.Keygen.generate/1\n\n      ...\n    end\n\nThe function given to `:default_key_generator` must follow the format\n`&Mod.fun/arity`.","ref":"Nebulex.Caching.Decorators.html#module-default-key-generation"},{"type":"module","title":"Custom Key Generation Declaration - Nebulex.Caching.Decorators","doc":"Since caching is generic, it is quite likely the target functions have\nvarious signatures that cannot be simply mapped on top of the cache\nstructure. This tends to become obvious when the target function has\nmultiple arguments out of which only some are suitable for caching\n(while the rest are used only by the function logic). For example:\n\n    @decorate cacheable(cache: Cache)\n    def find_book(isbn, check_warehouse?, include_used?) do\n      # the logic for retrieving the book ...\n    end\n\nAt first glance, while the two `boolean` arguments influence the way the\nbook is found, they are not used for the cache. Furthermore, what if only\none of the two is important while the other is not?\n\nFor such cases, the `cacheable` decorator allows the user to specify how\nthe key is generated through the `:key` option (the same applies to all\ndecorators). The developer can pick the arguments of interest (or their\nnested properties), perform operations or even invoke arbitrary functions\nwithout having to write any code or implement any interface. This is the\nrecommended approach over the default generator since functions tend to be\nquite different in signatures as the code base grows; while the default\nstrategy might work for some functions, it rarely does for all functions.\n\nThe following are some examples of generating keys:\n\n    @decorate cacheable(cache: Cache, key: isbn)\n    def find_book(isbn, check_warehouse?, include_used?) do\n      # the logic for retrieving the book ...\n    end\n\n    @decorate cacheable(cache: Cache, key: isbn.raw_number)\n    def find_book(isbn, check_warehouse?, include_used?) do\n      # the logic for retrieving the book ...\n    end\n\nIt is also possible to use an anonymous function to generate the key.\nThe function receives the [decorator's context](`t:context/0`)\nas an argument. For example:\n\n    @decorate cacheable(cache: Cache, key: &{&1.function_name, hd(&1.args)})\n    def find_book(isbn, check_warehouse?, include_used?) do\n      # the logic for retrieving the book ...\n    end\n\nThe key can be also the tuple `{:in, keys}` where `keys` is a list with the\nkeys to cache, evict, or update. For example:\n\n    @decorate cache_evict(cache: Cache, key: {:in, [isbn.id, isbn.raw_number]})\n    def remove_book(isbn) do\n      # the logic for removing the book ...\n    end\n\nThe tuple `{:in, [isbn.id, isbn.raw_number]}` instructs the `cache_evict`\ndecorator to remove the keys `isbn.id` and `isbn.raw_number` from the\ncache.\n\n> #### `key: {:in, [...]}` {: .info}\n>\n> The `:key` option only admits the value `{:in, [...]}` for\n> [`cache_evict`](`cache_evict/3`) and [`cache_put`](`cache_put/3`)\n> decorators only. When you need to cache the same value under different\n> keys, you usually decorate multiple functions, like so:\n>\n> ```elixir\n> @decorate cacheable(key: id)\n> def get_user(id)\n>\n> @decorate cacheable(key: email)\n> def get_user_by_email(email)\n> ```\n>\n> See [`cacheable`](`cacheable/3`) decorator for more information.","ref":"Nebulex.Caching.Decorators.html#module-custom-key-generation-declaration"},{"type":"module","title":"Custom options - Nebulex.Caching.Decorators","doc":"One can also provide options for the cache commands executed underneath,\nlike so:\n\n    @decorate cacheable(cache: Cache, key: isbn, opts: [ttl: :timer.hours(1)])\n    def find_book(isbn, check_warehouse?, include_used?) do\n      # the logic for retrieving the book ...\n    end\n\nIn that case, `opts: [ttl: :timer.hours(1)]` specifies the TTL for the\ncached value.\n\nSee the [\"Shared Options\"](#module-shared-options) section\nfor more information.","ref":"Nebulex.Caching.Decorators.html#module-custom-options"},{"type":"module","title":"Examples - Nebulex.Caching.Decorators","doc":"Supposing an app uses Ecto, and there is a context for accessing books\n`MyApp.Books`, we may decorate some functions as follows:\n\n    # The cache config\n    config :my_app, MyApp.Cache,\n      gc_interval: 86_400_000, #=> 1 day\n      max_size: 1_000_000 #=> Max 1M books\n\n    # The Cache\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\n    # Book schema\n    defmodule MyApp.Books.Book do\n      use Ecto.Schema\n\n      schema \"books\" do\n        field(:isbn, :string)\n        field(:title, :string)\n        field(:author, :string)\n        # The rest of the fields omitted\n      end\n\n      def changeset(book, attrs) do\n        book\n        |> cast(attrs, [:isbn, :title, :author])\n        |> validate_required([:isbn, :title, :author])\n      end\n    end\n\n    # Books context\n    defmodule MyApp.Books do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      alias MyApp.Repo\n      alias MyApp.Books.Book\n\n      @decorate cacheable(key: id)\n      def get_book(id) do\n        Repo.get(Book, id)\n      end\n\n      @decorate cacheable(key: isbn)\n      def get_book_by_isbn(isbn) do\n        Repo.get_by(Book, [isbn: isbn])\n      end\n\n      @decorate cache_put(\n                  key: {:in, [book.id, book.isbn]},\n                  match: &__MODULE__.match_fun/1\n                )\n      def update_book(%Book{} = book, attrs) do\n        book\n        |> Book.changeset(attrs)\n        |> Repo.update()\n      end\n\n      def match_fun({:ok, usr}), do: {true, usr}\n      def match_fun({:error, _}), do: false\n\n      @decorate cache_evict(key: {:in, [book.id, book.isbn]})\n      def delete_book(%Book{} = book) do\n        Repo.delete(book)\n      end\n\n      def create_book(attrs \\\\ %{}) do\n        %Book{}\n        |> Book.changeset(attrs)\n        |> Repo.insert()\n      end\n    end","ref":"Nebulex.Caching.Decorators.html#module-examples"},{"type":"module","title":"Functions with multiple clauses - Nebulex.Caching.Decorators","doc":"Since [`decorator`](https://github.com/arjan/decorator#functions-with-multiple-clauses)\nlibrary is used, it is important to be aware of its recommendations,\ncaveats, limitations, and so on. For instance, for functions with multiple\nclauses the general advice is to create an empty function head, and call\nthe decorator on that head, like so:\n\n    @decorate cacheable(cache: Cache)\n    def get_user(id \\\\ nil)\n\n    def get_user(nil), do: nil\n\n    def get_user(id) do\n      # your logic ...\n    end\n\nHowever, the previous example works because we are not using the function\nattributes for defining a custom key via the `:key` option. If we add\n`key: id` for instance, we will get errors and/or warnings, since the\ndecorator is expecting the attribute `id` to be present, but it is not\nin the first function clause. In other words, when we take this approach,\nis like the decorator was applied to all function clauses separately.\nTo overcome this issue, the arguments used in the decorator must be\npresent in the function clauses, which could be achieved in different\nways. A simple way would be to decorate a wrapper function with the\narguments the decorator use and do the pattern-matching in a separate\nfunction.\n\n    @decorate cacheable(cache: Cache, key: id)\n    def get_user(id \\\\ nil) do\n      do_get_user(id)\n    end\n\n    defp do_get_user(nil), do: nil\n\n    defp do_get_user(id) do\n      # your logic ...\n    end\n\nAlternatively, you could decorate only the function clause needing the\ncaching.\n\n    def get_user(nil), do: nil\n\n    @decorate cacheable(cache: Cache, key: id)\n    def get_user(id) do\n      # your logic ...\n    end","ref":"Nebulex.Caching.Decorators.html#module-functions-with-multiple-clauses"},{"type":"module","title":"Further readings - Nebulex.Caching.Decorators","doc":"* [Cache Usage Patterns Guide](http://hexdocs.pm/nebulex/3.0.0-rc.1/cache-usage-patterns.html).","ref":"Nebulex.Caching.Decorators.html#module-further-readings"},{"type":"type","title":"Nebulex.Caching.Decorators.cache/0","doc":"The type for the `:cache` option value.\n\nWhen defining the `:cache` option on the decorated function,\nthe value can be:\n\n  * The defined cache module.\n  * A dynamic cache spec created with the macro\n    [`dynamic_cache/2`](`Nebulex.Caching.dynamic_cache/2`).\n  * An anonymous function to call to resolve the cache value in runtime.\n    The function optionally receives the decorator context as an argument\n    and must return either a cache module or a dynamic cache.","ref":"Nebulex.Caching.Decorators.html#t:cache/0"},{"type":"macro","title":"Nebulex.Caching.Decorators.cache_evict/0","doc":"","ref":"Nebulex.Caching.Decorators.html#cache_evict/0"},{"type":"macro","title":"Nebulex.Caching.Decorators.cache_evict/1","doc":"","ref":"Nebulex.Caching.Decorators.html#cache_evict/1"},{"type":"function","title":"Nebulex.Caching.Decorators.cache_evict/3","doc":"Decorator indicating that a function triggers a cache evict operation\n(`delete` or `delete_all`).","ref":"Nebulex.Caching.Decorators.html#cache_evict/3"},{"type":"function","title":"Options - Nebulex.Caching.Decorators.cache_evict/3","doc":"* `:all_entries` (`t:boolean/0`) - Defines whether or not the decorator must remove all the entries inside\n  the cache. The default value is `false`.\n\n* `:before_invocation` (`t:boolean/0`) - Defines whether or not the decorator should run before invoking the\n  decorated function. The default value is `false`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Caching.Decorators.html#cache_evict/3-options"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.cache_evict/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      @decorate cache_evict(key: id)\n      def delete(id) do\n        # your logic (maybe write/delete data to the SoR)\n      end\n\n      @decorate cache_evict(key: {:in, [object.name, object.id]})\n      def delete_object(object) do\n        # your logic (maybe write/delete data to the SoR)\n      end\n\n      @decorate cache_evict(all_entries: true)\n      def delete_all do\n        # your logic (maybe write/delete data to the SoR)\n      end\n    end\n\n> #### **Write-through** pattern {: .info}\n>\n> This decorator supports the **Write-through** pattern. Your function\n> provides the logic to write data to the system of record (SoR), and the\n> decorator under the hood provides the rest. But in contrast with the\n> `update` decorator, the data is deleted from the cache instead of updated.","ref":"Nebulex.Caching.Decorators.html#cache_evict/3-examples"},{"type":"macro","title":"Nebulex.Caching.Decorators.cache_put/0","doc":"","ref":"Nebulex.Caching.Decorators.html#cache_put/0"},{"type":"macro","title":"Nebulex.Caching.Decorators.cache_put/1","doc":"","ref":"Nebulex.Caching.Decorators.html#cache_put/1"},{"type":"function","title":"Nebulex.Caching.Decorators.cache_put/3","doc":"Decorator indicating that a function triggers a\n[cache put](`c:Nebulex.Cache.put/3`) operation.\n\nIn contrast to the [`cacheable`](`cacheable/3`) decorator, this decorator\ndoes not cause the decorated function to be skipped. Instead, it always\ncauses the function to be invoked and its result to be stored in the\nassociated cache if the condition given by the `:match` option matches\naccordingly.","ref":"Nebulex.Caching.Decorators.html#cache_put/3"},{"type":"function","title":"Options - Nebulex.Caching.Decorators.cache_put/3","doc":"See the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Caching.Decorators.html#cache_put/3-options"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.cache_put/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      @ttl :timer.hours(1)\n\n      @decorate cache_put(key: id, opts: [ttl: @ttl])\n      def update!(id, attrs \\\\ %{}) do\n        # your logic (maybe write data to the SoR)\n      end\n\n      @decorate cache_put(\n                  key: id,\n                  match: &__MODULE__.match_fun/1,\n                  opts: [ttl: @ttl]\n                )\n      def update(id, attrs \\\\ %{}) do\n        # your logic (maybe write data to the SoR)\n      end\n\n      @decorate cache_put(\n                  key: {:in, [object.name, object.id]},\n                  match: &__MODULE__.match_fun/1,\n                  opts: [ttl: @ttl]\n                )\n      def update_object(object) do\n        # your logic (maybe write data to the SoR)\n      end\n\n      def match_fun({:ok, updated}), do: {true, updated}\n      def match_fun({:error, _}), do: false\n    end\n\n> #### **Write-through** pattern {: .info}\n>\n> This decorator supports the **Write-through** pattern. Your function\n> provides the logic to write data to the system of record (SoR), and the\n> decorator under the hood provides the rest.","ref":"Nebulex.Caching.Decorators.html#cache_put/3-examples"},{"type":"function","title":"Nebulex.Caching.Decorators.cache_put/4","doc":"Convenience function for the `cache_put` decorator.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#cache_put/4"},{"type":"type","title":"Nebulex.Caching.Decorators.cache_value/0","doc":"The type for the cache value","ref":"Nebulex.Caching.Decorators.html#t:cache_value/0"},{"type":"macro","title":"Nebulex.Caching.Decorators.cacheable/0","doc":"","ref":"Nebulex.Caching.Decorators.html#cacheable/0"},{"type":"macro","title":"Nebulex.Caching.Decorators.cacheable/1","doc":"","ref":"Nebulex.Caching.Decorators.html#cacheable/1"},{"type":"function","title":"Nebulex.Caching.Decorators.cacheable/3","doc":"As the name implies, the `cacheable` decorator indicates storing in cache\nthe result of invoking a function.\n\nEach time a decorated function is invoked, the caching behavior will be\napplied, checking whether the function has already been invoked for the\ngiven arguments. A default algorithm uses the function arguments to compute\nthe key. Still, a custom key can be provided through the `:key` option,\nor a custom key-generator implementation can replace the default one\n(See [\"Key Generation\"](#module-key-generation) section in the module\ndocumentation).\n\nIf no value is found in the cache for the computed key, the target function\nwill be invoked, and the returned value will be stored in the associated\ncache. Note that what is cached can be handled with the `:match` option.","ref":"Nebulex.Caching.Decorators.html#cacheable/3"},{"type":"function","title":"Options - Nebulex.Caching.Decorators.cacheable/3","doc":"* `:references` (`t:references/0`) - Indicates the key given by the option `:key` references another key\n  provided by the option `:references`. In other words, when present,\n  this option tells the `cacheable` decorator to store the decorated\n  function's block result under the referenced key given by the option\n  `:references` and the referenced key under the key provided by the\n  option `:key`.\n\n  See the [\"Referenced keys\"](#cacheable/3-referenced-keys) section below\n  for more information.\n\n  The default value is `nil`.\n\n\n\nSee the [\"Shared options\"](#module-shared-options) section in the module\ndocumentation for more options.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-options"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.cacheable/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      @ttl :timer.hours(1)\n\n      @decorate cacheable(key: id, opts: [ttl: @ttl])\n      def get_by_id(id) do\n        # your logic (maybe the loader to retrieve the value from the SoR)\n      end\n\n      @decorate cacheable(key: email, references: & &1.id)\n      def get_by_email(email) do\n        # your logic (maybe the loader to retrieve the value from the SoR)\n      end\n\n      @decorate cacheable(key: clauses, match: &match_fun/1)\n      def all(clauses) do\n        # your logic (maybe the loader to retrieve the value from the SoR)\n      end\n\n      defp match_fun([]), do: false\n      defp match_fun(_), do: true\n    end\n\n> #### **Read-through** pattern {: .info}\n>\n> This decorator supports the **Read-through** pattern. The loader to\n> retrieve the value from the system of record (SoR) is your function's\n> logic, and the macro under the hood provides the rest.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-examples"},{"type":"function","title":"Referenced keys - Nebulex.Caching.Decorators.cacheable/3","doc":"Referenced keys are handy when multiple keys keep the same value. For\nexample, let's imagine we have a schema `User` with multiple unique fields,\nlike `:id`, `:email`, and `:token`. We may have a module with functions\nretrieving the user account by any of those fields, like so:\n\n    defmodule MyApp.UserAccounts do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      @decorate cacheable(key: id)\n      def get_user_account(id) do\n        # your logic ...\n      end\n\n      @decorate cacheable(key: email)\n      def get_user_account_by_email(email) do\n        # your logic ...\n      end\n\n      @decorate cacheable(key: token)\n      def get_user_account_by_token(token) do\n        # your logic ...\n      end\n\n      @decorate cache_evict(key: {:in, [user.id, user.email, user.token]})\n      def update_user_account(user, attrs) do\n        # your logic ...\n      end\n    end\n\nAs you notice, all three functions will store the same user record under a\ndifferent key. It could be more efficient in terms of memory space. Besides,\nwhen the user record is updated, we have to invalidate the previously cached\nentries, which means we have to specify in the `cache_evict` decorator all\nthe different keys associated with the cached user account.\n\nUsing the referenced keys, we can address it better and more simply.\nThe module will look like this:\n\n    defmodule MyApp.UserAccounts do\n      use Nebulex.Caching, cache: MyApp.Cache\n\n      @decorate cacheable(key: id)\n      def get_user_account(id) do\n        # your logic ...\n      end\n\n      @decorate cacheable(key: email, references: & &1.id)\n      def get_user_account_by_email(email) do\n        # your logic ...\n      end\n\n      @decorate cacheable(key: token, references: & &1.id)\n      def get_user_account_by_token(token) do\n        # your logic ...\n      end\n\n      @decorate cache_evict(key: user.id)\n      def update_user_account(user, attrs) do\n        # your logic ...\n      end\n    end\n\nWith the option `:references`, we are indicating to the `cacheable`\ndecorator to store the user id (`& &1.id` - assuming the function returns a\nuser record) under the key `email` and the key `token`, and the user record\nitself under the user id, which is the referenced key. This time, instead of\nstoring the same object three times, the decorator will cache it only once\nunder the user ID, and the other entries will keep a reference to it. When\nthe functions `get_user_account_by_email/1` or `get_user_account_by_token/1`\nare executed, the decorator will automatically handle it; under the hood,\nit will fetch the referenced key given by `email` or `token` first, and\nthen get the user record under the referenced key.\n\nOn the other hand, in the eviction function `update_user_account/1`, since\nthe user record is stored only once under the user's ID, we could set the\noption `:key` to the user's ID, without specifying multiple keys like in the\nprevious case. However, there is a caveat: _\"the `cache_evict` decorator\ndoesn't evict the references automatically\"_. See the\n[\"CAVEATS\"](#cacheable/3-caveats) section below.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-referenced-keys"},{"type":"function","title":"The `match` function on references - Nebulex.Caching.Decorators.cacheable/3","doc":"The `cacheable` decorator also evaluates the `:match` option's function on\ncache key references to ensure consistency and correctness. Let's give an\nexample to understand what this is about.\n\nUsing the previous _\"user accounts\"_ example, here is the first call to\nfetch a user by email:\n\n    iex> user = MyApp.UserAccounts.get_user_account_by_email(\"me@test.com\")\n    #=> %MyApp.UserAccounts.User{id: 1, email: \"me@test.com\", ...}\n\nThe user is now available in the cache for subsequent calls. Now, let's\nsuppose we update the user's email by calling:\n\n    iex> MyApp.UserAccounts.update_user_account(user, %{\n    ...>   email: \"updated@test.com\", ...\n    ...> })\n    #=> %MyApp.UserAccounts.User{id: 1, email: \"updated@test.com\", ...}\n\nThe `update_user_account` function should have removed the user schema\nassociated with the `user.id` key (decorated with `cache_evict`) but not\nits references. Therefore, if we call `get_user_account_by_email` again:\n\n    iex> user = MyApp.UserAccounts.get_user_account_by_email(\"me@test.com\")\n    #=> %MyApp.UserAccounts.User{id: 1, email: \"updated@test.com\", ...}\n\nAnd here, we have an inconsistency because we are requesting a user with\nthe email `\"me@test.com\"` and we got a user with a different email\n`\"updated@test.com\"` (the updated one). How can we avoid this? The answer\nis to leverage the match function to ensure consistency and correctness.\nLet's provide a match function that helps us with it.\n\n    @decorate cacheable(\n                key: email,\n                references: & &1.id,\n                match: &match(&1, email)\n              )\n    def get_user_account_by_email(email) do\n      # your logic ...\n    end\n\n    defp match(%{email: email}, email), do: true\n    defp match(_, _), do: false\n\nWith the solution above, the `cacheable` decorator only caches the user's\nvalue if the email matches the one in the arguments. Otherwise, nothing is\ncached, and the decorator evaluates the function's block. Previously, the\ndecorator was caching the user regardless of the requested email value.\nWith this fix, if we try the previous call:\n\n    iex> MyApp.UserAccounts.get_user_account_by_email(\"me@test.com\")\n    #=> nil\n\nSince there is an email mismatch in the previous call, the decorator removes\nthe mismatch reference from the cache (eliminating the inconsistency) and\nexecutes the function body, assuming it uses `MyApp.Repo.get_by/2`, `nil`\nis returned because there is no such user in the database.\n\n> #### `:match` option {: .info}\n>\n> The `:match` option can and should be used when using references to allow\n> the decorator to remove inconsistent cache key references automatically.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-the-match-function-on-references"},{"type":"function","title":"External referenced keys - Nebulex.Caching.Decorators.cacheable/3","doc":"Previously, we saw how to work with referenced keys but on the same cache,\nlike \"internal references.\" Despite this being the typical case scenario,\nthere could be situations where you may want to reference a key stored in a\ndifferent or external cache. Why would I want to reference a key located in\na separate cache? There may be multiple reasons, but let's give a few\nexamples.\n\n  * One example is when you have a Redis cache; in such case, you likely\n    want to optimize the calls to Redis as much as possible. Therefore, you\n    should store the referenced keys in a local cache and the values in\n    Redis. This way, we only hit Redis to access the keys with the actual\n    values, and the decorator resolves the referenced keys locally.\n\n  * Another example is for keeping the cache key references isolated,\n    preferably locally. Then, apply a different eviction (or garbage\n    collection) policy for the references; one may want to expire the\n    references more often to avoid having dangling keys since the\n    `cache_evict` decorator doesn't remove the references automatically,\n    just the defined key (or keys). See the\n    [\"CAVEATS\"](#cacheable/3-caveats) section below.\n\nLet us modify the previous _\"user accounts\"_ example based on the Redis\nscenario:\n\n    defmodule MyApp.UserAccounts do\n      use Nebulex.Caching\n\n      alias MyApp.{LocalCache, RedisCache}\n\n      @decorate cacheable(cache: RedisCache, key: id)\n      def get_user_account(id) do\n        # your logic ...\n      end\n\n      @decorate cacheable(\n                  cache: LocalCache,\n                  key: email,\n                  references: &keyref(RedisCache, &1.id)\n                )\n      def get_user_account_by_email(email) do\n        # your logic ...\n      end\n\n      @decorate cacheable(\n                  cache: LocalCache,\n                  key: token,\n                  references: &keyref(RedisCache, &1.id)\n                )\n      def get_user_account_by_token(token) do\n        # your logic ...\n      end\n\n      @decorate cache_evict(cache: RedisCache, key: user.id)\n      def update_user_account(user) do\n        # your logic ...\n      end\n    end\n\nThe functions `get_user_account/1` and `update_user_account/2` use\n`RedisCache` to store the real value in Redis while\n`get_user_account_by_email/1` and `get_user_account_by_token/1` use\n`LocalCache` to store the cache key references. Then, with the option\n`references: &keyref(RedisCache, &1.id)` we are telling the `cacheable`\ndecorator the referenced key given by `&1.id` is located in the cache\n`RedisCache`; underneath, the macro [`keyref/2`](`Nebulex.Caching.keyref/2`)\nbuilds the particular return type for the external cache reference.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-external-referenced-keys"},{"type":"function","title":"Caveats about references - Nebulex.Caching.Decorators.cacheable/3","doc":"* When the `cache_evict` decorator annotates a key (or keys) to evict, the\n  decorator removes only the entry associated with that key. Therefore, if\n  the key has references, those are not automatically removed, which means\n  dangling keys. However, there are multiple ways to address dangling keys\n  (or references):\n\n  * The first one (perhaps the simplest one) sets a TTL to the reference.\n    For example:\n\n    ```elixir\n    @decorate cacheable(key: email, references: & &1.id, opts: [ttl: @ttl])\n    def get_user_by_email(email) do\n      # get the user from the database ...\n    end\n    ```\n\n    You could also specify a different TTL for the referenced key:\n\n    ```elixir\n    @decorate cacheable(\n                key: email,\n                references: &keyref(&1.id, ttl: @another_ttl),\n                opts: [ttl: @ttl]\n              )\n    def get_user_by_email(email) do\n      # get the user from the database ...\n    end\n    ```\n\n  * The second alternative, perhaps the recommended, is having a separate\n    cache to keep the references (e.g., a cache using the local adapter).\n    This way, you could provide a different eviction or GC configuration\n    to run the GC more often and keep the references cache clean. See\n    [\"External referenced keys\"](#cacheable/3-external-referenced-keys).\n\n  * The third alternative uses the `key: {:in, keys}` to specify a key and\n    its references. For example, if you have:\n\n    ```elixir\n    @decorate cacheable(key: email, references: & &1.id)\n    def get_user_by_email(email) do\n      # get the user from the database ...\n    end\n    ```\n\n    The eviction may look like this:\n\n    ```elixir\n    @decorate cache_evict(key: {:in, [user.id, user.email]})\n    def delete_user(user) do\n      # delete the user from the database ...\n    end\n    ```\n\n    This one is perhaps the least ideal option because it is cumbersome;\n    you have to know and specify the key and all its references, and at the\n    same time, you will need to have access to the key and references in the\n    arguments, which sometimes is not possible because you may receive only\n    the ID, but not the email.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-caveats-about-references"},{"type":"type","title":"Nebulex.Caching.Decorators.context/0","doc":"Proxy type to the decorator context","ref":"Nebulex.Caching.Decorators.html#t:context/0"},{"type":"function","title":"Nebulex.Caching.Decorators.default_match/1","doc":"Default match function.","ref":"Nebulex.Caching.Decorators.html#default_match/1"},{"type":"type","title":"Nebulex.Caching.Decorators.dynamic_cache/0","doc":"Type spec for a dynamic cache definition","ref":"Nebulex.Caching.Decorators.html#t:dynamic_cache/0"},{"type":"function","title":"Nebulex.Caching.Decorators.dynamic_cache_spec/2","doc":"A helper function to create a reserved tuple for a dynamic cache.\n\nThe first argument, `cache`, specifies the defined cache module,\nand the second argument, `name`, is the actual name of the cache.\n\nWhen creating a dynamic cache tuple form, use the macro\n`Nebulex.Caching.dynamic_cache/2` instead.","ref":"Nebulex.Caching.Decorators.html#dynamic_cache_spec/2"},{"type":"function","title":"Example - Nebulex.Caching.Decorators.dynamic_cache_spec/2","doc":"defmodule MyApp.Books do\n      use Nebulex.Caching\n\n      @decorate cacheable(cache: dynamic_cache(MyApp.Cache, :books))\n      def find_book(isbn) do\n        # your logic ...\n      end\n    end","ref":"Nebulex.Caching.Decorators.html#dynamic_cache_spec/2-example"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_cache/2","doc":"Convenience function for evaluating the `cache` argument.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#eval_cache/2"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_cache_evict/6","doc":"Convenience function for wrapping and/or encapsulating\nthe **cache_evict** decorator logic.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#eval_cache_evict/6"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_cache_put/6","doc":"Convenience function for wrapping and/or encapsulating\nthe **cache_put** decorator logic.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#eval_cache_put/6"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_cacheable/7","doc":"Convenience function for wrapping and/or encapsulating\nthe **cacheable** decorator logic.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#eval_cacheable/7"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_key/3","doc":"Convenience function for evaluating the `key` argument.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#eval_key/3"},{"type":"function","title":"Nebulex.Caching.Decorators.generate_key/1","doc":"Default key generation function.","ref":"Nebulex.Caching.Decorators.html#generate_key/1"},{"type":"type","title":"Nebulex.Caching.Decorators.key/0","doc":"The type for the `:key` option value.\n\nWhen defining the `:key` option on the decorated function,\nthe value can be:\n\n  * An anonymous function to call to generate the key in runtime.\n    The function optionally receives the decorator context as an argument\n    and must return the key for caching.\n  * The tuple `{:in, keys}`, where `keys` is a list with the keys to evict\n    or update. This option is allowed for `cache_evict` and `cache_put`\n    decorators only.\n  * Any term.","ref":"Nebulex.Caching.Decorators.html#t:key/0"},{"type":"type","title":"Nebulex.Caching.Decorators.keyref/0","doc":"Type for a key reference","ref":"Nebulex.Caching.Decorators.html#t:keyref/0"},{"type":"type","title":"Nebulex.Caching.Decorators.keyref_spec/0","doc":"Type for a key reference spec","ref":"Nebulex.Caching.Decorators.html#t:keyref_spec/0"},{"type":"function","title":"Nebulex.Caching.Decorators.keyref_spec/3","doc":"A helper function to create a reserved tuple for a reference.","ref":"Nebulex.Caching.Decorators.html#keyref_spec/3"},{"type":"function","title":"Arguments - Nebulex.Caching.Decorators.keyref_spec/3","doc":"* `cache` - The cache where the referenced key is stored. If it is `nil`,\n    the referenced key is looked up in the same cache provided via the\n    `:cache` option.\n  * `key` - The referenced key.\n  * `ttl` - The TTL for the referenced key. If configured, it overrides the\n    TTL given in the decorator's option `:opts`.\n\nWhen creating a reference tuple form, use the macro\n`Nebulex.Caching.keyref/2` instead.\n\nSee the [\"Referenced keys\"](#cacheable/3-referenced-keys) section in the\n`cacheable` decorator for more information.","ref":"Nebulex.Caching.Decorators.html#keyref_spec/3-arguments"},{"type":"type","title":"Nebulex.Caching.Decorators.match/0","doc":"Type for match function","ref":"Nebulex.Caching.Decorators.html#t:match/0"},{"type":"type","title":"Nebulex.Caching.Decorators.match_return/0","doc":"Type for the match function return","ref":"Nebulex.Caching.Decorators.html#t:match_return/0"},{"type":"type","title":"Nebulex.Caching.Decorators.on_error/0","doc":"Type for on_error action","ref":"Nebulex.Caching.Decorators.html#t:on_error/0"},{"type":"type","title":"Nebulex.Caching.Decorators.references/0","doc":"Type spec for the option `:references`.\n\nWhen defining the `:references` option on the decorated function,\nthe value can be:\n\n  * A reserved tuple that the type `t:keyref/0` defines. It must be created\n    using the macro [`keyref/2`](`Nebulex.Caching.keyref/2`).\n  * An anonymous function expects the result of the decorated function\n    evaluation as an argument. Alternatively, the decorator context can be\n    received as a second argument. It must return the referenced key, which\n    could be `t:keyref/0` or any term.\n  * `nil` means there are no key references (ignored).\n  * Any term.\n\nSee `cacheable/3` decorator for more information.","ref":"Nebulex.Caching.Decorators.html#t:references/0"},{"type":"function","title":"Nebulex.Caching.Decorators.run_cmd/4","doc":"Convenience function for running a cache command.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Caching.Decorators.html#run_cmd/4"},{"type":"module","title":"Nebulex.Caching.Decorators.Context","doc":"Decorator context.","ref":"Nebulex.Caching.Decorators.Context.html"},{"type":"type","title":"Nebulex.Caching.Decorators.Context.t/0","doc":"Decorator context type.\n\nThe decorator context defines the following keys:\n\n  * `:decorator` - Decorator's name.\n  * `:module` - The invoked module.\n  * `:function_name` - The invoked function name\n  * `:arity` - The arity of the invoked function.\n  * `:args` - The arguments that are given to the invoked function.","ref":"Nebulex.Caching.Decorators.Context.html#t:t/0"},{"type":"type","title":"Caveats about the `:args` - Nebulex.Caching.Decorators.Context.t/0","doc":"The following are some caveats about the context's `:args`\nto keep in mind:\n\n  * Only arguments explicitly assigned to a variable will be included.\n  * Ignored or underscored arguments will be ignored.\n  * Pattern-matching expressions without a variable assignment will be\n    ignored. Therefore, if there is a pattern-matching and you want to\n    include its value, it has to be explicitly assigned to a variable.\n\nFor example, suppose you have a module with a decorated function:\n\n    defmodule MyApp.SomeModule do\n      use Nebulex.Caching\n\n      alias MyApp.Cache\n\n      @decorate cacheable(cache: Cache, key: &__MODULE__.key_generator/1)\n      def get_something(x, _y, _, {_, _}, [_, _], %{a: a}, %{} = z) do\n        # Function's logic\n      end\n\n      def key_generator(context) do\n        # Key generation logic\n      end\n    end\n\nThe generator will be invoked like so:\n\n    key_generator(%Nebulex.Caching.Decorators.Context{\n      decorator: :cacheable,\n      module: MyApp.SomeModule,\n      function_name: :get_something,\n      arity: 7,\n      args: [x, z]\n    })\n\nAs you may notice, only the arguments `x` and `z` are included in the\ncontext args when calling the `key_generator/1` function.","ref":"Nebulex.Caching.Decorators.Context.html#t:t/0-caveats-about-the-args"},{"type":"behaviour","title":"Nebulex.Adapter","doc":"Specifies the minimal API required from adapters.","ref":"Nebulex.Adapter.html"},{"type":"macrocallback","title":"Nebulex.Adapter.__before_compile__/1","doc":"The callback invoked in case the adapter needs to inject code.","ref":"Nebulex.Adapter.html#c:__before_compile__/1"},{"type":"type","title":"Nebulex.Adapter.adapter_meta/0","doc":"The metadata returned by the adapter `c:init/1`.\n\nIt must be a map and Nebulex itself will always inject\nthe following keys into the meta:\n\n  * `:cache` - The defined cache module.\n  * `:name` - The name of the cache supervisor process.\n  * `:pid` - The PID returned by the child spec returned in `c:init/1`.\n  * `:adapter` - The defined cache adapter.\n  * `:telemetry` - Whether Telemetry is enabled or not.\n  * `:telemetry_prefix` – The Telemetry prefix.","ref":"Nebulex.Adapter.html#t:adapter_meta/0"},{"type":"macro","title":"Nebulex.Adapter.defcommand/2","doc":"Builds up a public wrapper function for invoking an adapter command.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Adapter.html#defcommand/2"},{"type":"macro","title":"Nebulex.Adapter.defcommandp/2","doc":"Builds up a private wrapper function for invoking an adapter command.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Adapter.html#defcommandp/2"},{"type":"function","title":"Nebulex.Adapter.handle_command_response/1","doc":"Helper function for handling a Nebulex command response.","ref":"Nebulex.Adapter.html#handle_command_response/1"},{"type":"function","title":"Examples - Nebulex.Adapter.handle_command_response/1","doc":"iex> Nebulex.Adapter.handle_command_response({:ok, \"ok\"})\n    {:ok, \"ok\"}\n\n    iex> Nebulex.Adapter.handle_command_response(:ok)\n    :ok\n\n    iex> Nebulex.Adapter.handle_command_response(\n    ...>   {:error, %Nebulex.Error{reason: :error}}\n    ...> )\n    {:error, %Nebulex.Error{reason: :error}}\n\n    iex> Nebulex.Adapter.handle_command_response({:error, :error})\n    {:error, %Nebulex.Error{reason: :error}}\n\n    iex> Nebulex.Adapter.handle_command_response({:error, %RuntimeError{}})\n    {:error, %Nebulex.Error{reason: %RuntimeError{}}}","ref":"Nebulex.Adapter.html#handle_command_response/1-examples"},{"type":"callback","title":"Nebulex.Adapter.init/1","doc":"Initializes the adapter supervision tree by returning the children\nand adapter metadata.","ref":"Nebulex.Adapter.html#c:init/1"},{"type":"function","title":"Nebulex.Adapter.lookup_meta/1","doc":"Returns the adapter metadata from its `c:init/1` callback.\n\nIt expects a process name of the cache. The name is either\nan atom or a PID. For a given cache, you often want to call\nthis function based on the dynamic cache:\n\n    Nebulex.Adapter.lookup_meta(cache.get_dynamic_cache())","ref":"Nebulex.Adapter.html#lookup_meta/1"},{"type":"type","title":"Nebulex.Adapter.nbx_error/0","doc":"Nebulex wrappable error types","ref":"Nebulex.Adapter.html#t:nbx_error/0"},{"type":"function","title":"Nebulex.Adapter.run_command/4","doc":"Convenience function for invoking the adapter running a command.\n\n**NOTE:** Internal purposes only.","ref":"Nebulex.Adapter.html#run_command/4"},{"type":"type","title":"Nebulex.Adapter.t/0","doc":"Adapter","ref":"Nebulex.Adapter.html#t:t/0"},{"type":"behaviour","title":"Nebulex.Adapter.Info","doc":"Specifies the adapter Info API.\n\nSee `Nebulex.Adapters.Common.Info`.","ref":"Nebulex.Adapter.Info.html"},{"type":"callback","title":"Nebulex.Adapter.Info.info/3","doc":"Returns `{:ok, info}` where `info` contains the requested cache information,\nas specified by the `spec`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nThe `spec` (information specification key) can be:\n\n  * **The atom `:all`**: returns a map with all information items.\n  * **An atom**: returns the value for the requested information item.\n  * **A list of atoms**: returns a map only with the requested information\n    items.\n\nThe adapters are free to add the information specification keys they want,\nhowever, Nebulex suggests the adapters add the following specs:\n\n  * `:server` - General information about the cache server (e.g., cache name,\n    adapter, PID, etc.).\n  * `:memory` - Memory consumption information (e.g., used memory,\n    allocated memory, etc.).\n  * `:stats` - Cache statistics (e.g., hits, misses, etc.).\n\nSee `c:Nebulex.Cache.info/2`.","ref":"Nebulex.Adapter.Info.html#c:info/3"},{"type":"behaviour","title":"Nebulex.Adapter.KV","doc":"Specifies the adapter Key/Value API.\n\nThis behaviour specifies all read/write key-based functions,\napplied to a specific cache key.","ref":"Nebulex.Adapter.KV.html"},{"type":"type","title":"Nebulex.Adapter.KV.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Nebulex.Adapter.KV.html#t:adapter_meta/0"},{"type":"callback","title":"Nebulex.Adapter.KV.delete/3","doc":"Deletes a single entry from cache.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nSee `c:Nebulex.Cache.delete/2`.","ref":"Nebulex.Adapter.KV.html#c:delete/3"},{"type":"type","title":"Nebulex.Adapter.KV.entries/0","doc":"Proxy type to the cache entries","ref":"Nebulex.Adapter.KV.html#t:entries/0"},{"type":"callback","title":"Nebulex.Adapter.KV.expire/4","doc":"Returns `{:ok, true}` if the given `key` exists and the new `ttl` is\nsuccessfully updated; otherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned; where `reason` is the cause of the error.\n\nSee `c:Nebulex.Cache.expire/3`.","ref":"Nebulex.Adapter.KV.html#c:expire/4"},{"type":"callback","title":"Nebulex.Adapter.KV.fetch/3","doc":"Fetches the value for a specific `key` in the cache.\n\nIf the cache contains the given `key`, then its value is returned\nin the shape of `{:ok, value}`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned. `reason` is the cause of the error and can be\n`Nebulex.KeyError` if the cache does not contain `key`,\n`Nebulex.Error` otherwise.\n\nSee `c:Nebulex.Cache.fetch/2`.","ref":"Nebulex.Adapter.KV.html#c:fetch/3"},{"type":"callback","title":"Nebulex.Adapter.KV.has_key?/3","doc":"Determines if the cache contains an entry for the specified `key`.\n\nMore formally, it returns `{:ok, true}` if the cache contains the given `key`.\nIf the cache doesn't contain `key`, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nSee `c:Nebulex.Cache.has_key?/2`.","ref":"Nebulex.Adapter.KV.html#c:has_key?/3"},{"type":"type","title":"Nebulex.Adapter.KV.keep_ttl/0","doc":"Keep TTL flag","ref":"Nebulex.Adapter.KV.html#t:keep_ttl/0"},{"type":"type","title":"Nebulex.Adapter.KV.key/0","doc":"Proxy type to the cache key","ref":"Nebulex.Adapter.KV.html#t:key/0"},{"type":"type","title":"Nebulex.Adapter.KV.on_write/0","doc":"Write command type","ref":"Nebulex.Adapter.KV.html#t:on_write/0"},{"type":"type","title":"Nebulex.Adapter.KV.opts/0","doc":"Proxy type to the cache options","ref":"Nebulex.Adapter.KV.html#t:opts/0"},{"type":"callback","title":"Nebulex.Adapter.KV.put/7","doc":"Puts the given `value` under `key` into the `cache`.\n\nReturns `{:ok, true}` if the `value` with key `key` is successfully inserted.\nOtherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nThe `ttl` argument defines the time-to-live for the stored entry. If it is not\nset, it means the entry doesn't have a time-to-live, so it shouldn't expire.\n\nThe `keep_ttl` argument tells whether to retain the time to live associated\nwith the key. Otherwise, the TTL is always overwritten.\n\n## `on_write` argument\n\nThe `on_write` argument supports the following values:\n\n  * `:put` - If the `key` already exists, it is overwritten. Any previous\n    time-to-live associated with the key is discarded on a successful `write`\n    operation.\n\n  * `:put_new` - It only stores the entry if the `key` does not exist.\n    Otherwise, `{:ok, false}` is returned.\n\n  * `:replace` - Alters the value stored under the given `key`, but only\n    if the key already exists in the cache. Otherwise, `{ok, false}` is\n    returned.\n\nSee `c:Nebulex.Cache.put/3`, `c:Nebulex.Cache.put_new/3`,\n`c:Nebulex.Cache.replace/3`.","ref":"Nebulex.Adapter.KV.html#c:put/7"},{"type":"callback","title":"Nebulex.Adapter.KV.put_all/5","doc":"Puts the given `entries` (key/value pairs) into the `cache` atomically\nor fail otherwise.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nThe `ttl` argument works the same as `c:put/7` but applies to all keys.\n\n## `on_write` argument\n\nThe `on_write` argument supports the following values:\n\n  * `:put` - If the `key` already exists, it is overwritten. Any previous\n    time-to-live associated with the key is discarded on a successful `write`\n    operation.\n\n  * `:put_new` - Insert all entries only if none exist in the cache, and it\n    returns `{:ok, true}` . Otherwise, `{:ok, false}` is returned.\n\nSee `c:Nebulex.Cache.put_all/2` and `c:Nebulex.Cache.put_new_all/2`.","ref":"Nebulex.Adapter.KV.html#c:put_all/5"},{"type":"callback","title":"Nebulex.Adapter.KV.take/3","doc":"Removes and returns the value associated with `key` in the cache.\n\nIf `key` is present in the cache, its value is removed and returned as\n`{:ok, value}`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned. `reason` is the cause of the error and can be\n`Nebulex.KeyError` if the cache does not contain `key` or\n`Nebulex.Error` otherwise.\n\nSee `c:Nebulex.Cache.take/2`.","ref":"Nebulex.Adapter.KV.html#c:take/3"},{"type":"callback","title":"Nebulex.Adapter.KV.touch/3","doc":"Returns `{:ok, true}` if the given `key` exists and the last access time is\nsuccessfully updated; otherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nSee `c:Nebulex.Cache.touch/2`.","ref":"Nebulex.Adapter.KV.html#c:touch/3"},{"type":"type","title":"Nebulex.Adapter.KV.ttl/0","doc":"TTL for a cache entry","ref":"Nebulex.Adapter.KV.html#t:ttl/0"},{"type":"callback","title":"Nebulex.Adapter.KV.ttl/3","doc":"Returns the remaining time-to-live for the given `key`.\n\nIf `key` is present in the cache, then its remaining TTL is returned\nin the shape of `{:ok, ttl}`.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned. `reason` is the cause of the error and can be\n`Nebulex.KeyError` if the cache does not contain `key`,\n`Nebulex.Error` otherwise.\n\nSee `c:Nebulex.Cache.ttl/2`.","ref":"Nebulex.Adapter.KV.html#c:ttl/3"},{"type":"callback","title":"Nebulex.Adapter.KV.update_counter/6","doc":"Updates the counter mapped to the given `key`.\n\nIf `amount` > 0, the counter is incremented by the given `amount`.\nIf `amount` < 0, the counter is decremented by the given `amount`.\nIf `amount` == 0, the counter is not updated.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nThe `ttl` is set when the key doesn't exist. Otherwise, only the counter is\nupdated keeping the TTL when the counter was updated for the first time.\n\nSee `c:Nebulex.Cache.incr/3`.\nSee `c:Nebulex.Cache.decr/3`.","ref":"Nebulex.Adapter.KV.html#c:update_counter/6"},{"type":"type","title":"Nebulex.Adapter.KV.value/0","doc":"Proxy type to the cache value","ref":"Nebulex.Adapter.KV.html#t:value/0"},{"type":"behaviour","title":"Nebulex.Adapter.Observable","doc":"Specifies the adapter Observable API.\n\nMaintains a registry of listeners and invokes them to handle cache events.","ref":"Nebulex.Adapter.Observable.html"},{"type":"behaviour","title":"Default implementation - Nebulex.Adapter.Observable","doc":"Nebulex provides a default implementation for the `Nebulex.Adapter.Observable`\nbehaviour, which uses a Telemetry handler to listen to cache command\ncompletion events, builds the corresponding cache entry event, and applies\nthe provided filter and listener.\n\nListeners should be implemented with care. In particular, it is important\nto consider their impact on performance and latency.\n\nListeners:\n\n  * are fired after the entry is mutated in the cache.\n  * block the calling process until the listener returns since the listener is\n    evaluated synchronously.\n\n> #### Function Captures {: .info}\n>\n> Due to how anonymous functions are implemented in the Erlang VM, it is best\n> to use function captures (`&Mod.fun/1`) as event listeners and filters\n> to achieve the best performance. In other words, avoid using literal\n> anonymous functions (`fn ... -> ... end`) or local function captures\n> (`&handle_event/1`) as event listeners and filters.","ref":"Nebulex.Adapter.Observable.html#module-default-implementation"},{"type":"type","title":"Nebulex.Adapter.Observable.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Nebulex.Adapter.Observable.html#t:adapter_meta/0"},{"type":"type","title":"Nebulex.Adapter.Observable.filter/0","doc":"Proxy type to a cache event filter","ref":"Nebulex.Adapter.Observable.html#t:filter/0"},{"type":"type","title":"Nebulex.Adapter.Observable.listener/0","doc":"Proxy type to a cache event listener","ref":"Nebulex.Adapter.Observable.html#t:listener/0"},{"type":"type","title":"Nebulex.Adapter.Observable.metadata/0","doc":"Proxy type to a cache event metadata","ref":"Nebulex.Adapter.Observable.html#t:metadata/0"},{"type":"type","title":"Nebulex.Adapter.Observable.opts/0","doc":"Proxy type to the cache options","ref":"Nebulex.Adapter.Observable.html#t:opts/0"},{"type":"callback","title":"Nebulex.Adapter.Observable.register_event_listener/5","doc":"Register a cache event listener.\n\nReturns `:ok` if successful; `{:error, reason}` otherwise.\n\nSee `c:Nebulex.Cache.register_event_listener/2`.","ref":"Nebulex.Adapter.Observable.html#c:register_event_listener/5"},{"type":"callback","title":"Nebulex.Adapter.Observable.unregister_event_listener/3","doc":"Un-register a cache event listener.\n\nReturns `:ok` if successful; `{:error, reason}` otherwise.\n\nSee `c:Nebulex.Cache.unregister_event_listener/2`.","ref":"Nebulex.Adapter.Observable.html#c:unregister_event_listener/3"},{"type":"behaviour","title":"Nebulex.Adapter.Queryable","doc":"Specifies the adapter Query API.","ref":"Nebulex.Adapter.Queryable.html"},{"type":"type","title":"Nebulex.Adapter.Queryable.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Nebulex.Adapter.Queryable.html#t:adapter_meta/0"},{"type":"callback","title":"Nebulex.Adapter.Queryable.execute/3","doc":"Executes a previously prepared query.\n\nThe `query_meta` field is a map containing some fields found in the query\noptions after they have been normalized. For example, the query value and\nthe selected entry fields to return can be found in the `query_meta`.\n\n Finally, `opts` is a keyword list of options given to the `Nebulex.Cache`\n operation that triggered the adapter call. Any option is allowed, as this\n is a mechanism to customize the adapter behavior per operation.\n\nThis callback returns:\n\n  * `{:ok, result}` - The query was successfully executed. The `result`\n    could be a list with the matched entries or its count.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause of the error.\n\nIt is used on `c:Nebulex.Cache.get_all/2`, `c:Nebulex.Cache.count_all/2`,\nand `c:Nebulex.Cache.delete_all/2`.","ref":"Nebulex.Adapter.Queryable.html#c:execute/3"},{"type":"type","title":"Nebulex.Adapter.Queryable.opts/0","doc":"Proxy type to the cache options","ref":"Nebulex.Adapter.Queryable.html#t:opts/0"},{"type":"type","title":"Nebulex.Adapter.Queryable.query_meta/0","doc":"Query metadata fields.","ref":"Nebulex.Adapter.Queryable.html#t:query_meta/0"},{"type":"type","title":"Operation - Nebulex.Adapter.Queryable.query_meta/0","doc":"The field `:op` defines the type of query operation. The possible values are:\n\n  * `:get_all` - Returns a list with all entries from cache matching the given\n    query.\n  * `:count_all` - Returns the number of matched entries with the given query.\n  * `:delete_all` - Deletes all entries matching the given query. It returns\n    the number of deleted entries.\n  * `:stream` - Similar to `:get_all` but returns a lazy enumerable that emits\n    all entries from the cache matching the given query.","ref":"Nebulex.Adapter.Queryable.html#t:query_meta/0-operation"},{"type":"type","title":"Query - Nebulex.Adapter.Queryable.query_meta/0","doc":"The field `:query` defines the entries in the cache to match. There are two\ntypes of query values. The ones recommended by Nebulex for all adapters to\nimplement (shared queries) and the adapter-specific ones.","ref":"Nebulex.Adapter.Queryable.html#t:query_meta/0-query"},{"type":"type","title":"Shared queries - Nebulex.Adapter.Queryable.query_meta/0","doc":"The following query values are the ones recommended by Nebulex\nfor all adapters to implement:\n\n  * `{:in, keys}` - Query the entries associated with the set of `keys`\n    requested.\n  * `{:q, nil}` - Query all the entries in the cache.","ref":"Nebulex.Adapter.Queryable.html#t:query_meta/0-shared-queries"},{"type":"type","title":"Adapter-specific queries - Nebulex.Adapter.Queryable.query_meta/0","doc":"For any other query the value must be:\n\n  * `{:q, query_value}` - Queries the entries in the cache matching with the\n    given `query_value`.\n\nThe `query_value` depends entirely on the adapter implementation, it could\nany term. Therefore, it is highly recommended to see adapters' documentation\nfor more information about building queries. For example, the\n`Nebulex.Adapters.Local` adapter supports\n[**\"ETS Match Spec\"**](https://www.erlang.org/doc/man/ets#match_spec).","ref":"Nebulex.Adapter.Queryable.html#t:query_meta/0-adapter-specific-queries"},{"type":"type","title":"Select fields to return - Nebulex.Adapter.Queryable.query_meta/0","doc":"The field `:select` defines the fields of the cached entry to return.\n\nSee [Nebulex.Cache.get_all/2 options][get_all_opts] for the possible values.\n\n[get_all_opts]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Cache.html#c:get_all/2-options","ref":"Nebulex.Adapter.Queryable.html#t:query_meta/0-select-fields-to-return"},{"type":"callback","title":"Nebulex.Adapter.Queryable.stream/3","doc":"Streams a previously prepared query.\n\nSee `c:execute/3` for a description of arguments.\n\nThis callback returns:\n\n  * `{:ok, stream}` - The query is valid, then the stream is returned.\n\n  * `{:error, reason}` - An error occurred executing the command.\n    `reason` is the cause of the error.\n\nSee `c:Nebulex.Cache.stream/2`.","ref":"Nebulex.Adapter.Queryable.html#c:stream/3"},{"type":"behaviour","title":"Nebulex.Adapter.Transaction","doc":"Specifies the adapter Transaction API.","ref":"Nebulex.Adapter.Transaction.html"},{"type":"behaviour","title":"Default implementation - Nebulex.Adapter.Transaction","doc":"This module also provides a default implementation which uses the Erlang\nlibrary `:global`.\n\nThis implementation accepts the following options:\n\n* `:keys` (list of `t:term/0`) - The list of keys the transaction will lock. Since the lock ID is generated\n  based on the key, the transaction uses a fixed lock ID if the option is\n  not provided or is an empty list. Then, all subsequent transactions\n  without this option (or set to an empty list) are serialized, and\n  performance is significantly affected. For that reason, it is recommended\n  to pass the list of keys involved in the transaction. The default value is `[]`.\n\n* `:nodes` (list of `t:atom/0`) - The list of the nodes where to set the lock.\n\n  The default value is `[node()]`.\n\n* `:retries` (`:infinity` | `t:non_neg_integer/0`) - If the key has already been locked by another process and retries are not\n  equal to 0, the process sleeps for a while and tries to execute the action\n  later. When `:retries` attempts have been made, an exception is raised. If\n  `:retries` is `:infinity` (the default), the function will eventually be\n  executed (unless the lock is never released). The default value is `:infinity`.\n\n\n\nLet's see an example:\n\n    MyCache.transaction fn ->\n      counter = MyCache.get(:counter)\n      MyCache.set(:counter, counter + 1)\n    end\n\nLocking only the involved key (recommended):\n\n    MyCache.transaction(\n      fn ->\n        counter = MyCache.get(:counter)\n        MyCache.set(:counter, counter + 1)\n      end,\n      [keys: [:counter]]\n    )\n\n    MyCache.transaction(\n      fn ->\n        alice = MyCache.get(:alice)\n        bob = MyCache.get(:bob)\n        MyCache.set(:alice, %{alice | balance: alice.balance + 100})\n        MyCache.set(:bob, %{bob | balance: bob.balance + 100})\n      end,\n      [keys: [:alice, :bob]]\n    )","ref":"Nebulex.Adapter.Transaction.html#module-default-implementation"},{"type":"callback","title":"Nebulex.Adapter.Transaction.in_transaction?/2","doc":"Returns `{:ok, true}` if the current process is inside a transaction;\notherwise, `{:ok, false}` is returned.\n\nIf there's an error with executing the command, `{:error, reason}`\nis returned, where `reason` is the cause of the error.\n\nSee `c:Nebulex.Cache.in_transaction?/1`.","ref":"Nebulex.Adapter.Transaction.html#c:in_transaction?/2"},{"type":"callback","title":"Nebulex.Adapter.Transaction.transaction/3","doc":"Runs the given function inside a transaction.\n\nIf an Elixir exception occurs, the exception will bubble up from the\ntransaction function. If the cache aborts the transaction, it returns\n`{:error, reason}`.\n\nA successful transaction returns the value returned by the function wrapped\nin a tuple as `{:ok, value}`.\n\nSee `c:Nebulex.Cache.transaction/2`.","ref":"Nebulex.Adapter.Transaction.html#c:transaction/3"},{"type":"module","title":"Nebulex.Adapters.Nil","doc":"The Nil adapter is a special cache adapter that turns off the cache. It loses\nall the items saved on it and returns nil for all read operations and true for\nall save operations. This adapter is mostly useful for tests.","ref":"Nebulex.Adapters.Nil.html"},{"type":"module","title":"Shared options - Nebulex.Adapters.Nil","doc":"All of the cache functions accept the following options\nwhen using this adapter:\n\n* `:before` (function of arity 0) - Pre-hook function. It calls the given function before the cache command\n  is executed.\n\n* `:after_return` (function of arity 1) - Post-hook function. It calls the given function after the cache command\n  is executed, and the returned result is passed as an argument.","ref":"Nebulex.Adapters.Nil.html#module-shared-options"},{"type":"module","title":"Example - Nebulex.Adapters.Nil","doc":"Suppose you have an application using Ecto for database access and Nebulex\nfor caching. Then, you have defined a cache and a repo within it. Since you\nare using a database, there might be some cases where you may want to turn\noff the cache to avoid issues when running the test. For example, in some\ntest cases, when accessing the database, you expect no data, but you can\nstill get unexpected data since the cache is not flushed. Therefore, you\nmust delete all entries from the cache before running each test to ensure\nthe cache is always empty. Here is where the Nil adapter comes in, instead\nof adding code to flush the cache before each test, you could define a test\ncache using the Nil adapter for the tests.\n\nOn one hand, you have defined the cache in your application within\n`lib/my_app/cache.ex`:\n\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nOn the other hand, in the tests, you have defined the test cache within\n`test/support/test_cache.ex`:\n\n    defmodule MyApp.TestCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Nil\n    end\n\nYou must tell the app what cache to use depending on the environment. For\ntests, you configure `MyApp.TestCache`; otherwise, it is always `MyApp.Cache`.\nYou can do this by introducing a new config parameter to decide which cache\nmodule to use. For tests, you can define the config `config/test.exs`:\n\n    config :my_app,\n      nebulex_cache: MyApp.TestCache,\n      ...\n\nThe final piece is to read the config parameter and start the cache properly.\nWithin `lib/my_app/application.ex`, you could have:\n\n    def start(_type, _args) do\n      children = [\n        {Application.get_env(:my_app, :nebulex_cache, MyApp.Cache), []},\n      ]\n\n      ...\n\nAs you may notice, `MyApp.Cache` is used by default unless the\n`:nebulex_cache` option points to a different module, which will\nbe when tests are executed (`test` env).","ref":"Nebulex.Adapters.Nil.html#module-example"},{"type":"module","title":"Nebulex.Adapters.Nil.Options","doc":"Options for the `Nebulex.Adapters.Nil` adapter.","ref":"Nebulex.Adapters.Nil.Options.html"},{"type":"function","title":"Nebulex.Adapters.Nil.Options.runtime_shared_options_docs/0","doc":"","ref":"Nebulex.Adapters.Nil.Options.html#runtime_shared_options_docs/0"},{"type":"function","title":"Nebulex.Adapters.Nil.Options.validate_runtime_shared_opts!/1","doc":"","ref":"Nebulex.Adapters.Nil.Options.html#validate_runtime_shared_opts!/1"},{"type":"module","title":"Nebulex.Adapters.Common.Info","doc":"A simple/default implementation for `Nebulex.Adapter.Info` behaviour.\n\nThe implementation defines the following information specifications:\n\n  * `:server` - A map with general information about the cache server.\n    Includes the following keys:\n\n    * `:nbx_version` - The Nebulex version.\n    * `:cache_module` - The defined cache module.\n    * `:cache_adapter` - The cache adapter.\n    * `:cache_name` - The cache name.\n    * `:cache_pid` - The cache PID.\n\n  * `:stats` - A map with the cache statistics keys, as specified by\n    `Nebulex.Adapters.Common.Info.Stats`.\n\nThe info data will look like this:\n\n    %{\n      server: %{\n        nbx_version: \"3.0.0\",\n        cache_module: \"MyCache\",\n        cache_adapter: \"Nebulex.Adapters.Local\",\n        cache_name: \"MyCache\",\n        cache_pid: #PID<0.111.0>\n      },\n      stats: %{\n        deletions: 0,\n        evictions: 0,\n        expirations: 0,\n        hits: 0,\n        misses: 0,\n        updates: 0,\n        writes: 0\n      }\n    }","ref":"Nebulex.Adapters.Common.Info.html"},{"type":"module","title":"Nebulex.Adapters.Common.Info.Stats","doc":"Stats implementation using [Erlang counters][erl_counters].\n\nAdapters are directly responsible for implementing the `Nebulex.Adapter.Info`\nbehaviour and adding an info spec for stats. However, this module provides a\nsimple implementation for stats using [Erlang counters][erl_counters].\n\nAn info specification `stats` is added to the info data, which is a map\nwith the following keys or measurements:\n\n  * `:hits` - The requested data is successfully retrieved from the cache.\n\n  * `:misses` - When a system or application makes a request to retrieve\n    data from a cache, but that specific data is not currently in cache\n    memory. A cache miss occurs either because the data was never placed\n    in the cache, or because the data was removed (“evicted”) from the\n    cache by either the caching system itself or an external application\n    that specifically made that eviction request.\n\n  * `:evictions` - Eviction by the caching system itself occurs when\n    space needs to be freed up to add new data to the cache, or if\n    the time-to-live policy on the data expired.\n\n  * `:expirations` - When the time-to-live policy on the data expired.\n\n  * `:updates` - When existing data is successfully updated.\n\n  * `:writes` - When data is inserted or overwritten.\n\n  * `:deletions` - The data was intentionally removed by either the\n    caching system or an external application that specifically made\n    that deletion request.\n\nSee the `Nebulex.Adapters.Local` adapter and `Nebulex.Adapters.Common.Info`\nfor more information about the usage.\n\n[erl_counters]: https://erlang.org/doc/man/counters.html","ref":"Nebulex.Adapters.Common.Info.Stats.html"},{"type":"function","title":"Nebulex.Adapters.Common.Info.Stats.count/1","doc":"Returns a map with all counters/stats count.","ref":"Nebulex.Adapters.Common.Info.Stats.html#count/1"},{"type":"function","title":"Examples - Nebulex.Adapters.Common.Info.Stats.count/1","doc":"Nebulex.Adapters.Common.Info.Stats.count(stats_counter)","ref":"Nebulex.Adapters.Common.Info.Stats.html#count/1-examples"},{"type":"function","title":"Nebulex.Adapters.Common.Info.Stats.count/2","doc":"Returns the current count for the stats counter given by `stat`.","ref":"Nebulex.Adapters.Common.Info.Stats.html#count/2"},{"type":"function","title":"Examples - Nebulex.Adapters.Common.Info.Stats.count/2","doc":"Nebulex.Adapters.Common.Info.Stats.count(stats_counter, :hits)","ref":"Nebulex.Adapters.Common.Info.Stats.html#count/2-examples"},{"type":"type","title":"Nebulex.Adapters.Common.Info.Stats.counter/0","doc":"Type for the counter","ref":"Nebulex.Adapters.Common.Info.Stats.html#t:counter/0"},{"type":"function","title":"Nebulex.Adapters.Common.Info.Stats.incr/3","doc":"Increments counter(s) for the given stat(s) by `incr`.","ref":"Nebulex.Adapters.Common.Info.Stats.html#incr/3"},{"type":"function","title":"Examples - Nebulex.Adapters.Common.Info.Stats.incr/3","doc":"Nebulex.Adapters.Common.Info.Stats.incr(stats_counter, :hits)\n\n    Nebulex.Adapters.Common.Info.Stats.incr(stats_counter, :writes, 10)\n\n    Nebulex.Adapters.Common.Info.Stats.incr(stats_counter, [:misses, :deletions])","ref":"Nebulex.Adapters.Common.Info.Stats.html#incr/3-examples"},{"type":"function","title":"Nebulex.Adapters.Common.Info.Stats.init/1","doc":"Returns the Erlang's counter to be used by the adapter for keeping the cache\nstats. It also initiates the Telemetry handler for handling and/or updating\nthe cache stats in runtime under the hood.\n\nAny adapter using `Nebulex.Adapters.Common.Info` implementation must call\nthis init function in the `c:Nebulex.Adapter.init/1` callback and include\nthe returned counter within the adapter metadata under the key\n`:stats_counter`. See the `Nebulex.Adapters.Nil` for example.","ref":"Nebulex.Adapters.Common.Info.Stats.html#init/1"},{"type":"function","title":"Example - Nebulex.Adapters.Common.Info.Stats.init/1","doc":"Nebulex.Adapters.Common.Info.Stats.init([:telemetry, :prefix])","ref":"Nebulex.Adapters.Common.Info.Stats.html#init/1-example"},{"type":"function","title":"Nebulex.Adapters.Common.Info.Stats.new/0","doc":"Convenience function for returning a map with all stats set to `0`.","ref":"Nebulex.Adapters.Common.Info.Stats.html#new/0"},{"type":"type","title":"Nebulex.Adapters.Common.Info.Stats.stat/0","doc":"The stat type","ref":"Nebulex.Adapters.Common.Info.Stats.html#t:stat/0"},{"type":"type","title":"Nebulex.Adapters.Common.Info.Stats.stats/0","doc":"Stats type","ref":"Nebulex.Adapters.Common.Info.Stats.html#t:stats/0"},{"type":"module","title":"Nebulex.Telemetry.CacheEntryHandler","doc":"A Telemetry handler listens to cache command completion events, builds the\ncorresponding cache entry event, and applies the provided filter and listener.","ref":"Nebulex.Telemetry.CacheEntryHandler.html"},{"type":"function","title":"Nebulex.Telemetry.CacheEntryHandler.register/5","doc":"Registers a new Telemetry handler for listening and handling cache entry\nevents.","ref":"Nebulex.Telemetry.CacheEntryHandler.html#register/5"},{"type":"function","title":"Nebulex.Telemetry.CacheEntryHandler.unregister/2","doc":"Un-registers a Telemetry handler that listens and handles cache entry events.","ref":"Nebulex.Telemetry.CacheEntryHandler.html#unregister/2"},{"type":"module","title":"Nebulex.Telemetry.CacheStatsCounterHandler","doc":"Telemetry handler for aggregating cache stats; it relies on the default\n`Nebulex.Adapters.Common.Info` implementation based on Erlang counters.\nSee `Nebulex.Adapters.Common.Info.Stats`.","ref":"Nebulex.Telemetry.CacheStatsCounterHandler.html"},{"type":"function","title":"Nebulex.Telemetry.CacheStatsCounterHandler.attach/2","doc":"Attach a new Telemetry handler for handling cache stats.","ref":"Nebulex.Telemetry.CacheStatsCounterHandler.html#attach/2"},{"type":"module","title":"Nebulex.Time","doc":"Time utilities.","ref":"Nebulex.Time.html"},{"type":"function","title":"Nebulex.Time.now/1","doc":"Returns the current system time in the given time unit.\n\nThe `unit` is set to `:millisecond` by default.","ref":"Nebulex.Time.html#now/1"},{"type":"function","title":"Examples - Nebulex.Time.now/1","doc":"iex> now = Nebulex.Time.now()\n    iex> is_integer(now) and now > 0\n    true","ref":"Nebulex.Time.html#now/1-examples"},{"type":"module","title":"Nebulex.Utils","doc":"General purpose utilities.","ref":"Nebulex.Utils.html"},{"type":"function","title":"Nebulex.Utils.camelize_and_concat/1","doc":"Concatenates a list of \"camelized\" aliases and returns a new alias.\n\nIt handles binaries, atoms, and numbers.","ref":"Nebulex.Utils.html#camelize_and_concat/1"},{"type":"function","title":"Examples - Nebulex.Utils.camelize_and_concat/1","doc":"iex> Nebulex.Utils.camelize_and_concat([Foo, :bar])\n    Foo.Bar\n\n    iex> Nebulex.Utils.camelize_and_concat([Foo, \"bar\"])\n    Foo.Bar\n\n    iex> Nebulex.Utils.camelize_and_concat([Foo, \"Bar\", 1])\n    :\"Elixir.Foo.Bar.1\"","ref":"Nebulex.Utils.html#camelize_and_concat/1-examples"},{"type":"function","title":"Nebulex.Utils.get_option/5","doc":"A wrapper for `Keyword.get/3` but validates the returned value invoking\nthe function `valid?`.\n\nRaises an `ArgumentError` in case the validation fails.","ref":"Nebulex.Utils.html#get_option/5"},{"type":"function","title":"Examples - Nebulex.Utils.get_option/5","doc":"iex> Nebulex.Utils.get_option(\n    ...>   [keys: [1, 2, 3]],\n    ...>   :keys,\n    ...>   \"a list with at least one element\",\n    ...>   &((is_list(&1) and length(&1) > 0) or is_nil(&1))\n    ...> )\n    [1, 2, 3]\n\n    iex> Nebulex.Utils.get_option(\n    ...>   [],\n    ...>   :keys,\n    ...>   \"a list with at least one element\",\n    ...>   &((is_list(&1) and length(&1) > 0) or is_nil(&1))\n    ...> )\n    nil\n\n    iex> Nebulex.Utils.get_option(\n    ...>   [],\n    ...>   :keys,\n    ...>   \"a list with at least one element\",\n    ...>   &((is_list(&1) and length(&1) > 0) or is_nil(&1)),\n    ...>   fn -> \"default\" end\n    ...> )\n    \"default\"\n\n    iex> Nebulex.Utils.get_option(\n    ...>   [keys: 123],\n    ...>   :keys,\n    ...>   \"non empty list\",\n    ...>   &((is_list(&1) and length(&1) > 0) or is_nil(&1))\n    ...> )\n    ** (ArgumentError) invalid value for :keys option: expected non empty list, got: 123","ref":"Nebulex.Utils.html#get_option/5-examples"},{"type":"macro","title":"Nebulex.Utils.is_nebulex_exception/1","doc":"Convenience guard to determine whether the given argument `e` is a Nebulex\nexception or not.","ref":"Nebulex.Utils.html#is_nebulex_exception/1"},{"type":"macro","title":"Example - Nebulex.Utils.is_nebulex_exception/1","doc":"iex> import Nebulex.Utils, only: [is_nebulex_exception: 1]\n    iex> is_nebulex_exception(%Nebulex.Error{reason: :error})\n    true\n    iex> is_nebulex_exception(%{})\n    false","ref":"Nebulex.Utils.html#is_nebulex_exception/1-example"},{"type":"macro","title":"Nebulex.Utils.is_nebulex_wrappable_exception/1","doc":"Convenience guard to determine whether the given argument `e` is a Nebulex\nwrappable exception.","ref":"Nebulex.Utils.html#is_nebulex_wrappable_exception/1"},{"type":"macro","title":"Example - Nebulex.Utils.is_nebulex_wrappable_exception/1","doc":"iex> import Nebulex.Utils, only: [is_nebulex_wrappable_exception: 1]\n    iex> is_nebulex_wrappable_exception(%Nebulex.Error{reason: :error})\n    true\n    iex> is_nebulex_wrappable_exception(%Nebulex.KeyError{reason: :error})\n    true\n    iex> is_nebulex_wrappable_exception(%Nebulex.CacheNotFoundError{})\n    false\n    iex> is_nebulex_wrappable_exception(%Nebulex.QueryError{})\n    false\n    iex> is_nebulex_wrappable_exception(%{})\n    false","ref":"Nebulex.Utils.html#is_nebulex_wrappable_exception/1-example"},{"type":"macro","title":"Nebulex.Utils.is_timeout/1","doc":"Custom guard to validate whether the given `value` is a timeout.","ref":"Nebulex.Utils.html#is_timeout/1"},{"type":"macro","title":"Examples - Nebulex.Utils.is_timeout/1","doc":"iex> import Nebulex.Utils, only: [is_timeout: 1]\n    iex> is_timeout(1)\n    true\n    iex> is_timeout(:infinity)\n    true\n    iex> is_timeout(-1)\n    false\n    iex> is_timeout(1.0)\n    false\n    iex> is_timeout(\"\")\n    false\n    iex> is_timeout(nil)\n    false","ref":"Nebulex.Utils.html#is_timeout/1-examples"},{"type":"function","title":"Nebulex.Utils.module_behaviours/1","doc":"Returns the implemented behaviours for the given `module`.","ref":"Nebulex.Utils.html#module_behaviours/1"},{"type":"macro","title":"Nebulex.Utils.unwrap_or_raise/1","doc":"Convenience macro for unwrapping a function call result and deciding whether\nto raise an exception or return the unwrapped value.","ref":"Nebulex.Utils.html#unwrap_or_raise/1"},{"type":"macro","title":"Example - Nebulex.Utils.unwrap_or_raise/1","doc":"iex> import Nebulex.Utils\n    iex> unwrap_or_raise {:ok, \"ok\"}\n    \"ok\"\n    iex> unwrap_or_raise {:error, %Nebulex.Error{reason: :error}}\n    ** (Nebulex.Error) command failed with reason: :error\n    iex> unwrap_or_raise :other\n    :other","ref":"Nebulex.Utils.html#unwrap_or_raise/1-example"},{"type":"macro","title":"Nebulex.Utils.wrap_error/2","doc":"Convenience macro for wrapping the given `exception` into a tuple in the\nshape of `{:error, exception}`.","ref":"Nebulex.Utils.html#wrap_error/2"},{"type":"macro","title":"Example - Nebulex.Utils.wrap_error/2","doc":"iex> import Nebulex.Utils\n    iex> wrap_error Nebulex.Error, reason: :error\n    {:error, %Nebulex.Error{reason: :error}}","ref":"Nebulex.Utils.html#wrap_error/2-example"},{"type":"macro","title":"Nebulex.Utils.wrap_ok/1","doc":"Convenience macro for wrapping the given `call` result into a tuple in the\nshape of `{:ok, result}`.","ref":"Nebulex.Utils.html#wrap_ok/1"},{"type":"macro","title":"Example - Nebulex.Utils.wrap_ok/1","doc":"iex> import Nebulex.Utils\n    iex> wrap_ok \"hello\"\n    {:ok, \"hello\"}","ref":"Nebulex.Utils.html#wrap_ok/1-example"},{"type":"exception","title":"Nebulex.CacheNotFoundError","doc":"It is raised when the cache cannot be retrieved from the registry\nbecause it was not started or does not exist.","ref":"Nebulex.CacheNotFoundError.html"},{"type":"type","title":"Nebulex.CacheNotFoundError.t/0","doc":"The type for this exception struct.\n\nThis exception has the following public fields:\n\n  * `:message` - the error message.\n\n  * `:cache` - the cache name or its PID.","ref":"Nebulex.CacheNotFoundError.html#t:t/0"},{"type":"exception","title":"Nebulex.Error","doc":"This exception represents command execution errors. For example, the cache\ncannot perform a command because it has not started, it does not exist, or\nthe adapter failed to perform it for any reason.","ref":"Nebulex.Error.html"},{"type":"exception","title":"Exception fields - Nebulex.Error","doc":"See `t:t/0`.","ref":"Nebulex.Error.html#module-exception-fields"},{"type":"exception","title":"Error reasons - Nebulex.Error","doc":"The `:reason` field can assume the following values:\n\n  * `:timeout` - if there is a timeout when executing the cache command.\n\n  * `:transaction_aborted` - if a transaction execution fails and aborts.\n\n  * `:event_listener_already_exists` - if another cache entry listener with\n    the same ID already exists.\n\n  * `:event_listener_error` - if a cache entry event listener fails when\n    processing an event.\n\n  * `t:Exception.t/0` - if the underlying adapter fails due to an exception.\n\n  * `t:any/0` - the command fails with an adapter-specific error.","ref":"Nebulex.Error.html#module-error-reasons"},{"type":"function","title":"Nebulex.Error.format_error/2","doc":"A callback invoked when a custom formatter module is provided.","ref":"Nebulex.Error.html#format_error/2"},{"type":"function","title":"Arguments - Nebulex.Error.format_error/2","doc":"* `reason` - the error reason.\n  * `metadata` - a keyword with the metadata given to the exception.\n\nFor example, if an adapter returns:\n\n    wrap_error Nebulex.Error,\n      reason: :my_reason,\n      module: MyAdapter.Formatter,\n      foo: :bar\n\nthe exception invokes:\n\n    MyAdapter.Formatter.format_error(:my_reason, foo: :bar)","ref":"Nebulex.Error.html#format_error/2-arguments"},{"type":"function","title":"Nebulex.Error.maybe_format_metadata/2","doc":"Formats the error metadata when not empty.","ref":"Nebulex.Error.html#maybe_format_metadata/2"},{"type":"type","title":"Nebulex.Error.reason/0","doc":"Error reason type","ref":"Nebulex.Error.html#t:reason/0"},{"type":"type","title":"Nebulex.Error.t/0","doc":"The type for this exception struct.\n\nThis exception has the following public fields:\n\n  * `:reason` - the error reason. It can be one of the Nebulex-specific\n    reasons described in the [\"Error reasons\"](#module-error-reasons)\n    section in the module documentation.\n\n  * `:module` - a custom error formatter module. When it is present, it\n    invokes `module.format_error(reason, metadata)` to format the error\n    reason. The argument `metadata` is a keyword with the metadata given\n    to the exception. See `format_error/2` for more information.\n\n  * `:metadata` - the metadata contains the options given to the\n    exception excluding the `:reason` and `:module` that are part\n    of the exception fields. For example, when raising an exception\n    `raise Nebulex.Error, reason: :test, foo: :bar`, the metadata\n    will be `[foo: :bar]`.","ref":"Nebulex.Error.html#t:t/0"},{"type":"exception","title":"Nebulex.KeyError","doc":"Raised at runtime when a key does not exist in the cache.\n\nThis exception denotes the cache executed a command, but there was an issue\nwith the requested key; for example, it doesn't exist.","ref":"Nebulex.KeyError.html"},{"type":"exception","title":"Exception fields - Nebulex.KeyError","doc":"See `t:t/0`.","ref":"Nebulex.KeyError.html#module-exception-fields"},{"type":"exception","title":"Error reasons - Nebulex.KeyError","doc":"The `:reason` field can assume a few Nebulex-specific values:\n\n  * `:not_found` - the key doesn't exist in the cache.\n\n  * `:expired` - The key doesn't exist in the cache because it is expired.","ref":"Nebulex.KeyError.html#module-error-reasons"},{"type":"type","title":"Nebulex.KeyError.t/0","doc":"The type for this exception struct.\n\nThis exception has the following public fields:\n\n  * `:reason` - the error reason. The two possible reasons are `:not_found`\n    or `:expired`. Defaults to `:not_found`.\n\n  * `:key` - the requested key.\n\n  * `:metadata` - the metadata contains the options given to the\n    exception excluding the `:reason` and `:key` that are part of\n    the exception fields. For example, when raising an exception\n    `raise Nebulex.KeyError, key: :test, foo: :bar`, the metadata\n    will be `[foo: :bar]`.","ref":"Nebulex.KeyError.html#t:t/0"},{"type":"exception","title":"Nebulex.QueryError","doc":"Raised at runtime when the query is invalid.","ref":"Nebulex.QueryError.html"},{"type":"type","title":"Nebulex.QueryError.t/0","doc":"The type for this exception struct.\n\nThis exception has the following public fields:\n\n  * `:message` - the error message.\n\n  * `:query` - the query value.","ref":"Nebulex.QueryError.html#t:t/0"},{"type":"extras","title":"Getting started","doc":"# Getting started\n\nThis guide is an introduction to [Nebulex][nbx_repo], a local and distributed\ncaching toolkit for Elixir. Nebulex API is pretty much inspired by\n[Ecto](https://github.com/elixir-ecto/ecto), taking advantage of its simplicity,\nflexibility and pluggable architecture. Same as Ecto, developers can provide\ntheir own cache (adapter) implementations. In this guide, we're going to learn\nsome basics about Nebulex, such as write, read and delete cache entries.\n\n[nbx_repo]: https://github.com/elixir-nebulex/nebulex","ref":"getting-started.html"},{"type":"extras","title":"Adding Nebulex to an application - Getting started","doc":"Let's start creating a new Elixir application by running this command:\n\n```\nmix new blog --sup\n```\n\nThe `--sup` option ensures that this application has\n[a supervision tree](https://hexdocs.pm/elixir/supervisor-and-application.html),\nwhich will be needed by Nebulex later on.\n\nTo add Nebulex to this application, there are a few steps that we need to take.\n\nThe first step will be adding both Nebulex and the cache adapter as a dependency\nto our `mix.exs` file, which we'll do by changing the `deps` definition in that\nfile to this:\n\n```elixir\ndefp deps do\n  [\n    {:nebulex, \"~> 3.0\"},\n    # Use the official local cache adapter\n    {:nebulex_local, \"~> 3.0\"},\n    # Required for caching decorators (recommended)\n    {:decorator, \"~> 1.4\"},\n    # Required for telemetry events (recommended)\n    {:telemetry, \"~> 1.0\"},\n    # Required for :shards backend in local adapter\n    {:shards, \"~> 1.1\"}\n  ]\nend\n```\n\nTo give more flexibility and load only needed dependencies, Nebulex makes all\ndependencies optional, including the adapters. For example:\n\n  * For enabling [declarative decorator-based caching][nbx_caching], you have\n    to add `:decorator` to the dependency list.\n\n  * For enabling Telemetry events dispatched by Nebulex, you have to add\n    `:telemetry` to the dependency list. See [telemetry guide][telemetry].\n\n  * For intensive workloads when using `Nebulex.Adapters.Local` adapter, you may\n    want to use `:shards` as the backend for partitioned ETS tables. In such a\n    case, you have to add `:shards` to the dependency list.\n\n[nbx_caching]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Caching.html\n[telemetry]: http://hexdocs.pm/nebulex/3.0.0-rc.1/telemetry.html\n\nTo install these dependencies, we will run this command:\n\n```\nmix deps.get\n```\n\nWe now need to define a Cache and setup some configuration for Nebulex so that\nwe can perform actions on a cache from within the application's code.\n\nWe can set up this configuration by running this command:\n\n```\nmix nbx.gen.cache -c Blog.Cache\n```\n\nThis command will generate the configuration required to use the cache. The\nfirst bit of configuration is in `config/config.exs`:\n\n```elixir\nconfig :blog, Blog.Cache,\n  # Sets :shards as backend (defaults to :ets)\n  # backend: :shards,\n  # GC interval for pushing a new generation (e.g., 12 hrs)\n  gc_interval: :timer.hours(12),\n  # Max number of entries (e.g, 1 million)\n  max_size: 1_000_000,\n  # Max memory size in bytes (e.g., 2GB)\n  allocated_memory: 2_000_000_000,\n  # GC interval for checking memory and maybe evict entries (e.g., 10 sec)\n  gc_memory_check_interval: :timer.seconds(10)\n```\n\nIf you want to use `:shards` as backend, uncomment the `backend:` option.\n\n> See adapters docs for more information.\n\nAnd the `Blog.Cache` module is defined in `lib/blog/cache.ex` by our\n`mix nbx.gen.cache` command:\n\n```elixir\ndefmodule Blog.Cache do\n  use Nebulex.Cache,\n    otp_app: :blog,\n    adapter: Nebulex.Adapters.Local\nend\n```\n\nThis module is what we'll be using to interact with the cache. It uses the\n`Nebulex.Cache` module and it expects the `:otp_app` as option. The `otp_app`\ntells Nebulex which Elixir application it can look for cache configuration in.\nIn this case, we've specified that it is the `:blog` application where Nebulex\ncan find that configuration and so Nebulex will use the configuration that was\nset up in `config/config.exs`.\n\nThe final piece of configuration is to setup the `Blog.Cache` as a\nsupervisor within the application's supervision tree, which we can do in\n`lib/blog/application.ex`, inside the `start/2` function:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Blog.Cache\n  ]\n\n  ...\n```\n\nThis piece of configuration will start the Nebulex process which receives and\nexecutes our application's commands. Without it, we wouldn't be able to use\nthe cache at all!\n\nWe've now configured our application so that it's able to execute commands\nagainst our cache.\n\n**IMPORTANT:** Make sure the cache is put in first place within the children\nlist, or at least before the process or processes using it. Otherwise, there\ncould be race conditions causing exceptions; processes attempting to use\nthe cache and this one hasn't been even started.","ref":"getting-started.html#adding-nebulex-to-an-application"},{"type":"extras","title":"Inserting entries - Getting started","doc":"We can insert a new entries into our blog cache with this code:\n\n```elixir\niex> user = %{id: 1, first_name: \"Galileo\", last_name: \"Galilei\"}\niex> Blog.Cache.put(user[:id], user, ttl: :timer.hours(1))\n:ok\n```\n\nTo insert the data into our cache, we call `put` on `Blog.Cache`. This function\ntells Nebulex that we want to insert a new key/value entry into the cache\ncorresponding `Blog.Cache`.\n\nIt is also possible to insert multiple entries at once:\n\n```elixir\niex> users = %{\n...>   1 => %{id: 1, first_name: \"Galileo\", last_name: \"Galilei\"},\n...>   2 => %{id: 2, first_name: \"Charles\", last_name: \"Darwin\"},\n...>   3 => %{id: 3, first_name: \"Albert\", last_name: \"Einstein\"}\n...> }\niex> Blog.Cache.put_all(users)\n:ok\n```\n\n> The given entries can be a `map` or a Key/Value tuple list.","ref":"getting-started.html#inserting-entries"},{"type":"extras","title":"Inserting new entries and replacing existing ones - Getting started","doc":"As we saw previously, `put` creates a new entry in cache if it doesn't exist,\nor overrides it if it does exist (including the `:ttl`). However, there might\nbe circumstances where we want to set the entry only if it doesn't exit or the\nother way around. For those cases you can use `put_new` and `replace` functions\ninstead.\n\nLet's try `put_new` and `put_new!` functions:\n\n```elixir\niex> new_user = %{id: 4, first_name: \"John\", last_name: \"Doe\"}\niex> Blog.Cache.put_new(new_user.id, new_user, ttl: 900)\n{:ok, true}\n\niex> Blog.Cache.put_new(new_user.id, new_user)\n{:ok, false}\n\n# same as previous one but raises `Nebulex.Error` in case of error\niex> Blog.Cache.put_new!(new_user.id, new_user)\nfalse\n```\n\nNow `replace` and `replace!` functions:\n\n```elixir\niex> existing_user = %{id: 5, first_name: \"John\", last_name: \"Doe2\"}\niex> Blog.Cache.replace(existing_user.id, existing_user)\n{:ok, false}\n\niex> Blog.Cache.put_new(existing_user.id, existing_user)\n{:ok, true}\n\niex> Blog.Cache.replace(existing_user.id, existing_user, ttl: 900)\n{:ok, true}\n\n# same as previous one but raises `Nebulex.Error` in case of error\niex> Blog.Cache.replace!(existing_user.id, existing_user)\ntrue\n\niex> Blog.Cache.replace!(\"unknown\", existing_user)\nfalse\n```\n\nIt is also possible to insert multiple new entries at once:\n\n```elixir\niex> new_users = %{\n...>   6 => %{id: 6, first_name: \"Isaac\", last_name: \"Newton\"},\n...>   7 => %{id: 7, first_name: \"Marie\", last_name: \"Curie\"}\n...> }\niex> Blog.Cache.put_new_all(new_users)\n{:ok, true}\n\n# none of the entries is inserted if at least one key already exists\niex> Blog.Cache.put_new_all(new_users)\n{:ok, false}\n\n# same as previous one but raises `Nebulex.Error` in case of error\niex> Blog.Cache.put_new_all!(new_users)\nfalse\n```","ref":"getting-started.html#inserting-new-entries-and-replacing-existing-ones"},{"type":"extras","title":"Retrieving entries - Getting started","doc":"Let's start off with fetching data by the key, which is the most basic and\ncommon operation to retrieve data from a cache.\n\n```elixir\n# Using `fetch` callback\niex> {:ok, user1} = Blog.Cache.fetch(1)\niex> user1.id\n1\n\n# If the key doesn't exist an error tuple is returned\niex> {:error, %Nebulex.KeyError{} = e} = Blog.Cache.fetch(\"unknown\")\niex> e.key\n\"unknown\"\n\n# Using `fetch!` (same as `fetch` but raises an exception in case of error)\niex> user1 = Blog.Cache.fetch!(1)\niex> user1.id\n1\n\n# Using `get` callback (returns the default in case the key doesn't exist)\niex> {:ok, user1} = Blog.Cache.get(1)\niex> user1.id\n1\n\n# Returns the default because the key doesn't exist\niex> Blog.Cache.get(\"unknown\")\n{:ok, nil}\niex> Blog.Cache.get(\"unknown\", \"default\")\n{:ok, \"default\"}\n\n# Using `get!` (same as `get` but raises an exception in case of error)\niex> user1 = Blog.Cache.get!(1)\niex> user1.id\n1\niex> Blog.Cache.get!(\"unknown\")\nnil\niex> Blog.Cache.get!(\"unknown\", \"default\")\n\"default\"\n\niex> Enum.map(1..3, &(Blog.Cache.get!(&1).first_name))\n[\"Galileo\", \"Charles\", \"Albert\"]\n```\n\nThere is a function `has_key?` to check if a key exist in cache:\n\n```elixir\niex> Blog.Cache.has_key?(1)\n{:ok, true}\n\niex> Blog.Cache.has_key?(10)\n{:ok, false}\n```","ref":"getting-started.html#retrieving-entries"},{"type":"extras","title":"Updating entries - Getting started","doc":"Nebulex provides `update` and `get_and_update` functions to update an\nentry value based on current one, for example:\n\n```elixir\niex> initial = %{id: 1, first_name: \"\", last_name: \"\"}\n\n# using `get_and_update`\niex> Blog.Cache.get_and_update(1, fn v ->\n...>   if v, do: {v, %{v | first_name: \"X\"}}, else: {v, initial}\n...> end)\n{:ok, {_old, _updated}}\n\n# using `update`\niex> Blog.Cache.update(1, initial, &(%{&1 | first_name: \"Y\"}))\n{:ok, _updated}\n```\n\n> You can also use the version with the trailing bang (`!`) `get_and_update!`\n> and `!update`.","ref":"getting-started.html#updating-entries"},{"type":"extras","title":"Counters - Getting started","doc":"The function `incr` is provided to increment or decrement a counter; by default,\na counter is initialized to `0`. Let's see how counters works:\n\n```elixir\n# by default, the counter is incremented by 1\niex> Blog.Cache.incr(:my_counter)\n{:ok, 1}\n\n# but we can also provide a custom increment value\niex> Blog.Cache.incr(:my_counter, 5)\n{:ok, 6}\n\n# to decrement the counter, just pass a negative value\niex> Blog.Cache.incr(:my_counter, -5)\n{:ok, 1}\n\n# using `incr!`\niex> Blog.Cache.incr!(:my_counter)\n2\n```","ref":"getting-started.html#counters"},{"type":"extras","title":"Deleting entries - Getting started","doc":"We've now covered inserting, reading and updating entries. Now let's see how to\ndelete an entry using Nebulex.\n\n```elixir\niex> Blog.Cache.delete(1)\n:ok\n\n# or `delete!`\niex> Blog.Cache.delete!(1)\n:ok\n```","ref":"getting-started.html#deleting-entries"},{"type":"extras","title":"Take - Getting started","doc":"This is another way not only for deleting an entry but also for retrieving it\nbefore its delete it:\n\n```elixir\niex> Blog.Cache.take(1)\n{:ok, _entry}\n\n# If the key doesn't exist an error tuple is returned\niex> {:error, %Nebulex.KeyError{} = e} = Blog.Cache.take(\"nonexistent\")\niex> e.key\n\"nonexistent\"\n\n# same as previous one but raises `Nebulex.KeyError`\niex> Blog.Cache.take!(\"nonexistent\")\n```","ref":"getting-started.html#take"},{"type":"extras","title":"Entry expiration - Getting started","doc":"You can get the remaining TTL or expiration time for a key like so:\n\n```elixir\n# If no TTL is set when the entry is created, `:infinity` is set by default\niex> Blog.Cache.ttl(1)\n{:ok, :infinity}\n\n# If the key doesn't exist an error tuple is returned\niex> {:error, %Nebulex.KeyError{} = e} = Blog.Cache.ttl(\"nonexistent\")\niex> e.key\n\"nonexistent\"\n\n# Same as `ttl` but an exception is raised if an error occurs\niex> Blog.Cache.ttl!(1)\n:infinity\n```\n\nYou could also change or update the expiration time using `expire`, like so:\n\n```elixir\niex> Blog.Cache.expire(1, :timer.hours(1))\n{:ok, true}\n\n# When the key doesn't exist false is returned\niex> Blog.Cache.expire(\"nonexistent\", :timer.hours(1))\n{:ok, false}\n\n# Same as `expire` but an exception is raised if an error occurs\niex> Blog.Cache.expire!(1, :timer.hours(1))\ntrue\n```","ref":"getting-started.html#entry-expiration"},{"type":"extras","title":"Query and/or Stream entries - Getting started","doc":"Nebulex provides functions to fetch, count, delete, or stream all entries from\ncache matching the given query.","ref":"getting-started.html#query-and-or-stream-entries"},{"type":"extras","title":"Fetch all entries from cache matching the given query - Getting started","doc":"```elixir\n# by default, returns all entries\niex> Blog.Cache.get_all() #=> The query is set to nil by default\n{:ok, _all_entries}\n\n# fetch all entries and return the keys\niex> Blog.Cache.get_all(select: :key)\n{:ok, _all_keys}\n\n# fetch all entries and return the values\niex> Blog.Cache.get_all(select: :value)\n{:ok, _all_values}\n\n# fetch entries associated to the requested keys\niex> Blog.Cache.get_all(in: [1, 2])\n{:ok, _fetched_entries}\n\n# raises an exception in case of error\niex> Blog.Cache.get_all!()\n_all_entries\n\n# raises an exception in case of error\niex> Blog.Cache.get_all!(in: [1, 2])\n_fetched_entries\n\n# built-in queries in `Nebulex.Adapters.Local` adapter\niex> Blog.Cache.get_all() #=> Equivalent to Blog.Cache.get_all(query: nil)\n\n# if we are using `Nebulex.Adapters.Local` adapter, the stored entry\n# is a tuple `{:entry, key, value, touched, ttl}`, then the match spec\n# could be something like:\niex> spec = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$1\", 10}], [{{:\"$1\", :\"$2\"}}]}]\niex> Blog.Cache.get_all(query: spec)\n{:ok, _all_matched}\n\n# using Ex2ms\niex> import Ex2ms\niex> spec =\n...>   fun do\n...>     {_, key, value, _, _} when key > 10 -> {key, value}\n...>   end\niex> Blog.Cache.get_all(query: spec)\n{:ok, _all_matched}\n```","ref":"getting-started.html#fetch-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Count all entries from cache matching the given query - Getting started","doc":"For example, to get the total number of cached objects (cache size):\n\n```elixir\n# by default, counts all entries\niex> Blog.Cache.count_all() #=> The query is set to nil by default\n{:ok, _num_cached_entries}\n\n# raises an exception in case of error\niex> Blog.Cache.count_all!()\n_num_cached_entries\n```\n\nSimilar to `get_all`, you can pass a query to count only the matched entries.\nFor example, `Blog.Cache.count_all(query: query)`.","ref":"getting-started.html#count-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Delete all entries from cache matching the given query - Getting started","doc":"Similar to `count_all/2`, Nebulex provides `delete_all/2` to not only count\nthe matched entries but also remove them from the cache at once, in one single\nexecution.\n\nThe first example is flushing the cache, delete all cached entries (which is\nthe default behavior when none query is provided):\n\n```elixir\niex> Blog.Cache.delete_all()\n{:ok, _num_of_removed_entries}\n\n# raises an exception in case of error\niex> Blog.Cache.delete_all!()\n_num_of_removed_entries\n```\n\nOne may also delete a list of keys at once (like a bulk delete):\n\n```elixir\niex> Blog.Cache.delete_all(in: [\"k1\", \"k2\"])\n{:ok, _num_of_removed_entries}\n\n# raises an exception in case of error\niex> Blog.Cache.delete_all!(in: [\"k1\", \"k2\"])\n_num_of_removed_entries\n```","ref":"getting-started.html#delete-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Stream all entries from cache matching the given query - Getting started","doc":"Similar to `get_all` but returns a lazy enumerable that emits all entries from\nthe cache matching the provided query.\n\nIf the query is `nil`, then all entries in cache match and are returned when the\nstream is evaluated (based on the `:select` option).\n\n```elixir\niex> {:ok, stream} = Blog.Cache.stream()\niex> Enum.to_list(stream)\n_all_matched\n\niex> {:ok, stream} = Blog.Cache.stream(select: :key)\niex> Enum.to_list(stream)\n_all_matched\n\niex> {:ok, stream} = Blog.Cache.stream([select: :value], max_entries: 100)\niex> Enum.to_list(stream)\n_all_matched\n\n# raises an exception in case of error\niex> stream = Blog.Cache.stream!()\niex> Enum.to_list(stream)\n_all_matched\n\n# using `Nebulex.Adapters.Local` adapter\niex> spec = [{{:entry, :\"$1\", :\"$2\", :_, :_}, [{:<, :\"$1\", 3}], [{{:\"$1\", :\"$2\"}}]}]\niex> {:ok, stream} = Blog.Cache.stream(query: spec)\niex> Enum.to_list(stream)\n_all_matched\n\n# using Ex2ms\niex> import Ex2ms\niex> spec =\n...>   fun do\n...>     {:entry, key, value, _, _} when key < 3 -> {key, value}\n...>   end\niex> {:ok, stream} = Blog.Cache.stream(query: spec)\niex> Enum.to_list(stream)\n_all_matched\n```","ref":"getting-started.html#stream-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Cache Info API - Getting started","doc":"Since Nebulex v3, there is a new API for getting information about the cache\n(including stats).\n\nAlthough Nebulex suggests the adapters add information items like `:server`,\n`:memory`, and `:stats`, the adapters are free to add the information\nspecification keys they want. Therefore, it is important to check the adapter\ndocumentation. For this example, we use the official local adapter that supports\nthe suggested items.\n\n```elixir\n# Returns all information items\niex> {:ok, info} = Blog.Cache.info()\niex> info\n%{\n  server: %{\n    nbx_version: \"3.0.0\",\n    cache_module: \"Blog.Cache\",\n    cache_adapter: \"Nebulex.Adapters.Local\",\n    cache_name: \"Blog.Cache\",\n    cache_pid: #PID<0.111.0>\n  },\n  memory: %{\n    total: 1_000_000,\n    used: 0\n  },\n  stats: %{\n    deletions: 0,\n    evictions: 0,\n    expirations: 0,\n    hits: 0,\n    misses: 0,\n    updates: 0,\n    writes: 0\n  }\n}\n\n# Returns a single item\niex> Blog.Cache.info!(:server)\n%{\n  nbx_version: \"3.0.0\",\n  cache_module: \"Blog.Cache\",\n  cache_adapter: \"Nebulex.Adapters.Local\",\n  cache_name: \"Blog.Cache\",\n  cache_pid: #PID<0.111.0>\n}\n\n# Returns the given items\niex> Blog.Cache.info!([:server, :stats])\n%{\n  server: %{\n    nbx_version: \"3.0.0\",\n    cache_module: \"Blog.Cache\",\n    cache_adapter: \"Nebulex.Adapters.Local\",\n    cache_name: \"Blog.Cache\",\n    cache_pid: #PID<0.111.0>\n  },\n  stats: %{\n    deletions: 0,\n    evictions: 0,\n    expirations: 0,\n    hits: 0,\n    misses: 0,\n    updates: 0,\n    writes: 0\n  }\n}\n```","ref":"getting-started.html#cache-info-api"},{"type":"extras","title":"Cache events - Getting started","doc":"Since Nebulex v3, a new API is available for registering cache event handlers,\nwhich are invoked after an entry is mutated in the cache.\n\n```elixir\ndefmodule Blog.Cache.EventHandler do\n  def handle(event) do\n    IO.inspect(event)\n  end\nend\n\niex> {:ok, id} = Blog.Cache.register_event_listener(&Blog.Cache.EventHandler.handle/1)\niex> Blog.Cache.put(\"event_test_key\", \"event_testvalue\")\n:ok\n\n#=> %Nebulex.Event.CacheEntryEvent{\n#=>   cache: Blog.Cache,\n#=>   name: Blog.Cache,\n#=>   type: :inserted,\n#=>   target: {:key, \"event_test_key\"},\n#=>   command: :put\n#=> }\n```","ref":"getting-started.html#cache-events"},{"type":"extras","title":"Distributed cache topologies - Getting started","doc":"","ref":"getting-started.html#distributed-cache-topologies"},{"type":"extras","title":"Partitioned Cache - Getting started","doc":"Nebulex provides the adapter `Nebulex.Adapters.Partitioned`, which allows to\nset up a partitioned cache topology. First of all, we need to add\n`:nebulex_distributed` to the dependencies in the `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:nebulex, \"~> 3.0\"},\n    # Use the official local cache adapter\n    {:nebulex_local, \"~> 3.0\"},\n    # Use the official distributed cache adapters\n    {:nebulex_distributed, \"~> 3.0\"},\n    # Required for caching decorators (recommended)\n    {:decorator, \"~> 1.4\"},\n    # Required for telemetry events (recommended)\n    {:telemetry, \"~> 1.0\"},\n    # Required for :shards backend in local adapter\n    {:shards, \"~> 1.1\"}\n  ]\nend\n```\n\nLet's set up the partitioned cache by using the `mix` task\n`mix nbx.gen.cache.partitioned`:\n\n```\nmix nbx.gen.cache.partitioned -c Blog.PartitionedCache\n```\n\nAs we saw previously, this command will generate the cache in\n`lib/bolg/partitioned_cache.ex` (in this case using the partitioned adapter)\nmodule along with the initial configuration in `config/config.exs`.\n\nThe cache:\n\n```elixir\ndefmodule Blog.PartitionedCache do\n  use Nebulex.Cache,\n    otp_app: :blog,\n    adapter: Nebulex.Adapters.Partitioned,\n    primary_storage_adapter: Nebulex.Adapters.Local\nend\n```\n\nAnd the config:\n\n```elixir\nconfig :blog, Blog.PartitionedCache,\n  primary: [\n    # When using :shards as backend\n    backend: :shards,\n    # GC interval for pushing new generation: 12 hrs\n    gc_interval: :timer.hours(12),\n    # Max 1 million entries in cache\n    max_size: 1_000_000,\n    # Max 2 GB of memory\n    allocated_memory: 2_000_000_000,\n    # GC memory check interval\n    gc_memory_check_interval: :timer.seconds(10)\n  ]\n```\n\nAnd remember to add the new cache `Blog.PartitionedCache` to your application's\nsupervision tree (such as we did it previously):\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Blog.Cache,\n    Blog.PartitionedCache\n  ]\n\n  ...\n```\n\nNow we are ready to start using our partitioned cache!\n\n#### Timeout option\n\nThe `Nebulex.Adapters.Partitioned` supports `:timeout` option, it is a value in\nmilliseconds for the command that will be executed.\n\n```elixir\niex> Blog.PartitionedCache.get(\"foo\", timeout: 10)\n#=> {:ok, value}\n\n# when the command's call timed out an error is returned\niex> Blog.PartitionedCache.put(\"foo\", \"bar\", timeout: 10)\n#=> {:error, %Nebulex.Error{reason: :timeout}}\n```\n\nTo learn more about how partitioned cache works, please check\n`Nebulex.Adapters.Partitioned` documentation, and also it is recommended see the\n[partitioned cache example](https://github.com/elixir-nebulex/nebulex_examples/tree/main/partitioned_cache).","ref":"getting-started.html#partitioned-cache"},{"type":"extras","title":"Multilevel Cache - Getting started","doc":"Nebulex also provides the adapter `Nebulex.Adapters.Multilevel`, which allows to\nsetup a multi-level caching hierarchy. The adapter is also included in the\n`:nebulex_distributed` dependency.\n\nLet's set up the multilevel cache by using the `mix` task\n`mix nbx.gen.cache.multilevel`:\n\n```\nmix nbx.gen.cache.multilevel -c Blog.NearCache\n```\n\nBy default, the command generates a 2-level near-cache topology. The first\nlevel or `L1` using `Nebulex.Adapters.Local` adapter, and the second one or `L2`\nusing `Nebulex.Adapters.Partitioned` adapter.\n\nThe generated cache module `lib/blog/near_cache.ex`:\n\n```elixir\ndefmodule Blog.NearCache do\n  use Nebulex.Cache,\n    otp_app: :blog,\n    adapter: Nebulex.Adapters.Multilevel","ref":"getting-started.html#multilevel-cache"},{"type":"extras","title":"Cache Levels - Getting started","doc":"# Default auto-generated L1 cache (local)\n  defmodule L1 do\n    use Nebulex.Cache,\n      otp_app: :blog,\n      adapter: Nebulex.Adapters.Local\n  end\n\n  # Default auto-generated L2 cache (partitioned cache)\n  defmodule L2 do\n    use Nebulex.Cache,\n      otp_app: :blog,\n      adapter: Nebulex.Adapters.Partitioned\n  end","ref":"getting-started.html#cache-levels"},{"type":"extras","title":"TODO: Add, remove or modify the auto-generated cache levels above - Getting started","doc":"end\n```\n\nAnd the configuration (`config/config.exs`):\n\n```elixir\nconfig :blog, Blog.NearCache,\n  model: :inclusive,\n  levels: [\n    # Default auto-generated L1 cache (local)\n    {\n      Blog.NearCache.L1,\n      # GC interval for pushing new generation: 12 hrs\n      gc_interval: :timer.hours(12),\n      # Max 1 million entries in cache\n      max_size: 1_000_000\n    },\n    # Default auto-generated L2 cache (partitioned cache)\n    {\n      Blog.NearCache.L2,\n      primary: [\n        # GC interval for pushing new generation: 12 hrs\n        gc_interval: :timer.hours(12),\n        # Max 1 million entries in cache\n        max_size: 1_000_000\n      ]\n    }\n  ]\n```\n\n> Remember you can add `backend: :shards` to use Shards as backend.\n\nFinally, add the new cache `Blog.NearCache` to your application's supervision\ntree (such as we did it previously):\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Blog.Cache,\n    Blog.PartitionedCache,\n    Blog.NearCache\n  ]\n\n  ...\n```\n\nLet's try it out!\n\n```elixir\niex> Blog.NearCache.put(\"foo\", \"bar\", ttl: :timer.hours(1))\n:ok\n\niex> Blog.NearCache.get!(\"foo\")\n\"bar\"\n```\n\nTo learn more about how multilevel-cache works, please check\n`Nebulex.Adapters.Multilevel` documentation, and also it is recommended see the\n[near cache example](https://github.com/elixir-nebulex/nebulex_examples/tree/main/near_cache).","ref":"getting-started.html#todo-add-remove-or-modify-the-auto-generated-cache-levels-above"},{"type":"extras","title":"Next - Getting started","doc":"* [Decorators-based DSL for cache usage patterns][cache-usage-patterns].\n\n[cache-usage-patterns]: http://hexdocs.pm/nebulex/3.0.0-rc.1/cache-usage-patterns.html","ref":"getting-started.html#next"},{"type":"extras","title":"Official adapters","doc":"# Official adapters\n\nCurrently we officially support the following adapters:\n\n| Cache | Nebulex Adapter | Dependency |\n|:------|:----------------|:-----------|\n| Nil (special adapter to disable caching) | [Nebulex.Adapters.Nil][nil] | Built-In |\n| Generational Local Cache | [Nebulex.Adapters.Local][la] | [nebulex_local][la] |\n| Partitioned | [Nebulex.Adapters.Partitioned][pa] | [nebulex_distributed][pa] |\n| Multilevel | [Nebulex.Adapters.Multilevel][ma] | [nebulex_distributed][ma] |\n| Redis | [Nebulex.Adapters.Redis][nbx_redis] | [nebulex_redis_adapter][nbx_redis] |\n| Cachex | [Nebulex.Adapters.Cachex][nbx_cachex] | [nebulex_adapters_cachex][nbx_cachex] |\n\n[nil]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Adapters.Nil.html\n[la]: http://hexdocs.pm/nebulex_local/3.0.0-rc.1/Nebulex.Adapters.Local.html\n[pa]: http://hexdocs.pm/nebulex_distributed/3.0.0-rc.1/Nebulex.Adapters.Partitioned.html\n[ma]: http://hexdocs.pm/nebulex_distributed/3.0.0-rc.1/Nebulex.Adapters.Multilevel.html\n[nbx_redis]: http://hexdocs.pm/nebulex_redis_adapter/3.0.0-rc.1/Nebulex.Adapters.Redis.html\n[nbx_cachex]: http://hexdocs.pm/nebulex_adapters_cachex/3.0.0-rc.1/Nebulex.Adapters.Cachex.html\n\nThe adapter documentation links above will help you get started with your\nadapter of choice. For API reference, you can check out the\n[Nebulex documentation](http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.html).","ref":"nbx-adapters.html"},{"type":"extras","title":"Cache usage patterns with caching decorators","doc":"# Cache usage patterns with caching decorators\n\nNebulex supports several common cache access patterns via\n[caching decorators][nbx_caching].\n\n[nbx_caching]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Caching.Decorators.html\n\n> The following documentation about caching patterns is based on\n> [EHCache Docs][EHCache]\n\n[EHCache]: https://www.ehcache.org/documentation/3.10/caching-patterns.html","ref":"cache-usage-patterns.html"},{"type":"extras","title":"Cache-aside - Cache usage patterns with caching decorators","doc":"The cache-aside pattern involves direct cache usage in application code.\n\nWhen accessing the system-of-record (SoR), the application first checks the\ncache. If the data exists in the cache, it's returned directly, bypassing the\nSoR. Otherwise, the application fetches the data from the SoR, stores it in the\ncache, and then returns it. When writing data, both the cache and SoR must be\nupdated.","ref":"cache-usage-patterns.html#cache-aside"},{"type":"extras","title":"Reading values - Cache usage patterns with caching decorators","doc":"```elixir\n# Check cache first, then fall back to SoR\nwith {:error, _reason} <- MyCache.fetch(key) do\n  value = SoR.get(key)  # e.g., Ecto.Repo\n  MyCache.put(key, value)\n  value\nend\n```","ref":"cache-usage-patterns.html#reading-values"},{"type":"extras","title":"Writing values - Cache usage patterns with caching decorators","doc":"```elixir\n# Update both cache and SoR\nMyCache.put(key, value)\nSoR.insert(key, value)  # e.g., Ecto.Repo\n```\n\nThis is the default behavior for most caches, requiring direct interaction with\nboth the cache and the SoR (typically a database).","ref":"cache-usage-patterns.html#writing-values"},{"type":"extras","title":"Cache-as-SoR - Cache usage patterns with caching decorators","doc":"The cache-as-SoR pattern uses the cache as the primary system-of-record (SoR).\n\nThe pattern delegates SoR reading and writing activities to the cache, so that\napplication code is (at least directly) absolved of this responsibility.\nTo implement the cache-as-SoR pattern, use a combination of the following\nread and write patterns:\n\n * **Read-through**\n\n * **Write-through**\n\nAdvantages of using the cache-as-SoR pattern are:\n\n * Less cluttered application code (improved maintainability through centralized\n   SoR read/write operations)\n\n * Choice of write-through or write-behind strategies on a per-cache basis\n\n * Allows the cache to solve the thundering-herd problem\n\nA disadvantage of using the cache-as-SoR pattern is:\n\n * Less directly visible code-path\n\nBut how to get all this out-of-box? This is where declarative decorator-based\ncaching comes in. Nebulex provides a set of annotation to abstract most of the\nlogic behind **Read-through** and **Write-through** patterns and make the\nimplementation extremely easy. But let's go over these patterns more in detail\nand how to implement them by using [Nebulex decorators][nbx_caching].","ref":"cache-usage-patterns.html#cache-as-sor"},{"type":"extras","title":"Read-through - Cache usage patterns with caching decorators","doc":"Under the read-through pattern, the cache is configured with a loader component\nthat knows how to load data from the system-of-record (SoR).\n\nWhen the cache is asked for the value associated with a given key and such an\nentry does not exist within the cache, the cache invokes the loader to retrieve\nthe value from the SoR, then caches the value, then returns it to the caller.\n\nThe next time the cache is asked for the value for the same key it can be\nreturned from the cache without using the loader (unless the entry has been\nevicted or expired).\n\nThis pattern can be easily implemented using the `cacheable` decorator\nas follows:\n\n```elixir\ndefmodule MyApp.Example do\n  use Nebulex.Caching, cache: MyApp.Cache\n\n  @ttl :timer.hours(1)\n\n  @decorate cacheable(key: name)\n  def get_by_name(name) do\n    # your logic (the loader to retrieve the value from the SoR)\n  end\n\n  @decorate cacheable(key: age, opts: [ttl: @ttl])\n  def get_by_age(age) do\n    # your logic (the loader to retrieve the value from the SoR)\n  end\n\n  @decorate cacheable()\n  def all(query) do\n    # your logic (the loader to retrieve the value from the SoR)\n  end\nend\n```\n\nAs you can see, the loader to retrieve the value from the system-of-record (SoR)\nis the function logic itself.","ref":"cache-usage-patterns.html#read-through"},{"type":"extras","title":"Write-through - Cache usage patterns with caching decorators","doc":"Under the write-through pattern, the cache is configured with a writer component\nthat knows how to write data to the system-of-record (SoR).\n\nWhen the cache is asked to store a value for a key, the cache invokes the writer\nto store the value in the SoR, as well as updating (or deleting) the cache.\n\nThis pattern can be implemented using `cache_evict` or `cache_put` decorators.\nWhen the data is written to the system-of-record (SoR), you can update the\ncached value associated with the given key using `cache_put`, or just delete\nit using `cache_evict`.\n\n```elixir\ndefmodule MyApp.Example do\n  use Nebulex.Caching, cache: MyApp.Cache\n\n  # When the data is written to the SoR, it is updated in the cache\n  @decorate cache_put(key: something)\n  def update(something) do\n    # Write data to the SoR (most likely the Database)\n  end\n\n  # When the data is written to the SoR, it is deleted (evicted) from the cache\n  @decorate cache_evict(key: something)\n  def update_something(something) do\n    # Write data to the SoR (most likely the Database)\n  end\nend\n```\n\nAs you can see, the logic to write data to the system-of-record (SoR) is the\nfunction logic itself.","ref":"cache-usage-patterns.html#write-through"},{"type":"extras","title":"Info API","doc":"# Info API\n\nSince Nebulex v3, the adapter's Info API is introduced. This is a more generic\nAPI to get information about the cache, including the stats. Adapters are\nresponsible for implementing the Info API and are also free to add the\ninformation specification keys they want. Therefore, it is highly recommended\nto review the adapter's documentation you're using.\n\n> See `c:Nebulex.Cache.info/2` for more information.\n\nNebulex also provides a simple implementation\n[`Nebulex.Adapters.Common.Info`][nbx_common_info], which is used by the\n`Nebulex.Adapters.Local` adapter. This implementation uses a Telemetry\nhandler to aggregate the stats and keep them updated, therefore, it requires\n`:telemetry` to be available.\n\n[nbx_common_info]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Adapters.Common.Info.html","ref":"info-api.html"},{"type":"extras","title":"Usage - Info API","doc":"Let's define our cache:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Nebulex.Cache,\n    otp_app: :my_app,\n    adapter: Nebulex.Adapters.Local\nend\n```\n\nAnd the configuration:\n\n```elixir\nconfig :my_app, MyApp.Cache,\n  gc_interval: :timer.hours(12),\n  max_size: 1_000_000,\n  allocated_memory: 1_000_000,\n  gc_memory_check_interval: :timer.seconds(10)\n```\n\nOnce you have set up the `MyApp.Cache` within the application's supervision\ntree, you can get the cache info like so:\n\n```elixir\niex> MyApp.Cache.info!()\n%{\n  server: %{\n    nbx_version: \"3.0.0\",\n    cache_module: \"MyCache\",\n    cache_adapter: \"Nebulex.Adapters.Local\",\n    cache_name: \"MyCache\",\n    cache_pid: #PID<0.111.0>\n  },\n  memory: %{\n    total: 1_000_000,\n    used: 0\n  },\n  stats: %{\n    deletions: 0,\n    evictions: 0,\n    expirations: 0,\n    hits: 0,\n    misses: 0,\n    updates: 0,\n    writes: 0\n  }\n}\n```\n\nYou could also request for a specific item or items:\n\n```elixir\niex> MyApp.Cache.info!(:stats)\n%{\n  deletions: 0,\n  evictions: 0,\n  expirations: 0,\n  hits: 0,\n  misses: 0,\n  updates: 0,\n  writes: 0\n}\n\niex> MyApp.Cache.info!([:stats, :memory])\n%{\n  memory: %{\n    total: 1_000_000,\n    used: 0\n  },\n  stats: %{\n    deletions: 0,\n    evictions: 0,\n    expirations: 0,\n    hits: 0,\n    misses: 0,\n    updates: 0,\n    writes: 0\n  }\n}\n```","ref":"info-api.html#usage"},{"type":"extras","title":"Telemetry Metrics - Info API","doc":"Now, let's see how we can provide metrics out of the info data.\n\nFirst of all, make sure you have added `:telemetry`, `:telemetry_metrics`, and\n`:telemetry_poller` packages as dependencies to your `mix.exs` file.\n\nCreate your Telemetry supervisor at `lib/my_app/telemetry.ex`:\n\n```elixir\n# lib/my_app/telemetry.ex\ndefmodule MyApp.Telemetry do\n  use Supervisor\n  import Telemetry.Metrics\n\n  def start_link(arg) do\n    Supervisor.start_link(__MODULE__, arg, name: __MODULE__)\n  end\n\n  def init(_arg) do\n    children = [\n      # Configure `:telemetry_poller` for reporting the cache stats\n      {:telemetry_poller, measurements: periodic_measurements(), period: 10_000},\n\n      # For example, we use the console reporter, but you can change it.\n      # See `:telemetry_metrics` for for information.\n      {Telemetry.Metrics.ConsoleReporter, metrics: metrics()}\n    ]\n\n    Supervisor.init(children, strategy: :one_for_one)\n  end\n\n  defp metrics do\n    [\n      # Stats\n      last_value(\"my_app.cache.info.stats.hits\", tags: [:cache]),\n      last_value(\"my_app.cache.info.stats.misses\", tags: [:cache]),\n      last_value(\"my_app.cache.info.stats.writes\", tags: [:cache]),\n      last_value(\"my_app.cache.info.stats.evictions\", tags: [:cache]),\n\n      # Memory\n      last_value(\"my_app.cache.info.memory.used\", tags: [:cache]),\n      last_value(\"my_app.cache.info.memory.total\", tags: [:cache])\n    ]\n  end\n\n  defp periodic_measurements do\n    [\n      {__MODULE__, :cache_stats, []},\n      {__MODULE__, :cache_memory, []}\n    ]\n  end\n\n  def cache_stats do\n    with {:ok, info} <- MyApp.Cache.info([:server, :stats]) do\n      :telemetry.execute(\n        [:my_app, :cache, :info, :stats],\n        info.stats,\n        %{cache: info.server[:cache_name]}\n      )\n    end\n\n    :ok\n  end\n\n  def cache_memory do\n    with {:ok, info} <- MyApp.Cache.info([:server, :memory]) do\n      :telemetry.execute(\n        [:my_app, :cache, :info, :memory],\n        info.memory,\n        %{cache: info.server[:cache_name]}\n      )\n    end\n\n    :ok\n  end\nend\n```\n\nThen add it to your main application's supervision tree\n(usually in `lib/my_app/application.ex`):\n\n```elixir\nchildren = [\n  MyApp.Cache,\n  MyApp.Telemetry,\n  ...\n]\n```\n\nNow start an IEx session and you should see something like the following output:\n\n```\n[Telemetry.Metrics.ConsoleReporter] Got new event!\nEvent name: my_app.cache.info.stats\nAll measurements: %{evictions: 2, hits: 1, misses: 2, writes: 2}\nAll metadata: %{cache: MyApp.Cache}\n\nMetric measurement: :hits (last_value)\nWith value: 1\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :misses (last_value)\nWith value: 2\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :writes (last_value)\nWith value: 2\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :evictions (last_value)\nWith value: 2\nTag values: %{cache: MyApp.Cache}\n\n[Telemetry.Metrics.ConsoleReporter] Got new event!\nEvent name: my_app.cache.info.memory\nAll measurements: %{total: 2000000, used: 0}\nAll metadata: %{cache: MyApp.Cache}\n\nMetric measurement: :total (last_value)\nWith value: 2000000\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :used (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache}\n```","ref":"info-api.html#telemetry-metrics"},{"type":"extras","title":"Custom metrics - Info API","doc":"In the same way, you can add another periodic measurement for reporting the\ncache size:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Nebulex.Cache,\n    otp_app: :my_app,\n    adapter: Nebulex.Adapters.Local\n\n  def dispatch_cache_size do\n    :telemetry.execute(\n      [:my_app, :cache, :size],\n      %{value: count_all()},\n      %{cache: __MODULE__, node: node()}\n    )\n  end\nend\n```\n\nNow let's add a new periodic measurement to invoke `dispatch_cache_size()`\nthrough `:telemetry_poller`:\n\n```elixir\ndefp periodic_measurements do\n  [\n    {__MODULE__, :cache_stats, []},\n    {__MODULE__, :cache_memory, []},\n    {MyApp.Cache, :dispatch_cache_size, []}\n  ]\nend\n```\n\n> Notice the node name was added to the metadata so we can use it in the\n> metric tags.\n\nMetrics:\n\n```elixir\ndefp metrics do\n  [\n    # Stats\n    last_value(\"my_app.cache.info.stats.hits\", tags: [:cache]),\n    last_value(\"my_app.cache.info.stats.misses\", tags: [:cache]),\n    last_value(\"my_app.cache.info.stats.writes\", tags: [:cache]),\n    last_value(\"my_app.cache.info.stats.evictions\", tags: [:cache]),\n\n    # Memory\n    last_value(\"my_app.cache.info.memory.used\", tags: [:cache]),\n    last_value(\"my_app.cache.info.memory.total\", tags: [:cache]),\n\n    # Nebulex custom Metrics\n    last_value(\"my_app.cache.size.value\", tags: [:cache, :node])\n  ]\nend\n```\n\nIf you start an IEx session like previously, you should see the new metric too:\n\n```\n[Telemetry.Metrics.ConsoleReporter] Got new event!\nEvent name: my_app.cache.size\nAll measurements: %{value: 0}\nAll metadata: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :value (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n```","ref":"info-api.html#custom-metrics"},{"type":"extras","title":"Telemetry","doc":"# Telemetry\n\nThis guide explains how to instrument and monitor cache telemetry events in your\nNebulex application. For general information about `:telemetry`, see the\n[official documentation][telemetry] or the [Phoenix Telemetry guide][phx_telemetry].\n\n[telemetry]: https://github.com/beam-telemetry/telemetry\n[phx_telemetry]: https://hexdocs.pm/phoenix/telemetry.html","ref":"telemetry.html"},{"type":"extras","title":"Telemetry Events - Telemetry","doc":"Many Elixir libraries, including Nebulex, use `:telemetry` to provide insights\ninto application behavior by emitting events at key lifecycle moments.\n\nFor detailed information about emitted events, measurements, and metadata, see\nthe [Telemetry Events documentation][nbx_telemetry_events].\n\n[nbx_telemetry_events]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Cache.html#module-telemetry-events","ref":"telemetry.html#telemetry-events"},{"type":"extras","title":"Nebulex Metrics - Telemetry","doc":"Assuming you have defined the cache `MyApp.Cache` with the default\n`:telemetry_prefix` (`[:my_app, :cache]`), you can use `Telemetry.Metrics`\nto define various metrics.","ref":"telemetry.html#nebulex-metrics"},{"type":"extras","title":"Counter Metric - Telemetry","doc":"Count the number of completed cache commands:\n\n```elixir\nTelemetry.Metrics.counter(\"my_app.cache.command.stop.duration\")\n```","ref":"telemetry.html#counter-metric"},{"type":"extras","title":"Distribution Metric - Telemetry","doc":"Track command completion times in specific buckets:\n\n```elixir\nTelemetry.Metrics.distribution(\n  \"my_app.cache.command.stop.duration\",\n  buckets: [100, 200, 300]  # Duration in milliseconds\n)\n```","ref":"telemetry.html#distribution-metric"},{"type":"extras","title":"Summary Metric - Telemetry","doc":"For more detailed analysis, you can define a summary metric to track:\n- Average command duration\n- Minimum and maximum execution times\n- Percentiles\n- Aggregation by command or callback name\n\n```elixir\nTelemetry.Metrics.summary(\n  \"my_app.cache.command.stop.duration\",\n  unit: {:native, :millisecond},\n  tags: [:command]\n)\n```","ref":"telemetry.html#summary-metric"},{"type":"extras","title":"Extracting tag values from adapter's metadata - Telemetry","doc":"Let's add another metric for the command event, this time to group by\n**command**, **cache**, and **name** (in case of dynamic caches):\n\n```elixir\nTelemetry.Metrics.summary(\n  \"my_app.cache.command.stop.duration\",\n  unit: {:native, :millisecond},\n  tags: [:command, :cache, :name],\n  tag_values:\n    &%{\n      cache: &1.adapter_meta.cache,\n      name: &1.adapter_meta.name,\n      command: &1.command\n    }\n)\n```\n\nWe've introduced the `:tag_values` option here, because we need to perform a\ntransformation on the event metadata in order to get to the values we need.","ref":"telemetry.html#extracting-tag-values-from-adapter-s-metadata"},{"type":"extras","title":"Creating a new adapter","doc":"# Creating a new adapter\n\nThis guide will walk you through creating a custom Nebulex adapter. We will\nstart by creating a new project, making tests pass, and then implementing a\nsimple in-memory adapter. It will be roughly based on\n[`Nebulex.Adapters.Redis`](https://github.com/elixir-nebulex/nebulex_redis_adapter/)\nso you can consult this repo as an example.","ref":"creating-new-adapter.html"},{"type":"extras","title":"Mix Project - Creating a new adapter","doc":"Nebulex's main repo contains some very useful shared tests that we are going to\ntake advantage of. To do so we will need to checkout Nebulex from Github as the\nversion published to Hex does not contain test code. To accommodate this\nworkflow we will start by creating a new project.\n\n```console\nmix new nebulex_memory_adapter\n```\n\nNow let's modify `mix.exs` so that we could fetch Nebulex repository.\n\n```elixir\ndefmodule NebulexMemoryAdapter.MixProject do\n  use Mix.Project\n\n  @nbx_vsn \"3.0.0\"\n  @version \"0.1.0\"\n\n  def project do\n    [\n      app: :nebulex_memory_adapter,\n      version: @version,\n      elixir: \"~> 1.15\",\n      elixirc_paths: elixirc_paths(Mix.env()),\n      aliases: aliases(),\n      deps: deps(),\n    ]\n  end\n\n  # Run \"mix help compile.app\" to learn about applications.\n  def application do\n    [\n      extra_applications: [:logger]\n    ]\n  end\n\n  defp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\n  defp elixirc_paths(_), do: [\"lib\"]\n\n  # Run \"mix help deps\" to learn about dependencies.\n  defp deps do\n    [\n      nebulex_dep(),\n      {:telemetry, \"~> 0.4 or ~> 1.0\", optional: true}\n    ]\n  end\n\n  defp nebulex_dep do\n    if path = System.get_env(\"NEBULEX_PATH\") do\n      {:nebulex, \"~> #{@nbx_vsn}\", path: path}\n    else\n      {:nebulex, \"~> #{@nbx_vsn}\"}\n    end\n  end\n\n  defp aliases do\n    [\n      \"nbx.setup\": [\n        \"cmd rm -rf nebulex\",\n        \"cmd git clone --depth 1 --branch v#{@nbx_vsn} https://github.com/elixir-nebulex/nebulex\"\n      ]\n    ]\n  end\nend\n```\n\nAs you can see here we define a `mix nbx.setup` task that will fetch a Nebulex\nversion to a folder specified in `NEBULEX_PATH` environmental variable. Let's\nrun it.\n\n```console\nexport NEBULEX_PATH=nebulex\nmix nbx.setup\n```\n\nNow is the good time to fetch other dependencies\n\n```console\nmix deps.get\n```","ref":"creating-new-adapter.html#mix-project"},{"type":"extras","title":"Tests - Creating a new adapter","doc":"Before we start implementing our custom adapter, let's make our tests up and\nrunning.\n\nWe start by defining a cache that uses our adapter in\n`test/support/test_cache.ex`\n\n```elixir\ndefmodule NebulexMemoryAdapter.TestCache do\n  use Nebulex.Cache,\n    otp_app: :nebulex_memory_adapter,\n    adapter: NebulexMemoryAdapter\nend\n```\n\nWe won't be writing tests ourselves. Instead, we will use shared tests from the\nNebulex parent repo. To do so, we will create a helper module in\n`test/shared/cache_test.exs` that will `use` test suites for behaviour we are\ngoing to implement. The minimal set of behaviours is `KV` and `Queryable` so\nwe'll go with them.\n\n```elixir\ndefmodule NebulexMemoryAdapter.CacheTest do\n  @moduledoc \"\"\"\n  Shared Tests\n  \"\"\"\n\n  defmacro __using__(_opts) do\n    quote do\n      use Nebulex.Cache.KVTest\n      use Nebulex.Cache.QueryableTest\n    end\n  end\nend\n```\n\nLet's now edit `test/nebulex_memory_adapter_test.exs` and make it run those\nshared tests by calling `use NebulexMemoryAdapter.CacheTest`. We also need to\ndefine a setup that will start our cache process and put the `cache` and `name`\nkeys into the test context.\n\n```elixir\ndefmodule NebulexMemoryAdapterTest do\n  use ExUnit.Case, async: true\n  use NebulexMemoryAdapter.CacheTest\n\n  alias NebulexMemoryAdapter.TestCache, as: Cache\n\n  setup do\n    {:ok, pid} = Cache.start_link()\n    Cache.delete_all()\n    :ok\n\n    on_exit(fn -> safe_stop(pid) end)\n\n    {:ok, cache: Cache, name: Cache}\n  end\n\n  defp safe_stop(pid) do\n    Cache.stop(pid)\n  catch\n    :exit, _ -> :ok\n  end\nend\n```\n\nNow it's time to grind through failing tests.\n\n```console\nmix test\n== Compilation error in file test/support/test_cache.ex ==\n** (ArgumentError) expected :adapter option given to Nebulex.Cache to list Nebulex.Adapter as a behaviour\n    (nebulex 2.4.2) lib/nebulex/cache/supervisor.ex:50: Nebulex.Cache.Supervisor.compile_config/1\n    test/support/test_cache.ex:2: (module)\n```\n\nLooks like our adapter needs to `Nebulex.Adapter` behaviour. Luckily, it's just\n2 callback that we can copy from `Nebulex.Adapters.Nil`\n\n```elixir\n# lib/nebulex_memory_adapter.ex\ndefmodule NebulexMemoryAdapter do\n  @behaviour Nebulex.Adapter\n\n  @impl Nebulex.Adapter\n  defmacro __before_compile__(_env), do: :ok\n\n  @impl Nebulex.Adapter\n  def init(_opts) do\n    child_spec = Supervisor.child_spec({Agent, fn -> :ok end}, id: {Agent, 1})\n    {:ok, child_spec, %{}}\n  end\nend\n```\n\nAnother try\n\n```console\nmix test\n== Compilation error in file test/nebulex_memory_adapter_test.exs ==\n** (CompileError) test/nebulex_memory_adapter_test.exs:3: module Nebulex.Cache.KVTest is not loaded and could not be found\n    (elixir 1.13.2) expanding macro: Kernel.use/1\n    test/nebulex_memory_adapter_test.exs:3: NebulexMemoryAdapterTest (module)\n    expanding macro: NebulexMemoryAdapter.CacheTest.__using__/1\n    test/nebulex_memory_adapter_test.exs:3: NebulexMemoryAdapterTest (module)\n    (elixir 1.13.2) expanding macro: Kernel.use/1\n    test/nebulex_memory_adapter_test.exs:3: NebulexMemoryAdapterTest (module)\n```\n\nLooks like files from Nebulex parent repo aren't automatically compiled. Let's\naddress this in `test/test_helper.exs`\n\n```elixir\n# Nebulex dependency path\nnbx_dep_path = Mix.Project.deps_paths()[:nebulex]\n\nfor file <- File.ls!(\"#{nbx_dep_path}/test/support\"), file != \"test_cache.ex\" do\n  Code.require_file(\"#{nbx_dep_path}/test/support/\" <> file, __DIR__)\nend\n\nfor file <- File.ls!(\"#{nbx_dep_path}/test/shared/cache\") do\n  Code.require_file(\"#{nbx_dep_path}/test/shared/cache/\" <> file, __DIR__)\nend\n\nfor file <- File.ls!(\"#{nbx_dep_path}/test/shared\"), file != \"cache\" do\n  Code.require_file(\"#{nbx_dep_path}/test/shared/\" <> file, __DIR__)\nend\n\n# Load shared tests\nfor file <- File.ls!(\"test/shared\"), not File.dir?(\"test/shared/\" <> file) do\n  Code.require_file(\"./shared/\" <> file, __DIR__)\nend\n\nExUnit.start()\n```\n\nOne more attempt\n\n```console\nmix test\n< ... >\n 54) test put_all/2 puts the given entries using different data types at once (NebulexMemoryAdapterTest)\n     test/nebulex_memory_adapter_test.exs:128\n     ** (UndefinedFunctionError) function NebulexMemoryAdapter.TestCache.delete_all/0 is undefined or private. Did you mean:\n\n           * delete/1\n           * delete/2\n\n     stacktrace:\n       (nebulex_memory_adapter 0.1.0) NebulexMemoryAdapter.TestCache.delete_all()\n       test/nebulex_memory_adapter_test.exs:9: NebulexMemoryAdapterTest.__ex_unit_setup_0/1\n       test/nebulex_memory_adapter_test.exs:1: NebulexMemoryAdapterTest.__ex_unit__/2\n\n\n\nFinished in 0.2 seconds (0.2s async, 0.00s sync)\n54 tests, 54 failures\n```","ref":"creating-new-adapter.html#tests"},{"type":"extras","title":"Implementation - Creating a new adapter","doc":"Now that we have our failing tests we can write some implementation. We'll start\nby making `delete_all/0` work as it is called in the setup.\n\n```elixir\ndefmodule NebulexMemoryAdapter do\n  @behaviour Nebulex.Adapter\n  @behaviour Nebulex.Adapter.Queryable\n\n  import Nebulex.Utils\n\n  @impl Nebulex.Adapter\n  defmacro __before_compile__(_env), do: :ok\n\n  @impl Nebulex.Adapter\n  def init(_opts) do\n    child_spec = Supervisor.child_spec({Agent, fn -> %{} end}, id: {Agent, 1})\n\n    {:ok, child_spec, %{}}\n  end\n\n  @impl Nebulex.Adapter.Queryable\n  def execute(adapter_meta, %{op: :delete_all} = query_meta, opts) do\n    deleted = Agent.get(adapter_meta.pid, &map_size/1)\n\n    Agent.update(adapter_meta.pid, fn _state -> %{} end)\n\n    wrap_ok deleted\n  end\nend\n```\n\nDid we make any progress?\n\n```console\nmix test\n< ... >\n\n 44) test decr/3 decrements a counter by the given amount with default (NebulexMemoryAdapterTest)\n     test/nebulex_memory_adapter_test.exs:355\n     ** (UndefinedFunctionError) function NebulexMemoryAdapter.update_counter/6 is undefined or private\n     stacktrace:\n       (nebulex_memory_adapter 0.1.0) NebulexMemoryAdapter.update_counter(%{cache: NebulexMemoryAdapter.TestCache, pid: #PID<0.549.0>}, :counter1, -1, :infinity, 10, [default: 10])\n       test/nebulex_memory_adapter_test.exs:356: (test)\n\n\n\nFinished in 5.7 seconds (5.7s async, 0.00s sync)\n54 tests, 44 failures\n```\n\nWe certainly did! From here you can continue to implement necessary callbacks\none-by-one or define them all in bulk. For posterity, we put a complete\n`NebulexMemoryAdapter` module here that passes all tests.\n\n```elixir\ndefmodule NebulexMemoryAdapter do\n  @behaviour Nebulex.Adapter\n  @behaviour Nebulex.Adapter.KV\n  @behaviour Nebulex.Adapter.Queryable\n\n  import Nebulex.Utils\n\n  @impl Nebulex.Adapter\n  defmacro __before_compile__(_env), do: :ok\n\n  @impl Nebulex.Adapter\n  def init(_opts) do\n    child_spec = Supervisor.child_spec({Agent, fn -> %{} end}, id: {Agent, 1})\n\n    {:ok, child_spec, %{}}\n  end\n\n  @impl Nebulex.Adapter.KV\n  def fetch(adapter_meta, key, _opts) do\n    wrap_ok Agent.get(adapter_meta.pid, &Map.get(&1, key))\n  end\n\n  @impl Nebulex.Adapter.KV\n  def put(adapter_meta, key, value, ttl, op, opts)\n\n  def put(adapter_meta, key, value, ttl, :put_new, opts) do\n    if get(adapter_meta, key, []) do\n      false\n    else\n      put(adapter_meta, key, value, ttl, :put, opts)\n      true\n    end\n    |> wrap_ok()\n  end\n\n  def put(adapter_meta, key, value, ttl, :replace, opts) do\n    if get(adapter_meta, key, []) do\n      put(adapter_meta, key, value, ttl, :put, opts)\n\n      true\n    else\n      false\n    end\n    |> wrap_ok()\n  end\n\n  def put(adapter_meta, key, value, _ttl, _on_write, _opts) do\n    Agent.update(adapter_meta.pid, &Map.put(&1, key, value))\n\n    {:ok, true}\n  end\n\n  @impl Nebulex.Adapter.KV\n  def put_all(adapter_meta, entries, ttl, op, opts)\n\n  def put_all(adapter_meta, entries, ttl, :put_new, opts) do\n    if get_all(adapter_meta, Map.keys(entries), []) == %{} do\n      put_all(adapter_meta, entries, ttl, :put, opts)\n\n      true\n    else\n      false\n    end\n    |> wrap_ok()\n  end\n\n  def put_all(adapter_meta, entries, _ttl, _on_write, _opts) do\n    entries = Map.new(entries)\n\n    Agent.update(adapter_meta.pid, &Map.merge(&1, entries))\n\n    {:ok, true}\n  end\n\n  @impl Nebulex.Adapter.KV\n  def delete(adapter_meta, key, _opts) do\n    wrap_ok Agent.update(adapter_meta.pid, &Map.delete(&1, key))\n  end\n\n  @impl Nebulex.Adapter.KV\n  def take(adapter_meta, key, _opts) do\n    value = get(adapter_meta, key, [])\n\n    delete(adapter_meta, key, [])\n\n    {:ok, value}\n  end\n\n  @impl Nebulex.Adapter.KV\n  def update_counter(adapter_meta, key, amount, _ttl, default, _opts) do\n    Agent.update(adapter_meta.pid, fn state ->\n      Map.update(state, key, default + amount, fn v -> v + amount end)\n    end)\n\n    wrap_ok get(adapter_meta, key, [])\n  end\n\n  @impl Nebulex.Adapter.KV\n  def has_key?(adapter_meta, key, _opts) do\n    wrap_ok Agent.get(adapter_meta.pid, &Map.has_key?(&1, key))\n  end\n\n  @impl Nebulex.Adapter.KV\n  def ttl(_adapter_meta, _key, _opts) do\n    {:ok, nil}\n  end\n\n  @impl Nebulex.Adapter.KV\n  def expire(_adapter_meta, _key, _ttl, _opts) do\n    {:ok, true}\n  end\n\n  @impl Nebulex.Adapter.KV\n  def touch(_adapter_meta, _key, _opts) do\n    {:ok, true}\n  end\n\n  @impl Nebulex.Adapter.Queryable\n  def execute(adapter_meta, query_meta, _opts) do\n    do_execute(adapter_meta.pid, query_meta)\n  end\n\n  def do_execute(pid, %{op: :delete_all} = query_meta) do\n    deleted = do_execute(pid, %{query_meta | op: :count_all})\n\n    Agent.update(adapter_meta.pid, fn _state -> %{} end)\n\n    {:ok, deleted}\n  end\n\n  def do_execute(pid, %{op: :count_all}) do\n    wrap_ok Agent.get(pid, &map_size/1)\n  end\n\n  def do_execute(pid, %{op: :get_all, query: {:q, nil}}) do\n    wrap_ok Agent.get(pid, &Map.values/1)\n  end\n\n  # Fetching multiple keys\n  def do_execute(pid, %{op: :get_all, query: {:in, keys}}) do\n    pid\n    |> Agent.get(&Map.take(&1, keys))\n    |> Map.to_list()\n    |> wrap_ok()\n  end\n\n  @impl Nebulex.Adapter.Queryable\n  def stream(adapter_meta, query_meta, _opts) do\n    do_stream(adapter_meta.pid, query_meta)\n  end\n\n  def stream(pid, %{query: {:q, q}, select: select}) when q in [nil, :all] do\n    fun =\n      case Keyword.get(opts, :select) do\n        :value ->\n          &Map.values/1\n\n        {:key, :value} ->\n          &Map.to_list/1\n\n        _ ->\n          &Map.keys/1\n      end\n\n    wrap_ok Agent.get(pid, fun)\n  end\n\n  def stream(_pid_, query) do\n    wrap_error Nebulex.QueryError, query: query\n  end\nend\n```\n\nOf course, this isn't a useful adapter in any sense but it should be enough to\nget you started with your own.","ref":"creating-new-adapter.html#implementation"},{"type":"extras","title":"Upgrading to v3.0","doc":"# Upgrading to v3.0\n\nFor the v3, Nebulex introduces several breaking changes, including the Cache\nAPI itself. This guide aims to highlight most of these changes to make easier\nthe transition to v3. Be aware this guide won't focus on every change, just\nthe most significant ones that can affect how your application code interacts\nwith the cache. Also, it is not a detailed guide about how to translate the\ncurrent code from older versions to v3, just pointing out the areas the new\ndocumentation should be consulted on.","ref":"v3-0.html"},{"type":"extras","title":"Built-In Adapters - Upgrading to v3.0","doc":"All previously built-in adapters (`Nebulex.Adapters.Local`,\n`Nebulex.Adapters.Partitioned`, `Nebulex.Adapters.Replicated`, and\n`Nebulex.Adapters.Multilevel`) have been moved to separate repositories.\nTherefore, you must add the adapter dependency to the list of dependencies\nin your `mix.exs` file.\n\nFor example, if you are using the local adapter:\n\n```diff\ndefp deps do\n  [\n    {:nebulex, \"~> 3.0\"},\n+   {:nebulex_local, \"~> 3.0\"},\n    ...\n  ]\nend\n```","ref":"v3-0.html#built-in-adapters"},{"type":"extras","title":"Update Cache API calls - Upgrading to v3.0","doc":"The most significant change is on the [Cache API][cache_api]. Nebulex v3 has a\nnew API based on ok/error tuples.\n\nNebulex v3 brings a new API with two flavors:\n\n* An ok/error tuple API for all cache functions. This new approach is preferred\n  when you want to handle different outcomes using pattern-matching.\n* An alternative API version with trailing bang (`!`) functions. This approach\n  is preferred if you expect the outcome to always be successful.\n\n[cache_api]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Cache.html\n\nTherefore, there are two ways to address the API changes. The first is to review\nyour code's cache calls and handle the new ok/error tuple response. The second\nis to replace your cache calls using the function version with the trailing bang\n(`!`). But let's take a closer look at them.","ref":"v3-0.html#update-cache-api-calls"},{"type":"extras","title":"Using ok/error tuple API - Upgrading to v3.0","doc":"As mentioned above, all the Cache API callbacks can return `:ok` or\n`{:ok, result}` on success, or `{:error, reason}` otherwise. Hence, you have\nthree options:\n\n  - Ignore the result. However, there will be some cases where you don't\n    want (or can't) ignore the result. For example, when fetching a key,\n    it is intended to use its value, hence, it doesn't make sense to\n    ignore the result.\n  - Pattern-match the success case. If you decide to take this path, perhaps\n    you should consider using the trailing bang (`!`) functions mentioned in\n    the next section.\n  - Handle the success and error cases explicitly (the preferred option).\n\nIf you go for the first option of ignoring the result, you can replace your\ncache calls like this::\n\n```diff\n- :ok = MyApp.Cache.put(\"key\", \"value\")\n+ _ignore = MyApp.Cache.put(\"key\", \"value\")\n```\n\nOn the other hand, if you go for option two to pattern-match the success case:\n\n```diff\n- value = MyApp.Cache.get(\"key\")\n+ {:ok, value} =  MyApp.Cache.get(\"key\")\n```\n\nFinally, if you go for the third option, update the cache calls to handle the\nsuccess and error cases:\n\n```diff\n- :ok = MyApp.Cache.put(\"key\", \"value\")\n- value = MyApp.Cache.get(\"key\")\n```\n\nNew code may look like this:\n\n```elixir\ncase MyApp.Cache.put(\"key\", \"value\") do\n  :ok ->\n    #=> your logic handling success\n\n  {:error, reason} ->\n    #=> your logic handling the error\nend\n\ncase MyApp.Cache.fetch(\"key\") do\n  {:ok, value} ->\n    #=> your logic handling success\n\n  {:error, %Nebulex.KeyError{}} ->\n    #=> your logic handling when the key is not found\n\n  {:error, reason} ->\n    #=> your logic handling other errors\nend\n```\n\nNotice that the `fetch` function was introduced since v3. However, the `get`\nfunction is still available, but it has slightly different semantics.\nSee the Cache API docs for more information.\n\n> #### Migrating to ok/error tuple API {: .tip}\n>\n> You can apply the same idea in the examples above to migrate ALL the Cache API\n> calls.","ref":"v3-0.html#using-ok-error-tuple-api"},{"type":"extras","title":"Using API version with trailing bang (`!`) functions - Upgrading to v3.0","doc":"The second way to address the API changes (and perhaps the easiest one) is to\nreplace your cache calls by using the function version with the trailing bang\n(`!`). For example:\n\n```diff\n- :ok = MyApp.Cache.put(\"key\", \"value\")\n+ :ok = MyApp.Cache.put!(\"key\", \"value\")\n```\n\nDespite this fix of using bang functions (`!`) may work for most cases, there\nmay be a few where the outcome is not the same or the patch is just not\napplicable. See the next sections to learn more about it.","ref":"v3-0.html#using-api-version-with-trailing-bang-functions"},{"type":"extras","title":"Update `get` calls - Upgrading to v3.0","doc":"The previous callback `get/2` has changed the semantics a bit (aside from the\nok/error tuple API). Previously, returned `nil` when the given key wasn't in\nthe cache. Now, the callback accepts an argument to specify the default value\nwhen the key is not found (defaults to `nil`).\n\nThe easiest and quickest way to update the `get` calls is using the new version\nof it with the trailing bang, like so:\n\n```diff\n- value = MyApp.Cache.get(\"key\")\n+ value = MyApp.Cache.get!(\"key\")\n```\n\nOther alternatives:\n\n```elixir\n# Using the default\nvalue = MyApp.Cache.get!(\"key\", \"default\")\n\n# Handling the response\ncase MyApp.Cache.get(\"key\", \"default\") do\n  {:ok, \"default\"} ->\n    #=> your logic handling success with default value\n\n  {:ok, value} ->\n    #=> your logic handling success\n\n  {:error, reason} ->\n    #=> your logic handling other errors\nend\n```","ref":"v3-0.html#update-get-calls"},{"type":"extras","title":"Update `has_key?` calls - Upgrading to v3.0","doc":"The new API does not have a \"trailing bang function (`!`)\" version for the\n`has_key?` callback. Therefore, you must work with the ok/error tuple API.\nFor example:\n\n```diff\n- bool = MyApp.Cache.has_key?(\"key\")\n+ {:ok, bool} = MyApp.Cache.has_key?(\"key\")\n```","ref":"v3-0.html#update-has_key-calls"},{"type":"extras","title":"Update Transaction API calls - Upgrading to v3.0","doc":"The new API does not have a \"trailing bang function (`!`)\" version for the\nTransaction API callbacks. Therefore, you must work with the ok/error tuple API.\nFor example:\n\n```diff\n- bool = MyApp.Cache.in_transaction?()\n+ {:ok, bool} = MyApp.Cache.in_transaction?()\n\n- value = MyApp.Cache.transaction(fn -> MyApp.Cache.get(\"key\") end)\n+ {:ok, value} = MyApp.Cache.transaction(fn -> MyApp.Cache.get!(\"key\") end)\n```","ref":"v3-0.html#update-transaction-api-calls"},{"type":"extras","title":"Update `flush` calls - Upgrading to v3.0","doc":"The callback `flush` is deprecated, you should use `delete_all` instead:\n\n```diff\n- MyApp.Cache.flush()\n+ MyApp.Cache.delete_all()\n```","ref":"v3-0.html#update-flush-calls"},{"type":"extras","title":"Storing `nil` values - Upgrading to v3.0","doc":"Previously, Nebulex used to skip storing `nil` values in the cache. The main\nreason was the semantics assumed for the `nil` value, being used to validate\nwhether a key existed in the cache or not. However, this could be a limitation.\n\nSince Nebulex v3, any Elixir term can be stored in the cache (including `nil`),\nNebulex doesn't perform any validation whatsoever. Any meaning or semantics\nbehind `nil` (or any other term) is up to the user.\n\nAdditionally, a new callback `fetch/2` is introduced, which is the base or main\nfunction for retrieving a key from the cache; in fact, the `get` callback is\nimplemented using `fetch` underneath.","ref":"v3-0.html#storing-nil-values"},{"type":"extras","title":"Deprecated Stats API - Upgrading to v3.0","doc":"The `Nebulex.Adapter.Stats` behaviour has been deprecated. Therefore, the\ncallbacks `stats/0` and `dispatch_stats/1` are no longer available and must\nbe removed from your code.\n\nAs a quick fix, you can update the stats calls using the new Info API,\nlike this:\n\n```diff\n- :ok = MyApp.Cache.stats()\n+ stats = MyApp.Cache.info!(:stats)\n```\n\nSince Nebulex v3, the adapter's Info API is introduced. This is a more generic\nAPI to get information about the cache, including the stats. Adapters are\nresponsible for implementing the Info API and are also free to add the\ninformation specification keys they want. See\n[c:Nebulex.Cache.info/2][info_cb] and the [\"Cache Info Guide\"][cache_info_guide]\nfor more information.\n\n[info_cb]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Cache.html#c:info/2\n[cache_info_guide]: http://hexdocs.pm/nebulex/3.0.0-rc.1/info-api.html","ref":"v3-0.html#deprecated-stats-api"},{"type":"extras","title":"Deprecated Persistence API - Upgrading to v3.0","doc":"Persistence can be implemented in many different ways depending on the use case.\nA persistence API with a specific implementation may not be very useful because\nit won't cover all possible use cases. For example, if your application is on\nAWS, perhaps it makes more sense to use S3 as persistent storage, not the local\nfile system. Since Nebulex v3, persistence is an add-on that can be provided by\nthe backend (e.g., Redis), another library, or the application itself. And it\nshould be configured via the adapter's configuration (or maybe directly with the\nbackend).\n\nTherefore, the `Nebulex.Adapter.Persistence` behaviour has been deprecated, so\nthe callbacks `dump/2` and `load/2` are no longer available and must be removed\nfrom your code.\n\n```diff\n- MyApp.Cache.dump(\"my_backup\")\n- MyApp.Cache.load(\"my_backup\")\n```","ref":"v3-0.html#deprecated-persistence-api"},{"type":"extras","title":"Update Query API calls - Upgrading to v3.0","doc":"The old Query API received as an argument the query to run against the adapter\nor backend, which could be any Elixir term (based on the adapter). Now, the\nQuery API expects a [\"Query Spec\"][q_spec] as an argument to work.\n\n[q_spec]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Cache.html#c:get_all/2-query-specification","ref":"v3-0.html#update-query-api-calls"},{"type":"extras","title":"Updating `all` calls - Upgrading to v3.0","doc":"The callback `all/2` was merged into `get_all/2`; only the latter can be used\nnow. The new version of `get_all/2` accepts a query-spec as a first argument.\n\nInstead of using `all`, you'll call `get_all`:\n\n```diff\n- results = MyApp.Cache.all()\n+ results = MyApp.Cache.get_all!()\n```\n\nPassing a query as an argument:\n\n```diff\n- results = MyApp.Cache.all(my_query)\n+ results = MyApp.Cache.get_all!(query: my_query)\n```\n\nBesides, the option `:return` is deprecated. You may consider using the\n`:select` option of the query-spec. For example:\n\n```diff\n- results = MyApp.Cache.all(nil, return: :value)\n+ results = MyApp.Cache.get_all!(select: :value)\n```\n\n> #### `stream` function {: .info}\n>\n> The same changes and examples apply to the `stream` function.","ref":"v3-0.html#updating-all-calls"},{"type":"extras","title":"Updating `get_all` calls - Upgrading to v3.0","doc":"Similarly, you must update the `get_all` calls:\n\n```diff\n- results = MyApp.Cache.get_all([k1, k2, ...])\n+ results = MyApp.Cache.get_all!(in: [k1, k2, ...])\n```","ref":"v3-0.html#updating-get_all-calls"},{"type":"extras","title":"Updating `count_all` and `delete_all` calls - Upgrading to v3.0","doc":"Similarly to `get_all`, you must use a query-spec as the first argument when\ncalling `count_all` or `delete_all`.\n\n```diff\n- count = MyApp.Cache.count_all()\n+ count = MyApp.Cache.count_all!()\n\n- count = MyApp.Cache.delete_all()\n+ count = MyApp.Cache.delete_all!()\n\n- count = MyApp.Cache.delete_all(my_query)\n+ count = MyApp.Cache.delete_all!(query: my_query)\n```\n\nSee [Cache API Docs][cache_api] for more information and examples.","ref":"v3-0.html#updating-count_all-and-delete_all-calls"},{"type":"extras","title":"Update caching decorators - Upgrading to v3.0","doc":"Nebulex v3 introduces some changes and new features to the Declarative Caching\nAPI (a.k.a caching decorators). We will highlight mostly the changes and perhaps\na few new features. However, it is highly recommended you check the\ndocumentation for more information about all the new features and changes.","ref":"v3-0.html#update-caching-decorators"},{"type":"extras","title":"Update `:cache` option - Upgrading to v3.0","doc":"The `:cache` option doesn't support MFA tuples anymore. The possible values are\na cache module, a dynamic cache spec, or an anonymous function that optionally\nreceives the decorator's context as an argument and must return the cache to use\n(a cache module or a dynamic cache spec).\n\n```diff\n- @decorate cacheable(cache: {MyApp.Caching, :get_cache, []})\n+ @decorate cacheable(cache: &MyApp.Caching.get_cache/1)\n```\n\nOptionally, if you don't need the decorator context to compute the cache,\nuse a 0-arity function instead:\n\n```diff\n- @decorate cacheable(cache: {MyApp.Caching, :get_cache, []})\n+ @decorate cacheable(cache: &MyApp.Caching.get_cache/0)\n```\n\nThe `get_cache` function may look like this:\n\n```elixir\ndefmodule MyApp.Caching do\n  alias Nebulex.Caching.Decorators.Context\n\n  def get_cache(%Context{} = context) do\n    #=> return the cache\n  end\n\n  # Without the context\n  def get_cache do\n    #=> return the cache\n  end\nend\n```","ref":"v3-0.html#update-cache-option"},{"type":"extras","title":"Update `:keys` option - Upgrading to v3.0","doc":"The option `:keys` is deprecated. Instead, consider using the option `:key`\nlike this:\n\n```diff\n- @decorate cacheable(cache: MyApp.Cache, keys: [\"foo\", \"bar\"])\n+ @decorate cacheable(cache: MyApp.Cache, key: {:in, [\"foo\", \"bar\"]})\n```","ref":"v3-0.html#update-keys-option"},{"type":"extras","title":"Update `:key_generator` option - Upgrading to v3.0","doc":"The `:key_generator` option is deprecated. Instead, you can use the `:key`\noption with an anonymous function that optionally receives the decorator's\ncontext as an argument and must return the key to use.\n\n```diff\n- @decorate cacheable(cache: MyApp.Cache, key_generator: CustomKeyGenerator)\n+ @decorate cacheable(cache: MyApp.Cache, key: &MyApp.Caching.compute_key/1)\n```\n\nOptionally, if you don't need the decorator context to compute the key,\nuse a 0-arity function instead:\n\n```diff\n- @decorate cacheable(cache: MyApp.Cache, key_generator: CustomKeyGenerator)\n+ @decorate cacheable(cache: MyApp.Cache, key: &MyApp.Caching.compute_key/0)\n```\n\nThe `compute_key` function may look like this:\n\n```elixir\ndefmodule MyApp.Caching do\n  alias Nebulex.Caching.Decorators.Context\n\n  def compute_key(%Context{} = context) do\n    #=> return the key\n  end\n\n  # Without the context\n  def compute_key do\n    #=> return the key\n  end\nend\n```","ref":"v3-0.html#update-key_generator-option"},{"type":"extras","title":"Update `:default_key_generator` option - Upgrading to v3.0","doc":"The `Nebulex.Caching.KeyGenerator` behaviour is deprecated. You can use\nan anonymous function for the `:default_key_generator` option instead\n(the function must be provided in the format `&Mod.fun/arity`). Besides,\nthe `:default_key_generator` option must be provided to `use Nebulex.Caching`.\n\nFirst, remove `:default_key_generator` option from all places it is used:\n\n```diff\ndefmodule MyApp.Cache do\n  use Nebulex.Cache,\n    otp_app: :my_app,\n    adapter: Nebulex.Adapters.Local,\n-   default_key_generator: __MODULE__\n\n  ...\nend\n```\n\nThen, add it to the modules using `use Nebulex.Caching`:\n\n```diff\ndefmodule MyApp.Books do\n  use Nebulex.Caching,\n+   default_key_generator: &MyApp.Keygen.generate/1\n\n  ...\nend\n```","ref":"v3-0.html#update-default_key_generator-option"},{"type":"extras","title":"Global options - Upgrading to v3.0","doc":"Decorators may become verbose sometimes, since you have to provide options like\nthe `:cache` on each decorator definition. Therefore, Nebulex supports adding\nsome of the common options globally when using `use Nebulex.Caching`.\n\nYou can update your modules using declarative caching like this:\n\n```diff\ndefmodule MyApp.Books do\n  use Nebulex.Caching,\n+   cache: MyApp.Cache\n+   match: &__MODULE__.match/1\n+   on_error: :raise\n\n- @decorate cacheable(cache: MyApp.Cache, key: id, match: &match/1, on_error: :raise)\n+ @decorate cacheable(key: id)\n  def get_book(id) do\n    # ... logic to retrieve a book\n  end\n\n- @decorate cache_put(cache: MyApp.Cache, key: book.id, match: &match/1, on_error: :raise)\n+ @decorate cache_put(key: book.id)\n  def update_book(book, update_attrs) do\n    # ... logic to update a book\n  end\n\n- @decorate cache_evict(cache: MyApp.Cache, key: book.id, on_error: :raise)\n+ @decorate cache_evict(key: book.id)\n  def delete_book(book) do\n    # ... logic to delete a book\n  end\n\n  ...\nend\n```\n\nSee [`Nebulex.Caching` options][nbx_caching] for more information.\n\n[nbx_caching]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Caching.html#module-compilation-time-options\n\n### `:references` option and dynamic caches\n\nThe option `:references` in the `cacheable` decorator supports referencing\na dynamic cache. For example:\n\n```elixir\ndefmodule MyApp.Books do\n  use Nebulex.Caching\n\n  @decorate cacheable(cache: dynamic_cache(MyApp.Cache, :books_cache))\n  def find_book(isbn) do\n    # your logic ...\n  end\nend\n```\n\n> **_See [\"Caching Decorators\"][caching_decorators] for more information._**\n\n[caching_decorators]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Caching.Decorators.html","ref":"v3-0.html#global-options"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}